<?php

/**
 * Seo management
 *
 * @package     PG_Core
 * @subpackage  Helpers
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('seo_tags')) {

    /**
     * Display page seo tags
     *
     * Tags value seprated by '|'
     *
     * @param string $tags tags names
     *
     * @return void
     */
    function seo_tags($tags = 'title')
    {
        $CI = &get_instance();

        $controller = $CI->router->fetch_class(true);
        if (substr($controller, 0, 6) === 'admin_') {
            $module_gid = strtolower(substr($controller, 6));
            $controller = 'admin';
        } else {
            $module_gid = strtolower($controller);
            $controller = 'user';
        }
        if (empty($module_gid)) {
            $module_gid = 'start';
        }

        $method = $CI->router->fetch_method();
        if (empty($method)) {
            $method = 'index';
        }

        $html = $CI->pg_seo->session_seo_tags_html($controller, $module_gid, $method);

        $return = '';
        $tags_array = explode('|', $tags);
        if (!empty($tags_array)) {
            foreach ($tags_array as $tag) {
                $tag = trim(strtolower($tag));
                if (!empty($html[$tag])) {
                    $return .= $html[$tag];
                }
            }
        }

        echo $return;
    }
}

if (!function_exists('seo_tags_default')) {

    /**
     * Display page seo tags by default
     *
     * Tags value seprated by '|'
     *
     * @param string $tags tags names
     *
     * @return void
     */
    function seo_tags_default($tags = 'title')
    {
        $CI = &get_instance();
        $html = $CI->pg_seo->session_seo_tags_html('user', 'start', 'index');
        $return = '';
        $tags_array = explode("|", $tags);
        if (!empty($tags_array)) {
            foreach ($tags_array as $tag) {
                $tag = trim(strtolower($tag));
                if (!empty($html[$tag])) {
                    $return .= $html[$tag];
                }
            }
        }
        echo $return;
    }
}

if (!function_exists('rewrite_link')) {

    /**
     * Generate seo link
     *
     * @param string  $module         module name
     * @param string  $method         method name
     * @param array   $data           method data
     * @param boolean $is_admin       admin mode
     * @param string  $lang_code      language code
     * @param boolean $no_lang_in_url no usage language code in url
     *
     * @return string
     */
    function rewrite_link($module, $method, $data = array(), $is_admin = false, $lang_code = null, $no_lang_in_url = false)
    {
        $ci = &get_instance();
        return $ci->pg_seo->create_url($module, $method, $data, $is_admin, $lang_code, $no_lang_in_url);
    }
}

if ( ! function_exists('seolink')) {
    function seolink($module, $method, array $data = array(), array $additional = array())
    {
        return rewrite_link($module, $method, array_merge($data, $additional));
    }
}
