<?php

namespace Pg\Modules\Banners\Models;

use Pg\Libraries\Setup;

/**
 * Banners install model
 *
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Banners_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;
    protected $modules_data;

    /**
     * Class constructor
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->modules_data = Setup::getModuleData('banners', Setup::TYPE_MODULES_DATA);
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]['id'] = linked_install_set_menu($gid, $menu_data['action'], $menu_data['name']);
            linked_install_process_menu_items($this->modules_data['menu'], 'create', $gid, 0, $this->modules_data['menu'][$gid]['items']);
        }
        if (!empty($this->menu_indicators)) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->menu_indicators as $data) {
                $this->ci->Indicators_model->save_type(null, $data);
            }
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('banners', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');
            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu'], 'update', $gid, 0, $this->modules_data['menu'][$gid]['items'], $gid, $langs_file);
        }

        if (!empty($this->menu_indicators)) {
            $langs_file = $this->ci->Install_model->language_file_read('moderation', 'indicators', $langs_ids);
            if (!$langs_file) {
                log_message('info', '(resumes) Empty indicators langs data');
                return false;
            } else {
                $this->ci->load->model('menu/models/Indicators_model');
                $this->ci->Indicators_model->update_langs($this->menu_indicators, $langs_file, $langs_ids);
            }
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], 'export', $gid, 0, $this->modules_data['menu'][$gid]['items'], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }
        if (!empty($this->menu_indicators)) {
            $this->ci->load->model('menu/models/Indicators_model');
            $indicators_langs = $this->ci->Indicators_model->export_langs($this->menu_indicators, $langs_ids);
        }

        return array('menu' => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->modules_data['menu'][$gid]['items']);
            }
        }
        if (!empty($this->menu_indicators)) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->menu_indicators as $data) {
                $this->ci->Indicators_model->delete_type($data['gid']);
            }
        }
    }

    /**
     * Install data of uploads module
     *
     * @return void
     */
    public function install_uploads()
    {
        $this->ci->load->model('uploads/models/Uploads_config_model');
        $config_data = array(
            'gid' => 'banner',
            'name' => 'Banner image file',
            'max_height' => 800,
            'max_width' => 800,
            'max_size' => 100000,
            'name_format' => 'generate',
            'file_formats' => 'a:3:{i:0;s:3:"jpg";i:1;s:3:"gif";i:2;s:3:"png";}',
            'default_img' => '',
            'date_add' => date('Y-m-d H:i:s'),
        );
        $this->ci->Uploads_config_model->save_config(null, $config_data);
    }

    /**
     * Uninstall data of uploads module
     *
     * @return void
     */
    public function deinstall_uploads()
    {
        $this->ci->load->model('uploads/models/Uploads_config_model');
        $config_data = $this->ci->Uploads_config_model->get_config_by_gid('banner');
        if (!empty($config_data["id"])) {
            $this->ci->Uploads_config_model->delete_config($config_data["id"]);
        }
    }

    /**
     * Install data of service module
     *
     * @return void
     */
    public function install_services()
    {
        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $validate_data = $this->ci->Services_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Services_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $validate_data = $this->ci->Services_model->validate_service(null, $service_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Services_model->save_service(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_services_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("banners", "services", $langs_ids);
        if (!empty($langs_file)) {
            $services_data = array(
                "service" => array(),
                "template" => array(),
                "param" => array(),
            );

            $this->ci->load->model("Services_model");

            foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
                $data_admin = isset($template_data["data_admin"]) ? array_keys($template_data["data_admin"]) : array();
                $data_user = isset($template_data["data_user"]) ? array_keys($template_data["data_user"]) : array();
                $data_membership = isset($template_data["data_membership"]) ? array_keys($template_data["data_membership"]) : array();
                $services_data["template"][] = $template_data["gid"];
                $services_data["param"][$template_data["gid"]] = array_unique(array_merge($data_admin, $data_user, $data_membership));
            }

            foreach ((array) $this->modules_data['services']["services"] as $service_data) {
                $services_data["service"][] = $service_data["gid"];
            }

            $this->ci->Services_model->update_langs($services_data, $langs_file);
        } else {
            log_message("info", "Empty banners services langs data");
        }

        return true;
    }

    /**
     * Export languages of services modules
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_services_lang_export($langs_ids = null)
    {
        $services_data = array(
            "service" => array(),
            "template" => array(),
            "param" => array(),
        );

        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $data_admin = isset($template_data["data_admin"]) ? array_keys($template_data["data_admin"]) : array();
            $data_user = isset($template_data["data_user"]) ? array_keys($template_data["data_user"]) : array();
            $services_data["template"][] = $template_data["gid"];
            $services_data["param"][$template_data["gid"]] = array_merge($data_admin, $data_user);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $services_data["service"][] = $service_data["gid"];
        }

        $services = $this->ci->Services_model->export_langs($services_data, $langs_ids);

        return array("services" => $services);
    }

    /**
     * Uninstall data of services modules
     *
     * @return void
     */
    public function deinstall_services()
    {
        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $this->ci->Services_model->delete_template_by_gid($template_data["gid"]);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $this->ci->Services_model->delete_service_by_gid($service_data["gid"]);
        }
    }

    /**
     * Install data of service & notification
     *
     * @return void
     */
    public function install_services_notifications()
    {
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        $templates_ids = array();

        foreach ((array) $this->modules_data['services_notifications']["templates"] as $template_data) {
            if (is_array($template_data["vars"])) {
                $template_data["vars"] = implode(", ", $template_data["vars"]);
            }
            $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $templates_ids[$template_data['gid']] = $this->ci->Templates_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->modules_data['services_notifications']["notifications"] as $notification_data) {
            if (!isset($templates_ids[$notification_data["template"]])) {
                $template = $this->ci->Templates_model->get_template_by_gid($notification_data["template"]);
                $templates_ids[$notification_data["template"]] = $template["id"];
            }
            $notification_data["id_template_default"] = $templates_ids[$notification_data["template"]];
            $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Notifications_model->save_notification(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_services_notifications_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("banners", "services_notifications", $langs_ids);
        if (!empty($langs_file)) {
            $this->ci->load->model("Notifications_model");
            $this->ci->Notifications_model->update_langs($this->modules_data['services_notifications'], $langs_file, $langs_ids);
        } else {
            log_message("info", "Empty notifications langs data");
        }

        return true;
    }

    /**
     * Export languages of services & notifications
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_services_notifications_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Notifications_model");
        $notifications = $this->ci->Notifications_model->export_langs($this->modules_data['services_notifications'], $langs_ids);

        return array("notifications" => $notifications);
    }

    /**
     * Uninstall data of services & notifications
     *
     * @return void
     */
    public function deinstall_services_notifications()
    {
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach ((array) $this->modules_data['services_notifications']["notifications"] as $notification_data) {
            $this->ci->Notifications_model->delete_notification_by_gid($notification_data["gid"]);
        }

        foreach ((array) $this->modules_data['services_notifications']["templates"] as $template_data) {
            $this->ci->Templates_model->delete_template_by_gid($template_data["gid"]);
        }
    }

    /**
     * Install data of cronjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array(
            "name" => "Update banner statistic",
            "module" => "banners",
            "model" => "Banners_model",
            "method" => "update_statistic",
            "cron_tab" => "*/10 * * * *",
            "status" => "1",
        );
        $this->ci->Cronjob_model->save_cron(null, $cron_data);
    }

    /**
     * Uninstall data of cronjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array();
        $cron_data["where"]["module"] = "banners";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        $this->ci->load->model('Moderators_model');

        foreach ($this->modules_data['moderators_methods'] as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Import languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return void
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('banners', 'moderators', $langs_ids);

        // install moderators permissions
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'banners';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method['id'], array(), $langs_file[$method['method']]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'banners';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'banners';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of users & service & menu
     *
     * @return void
     */
    public function install_users_services_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_services_menu'] as $gid => $menu_data) {
            $this->modules_data['users_private_services_menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['users_services_menu'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import users languages to users & services & menu
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_users_services_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("banners", "users_services_menu", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty menu langs data for users services menu (banners)");
            return false;
        }

        $this->ci->load->helper("menu");
        foreach ($this->modules_data['users_services_menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['users_services_menu'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
        }
        return true;
    }

    /**
     * Export users languages from users & services & menu
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_users_services_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $users_services_menu = array();
        foreach ($this->modules_data['users_services_menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['users_services_menu'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $users_services_menu = array_merge($users_services_menu, $temp);
        }

        return array("users_services_menu" => $return);
    }

    /**
     * Uninstall data of users & services & menu
     *
     * @return void
     */
    public function deinstall_users_services_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_services_menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }
    }

    /**
     * Install users data to payments & services & menu
     *
     * @return void
     */
    public function install_users_private_services_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_private_services_menu'] as $gid => $menu_data) {
            $this->modules_data['users_private_services_menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['users_private_services_menu'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import users languages to users_private & services & menu
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_users_private_services_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("banners", "users_private_services_menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data for users_private_services_menu (banners)");
            return false;
        }

        $this->ci->load->helper("menu");
        foreach ($this->modules_data['users_private_services_menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['users_private_services_menu'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
        }
        return true;
    }

    /**
     * Export users languages from users_private & services & menu
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_users_private_services_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $users_private_services_menu = array();
        foreach ($this->modules_data['users_private_services_menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['users_private_services_menu'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $users_private_services_menu = array_merge($users_private_services_menu, $temp);
        }

        return array("users_private_services_menu" => $users_private_services_menu);
    }

    /**
     * Uninstall users data from users_private & services & menu module
     *
     * @return void
     */
    public function deinstall_users_private_services_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['users_private_services_menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }
    }

    public function install_content()
    {
        $this->ci->load->model('Content_model');
        $this->ci->Content_model->installPages($this->modules_data['content_pages']);
    }

    public function deinstall_content()
    {
        $this->ci->load->model('Content_model');
        foreach ($this->modules_data['content_pages'] as $page) {
            $this->ci->Content_model->delete_page($page['gid']);
        }
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        if ($this->pg_module->is_module_active('moderators')) {
            $this->ci->load->model("Moderators_model");
            $users = $this->ci->Moderators_model->get_users_list(null, 1, array('id' => 'DESC'), array('where' => array('user_type' => 'admin')));
            if (!empty($users)) {
                $this->ci->pg_module->set_module_config("banners", 'admin_moderation_emails', $users[0]["email"]);
            }
        }

        // Add demo banners
        $this->add_banners_to_site();
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        
    }

    /**
     * Add demo banners
     *
     * @return void
     */
    public function add_banners_to_site()
    {
        $this->ci->load->model('banners/models/Banner_group_model');
        $this->ci->load->model('Banners_model');

        $all_groups_ids = array_keys($this->ci->Banner_group_model->get_all_groups_key_id());

        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => '55286cb3e5.jpg',
            'banner_place_id' => 3,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 185*75 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:185px;height:75px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="5379732746"></ins>
				<script>
					(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://demo.realtysoft.pro/',
            'name' => 'Left banner №1',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );

        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => 'fcae62cd46.jpg',
            'banner_place_id' => 3,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 185*75 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:185px;height:75px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="5379732746"></ins>
				<script>
					(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://demo.realtysoft.pro/',
            'name' => 'Left banner №2',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );

        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => '93497f79ce.jpg',
            'banner_place_id' => 3,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 185*75 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:185px;height:75px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="5379732746"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://www.realtysoft.pro/pricing.php',
            'name' => 'Left banner №3',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );

        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => 'dfd669eba7.JPG',
            'banner_place_id' => 1,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 728*90 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:728px;height:90px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="2426266344"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://demo.realtysoft.pro/',
            'name' => 'Bottom banner №1',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );
        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => 'e9116ceea8.jpg',
            'banner_place_id' => 1,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 728*90 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:728px;height:90px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="2426266344"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://demo.realtysoft.pro/',
            'name' => 'Bottom banner №2',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );
        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => '58dc682f81.jpg',
            'banner_place_id' => 1,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 728*90 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:728px;height:90px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="2426266344"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://www.realtysoft.pro/pricing.php',
            'name' => 'Bottom banner №3',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );
        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => '537f411419.jpg',
            'banner_place_id' => 2,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 185*155 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:185px;height:155px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="8472799943"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://demo.realtysoft.pro/',
            'name' => 'Big left banner №1',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );
        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => 'b4ab5556ad.jpg',
            'banner_place_id' => 2,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 185*155 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:185px;height:155px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="8472799943"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://demo.realtysoft.pro/',
            'name' => 'Big left banner №2',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );
        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => 'c900b924eb.jpg',
            'banner_place_id' => 2,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE 185*155 -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:185px;height:155px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="8472799943"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://www.realtysoft.pro/pricing.php',
            'name' => 'Big left banner №3',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );
        $banners[] = array(
            'alt_text' => 'PG RealEstateSoftware',
            'approve' => 1,
            'banner_image' => '03041d2ac7.jpg',
            'banner_place_id' => 4,
            'banner_type' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? 2 : 1,
            'decline_reason' => '',
            'expiration_date' => '0000-00-00 00:00:00',
            'html' => ($_ENV['DEMO_MODE'] || $_ENV['TRIAL_MODE']) ? '
				<script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
				<!-- Demo RE -->
				<ins class="adsbygoogle"
					 style="display:inline-block;width:300px;height:250px"
					 data-ad-client="ca-pub-7776361119486928"
					 data-ad-slot="4042600349"></ins>
				<script>
				(adsbygoogle = window.adsbygoogle || []).push({});
				</script>
			' : '',
            'link' => 'http://www.realtysoft.pro/pricing.php',
            'name' => 'Right banner №1',
            'new_window' => 1,
            'is_admin' => 1,
            'number_of_clicks' => 0,
            'number_of_views' => 0,
            'status' => 1,
            'user_id' => 0,
            'stat_clicks' => 0,
            'stat_views' => 0,
            'user_activate_info' => '',
            'banner_groups' => $all_groups_ids,
        );

        foreach ($banners as $key => $banner) {
            $this->ci->Banners_model->save(null, $banner);
        }

        return true;
    }

}
