<?php

namespace Pg\Modules\Dynamic_block_map\Models;

use Pg\Modules\Listings\Models\Listings_model;

/**
 * Google map module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2015 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

class Dynamic_block_map_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;
    private $default_listings_count = 8;

    /**
     * Constructor
     * 
     * @return Google map block model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    private function determineListingType($view)
    {
        if (stripos($view, Listings_model::OPERATION_SALE) !== false) {
            $listings_type = Listings_model::OPERATION_SALE;
        } elseif (stripos($view, Listings_model::OPERATION_RENT) !== false) {
            $listings_type = Listings_model::OPERATION_RENT;
        } elseif (stripos($view, Listings_model::OPERATION_SOLD) !== false) {
            $listings_type = Listings_model::OPERATION_SOLD;
        } elseif (stripos($view, 'latest_added') !== false) {
            $listings_type = 'latest_added';
        }
        return $listings_type;
    }

    private function fetchFilters($view)
    {
        $listings_type = $this->determineListingType($view);

        $listings_filters = array();
        switch ($listings_type) {
            case 'sale':
            case 'rent':
            case 'sold':
                $listings_filters['type'] = $listings_type;
                break;
            case 'latest_added':
            default:
        }
        $listings_filters['status'] = 1;
        return $listings_filters;
    }

    public function _dynamic_block_get_google_map(array $params = array(), $view = 'sale_980_440', $width = 100)
    {
        if (!isset($params['count'])) {
            $params['count'] = $this->default_listings_count;
        }

        $this->ci->load->model(['geomap/models/Geomap_settings_model', 'Listings_model']);
        $view_settings = $this->ci->Geomap_settings_model->get_settings(
            'googlemapsv3', 0, 0, 'listing_dynamic_block'
        );
        $listings_filters = $this->fetchFilters($view);
        $name_segments = explode('_', $view);
        $size = array(
            'height' => array_pop($name_segments),
            'width' => array_pop($name_segments),
        );

        $f_lat = (float) $view_settings['lat'];
        $f_lon = (float) $view_settings['lon'];
        if (!empty($view_settings['markers_show_square']) && !empty($f_lat) && !empty($f_lon)) {
            $listings_filters['get_listings_by_radius'] = array(
                'radius' => $view_settings['markers_show_square'],
                'lat' => $view_settings['lat'],
                'lon' => $view_settings['lon'],
            );
        }

        $auto_identification = $this->ci->pg_module->get_module_config('country_identification', 'automatic_identification');
        $listings_filters['get_local_listings'] = $auto_identification && !empty($view_settings['use_geotargetting']);

        $this->ci->Listings_model->set_format_settings(['get_user', 'use_format'], false);
        $listings = $this->ci->Listings_model->get_listings_list($listings_filters, 1, $params['count']);
        $this->ci->Listings_model->set_format_settings(['get_user', 'use_format'], true);

        $markers = $this->getMarkers($listings, $view_settings);

        if (!empty($params['search_form'])) {
            $this->load->helper('start');
            $this->view->assign('map_search_form', main_search_form('sale', 'slider', true, 100, 'select'));
        }

        $this->ci->view->assign('markers', $markers);
        $this->ci->view->assign('map_settings', $view_settings);
        $this->ci->view->assign('params', $params);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('random_number', rand(1, 10000));
        $this->ci->view->assign('is_full_width_map', stripos($view, '1600_440') !== false);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);
        return $this->ci->view->fetch('dynamic_block_google_map', 'user', 'dynamic_block_map');
    }

    private function getMarkers($listings, $view_settings)
    {
        $markers_color = array(
            'sale' => get($view_settings['sale_marker_color']),
            'rent' => get($view_settings['rent_marker_color']),
            'sold' => get($view_settings['sold_marker_color']),
        );
        $markers = array();
        foreach ($listings as $listing) {
            if ((float) $listing['lat'] == 0 && (float) $listing['lon'] == 0) {
                continue;
            }

            // TODO: We should load this via ajax.
            $this->ci->view->assign('listing', $listing);
            $info = $this->ci->view->fetch('listing_map_block', 'user', 'dynamic_block_map');

            if (!empty($listing['sold'])) {
                $listing['operation_type'] = 'sold';
            }
            $markers[] = array(
                'gid' => $listing['id'],
                'country' => $listing['country'],
                'region' => $listing['region'],
                'city' => $listing['city'],
                'address' => $listing['address'],
                'postal_code' => $listing['zip'],
                'lat' => (float) $listing['lat'],
                'lon' => (float) $listing['lon'],
                'info' => $info,
                'marker_color' => $markers_color[$listing['operation_type']],
            );
        }
        return $markers;
    }
    
    public function getMapSettings()
    {
        $this->ci->load->model('geomap/models/Geomap_settings_model');
        $settings = $this->ci->Geomap_settings_model->get_settings(
            'googlemapsv3', 0, 0, 'listing_dynamic_block'
        );        
        
        return $settings;
    }

}
