<?php

/**
 * Export module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * Vast xml driver export model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Export
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 **/
class Export_vast_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Driver configuration
     *
     * @var array
     */
    public $drivers = array(
        "gid"      => "vast",
        "link"     => "http://www.vast.com/info/submitcontent",
        "settings" => array(),
        "elements" => array(
            array("name" => "record_id", "type" => "int", "required" => 0),
            array("name" => "title", "type" => "text", "required" => 0),
            array("name" => "url", "type" => "text", "required" => 0),
            array("name" => "category", "type" => "text", "required" => 0),
            array("name" => "subcategory", "type" => "text", "required" => 0),
            array("name" => "image", "type" => "url", "required" => 0),
            array("name" => "address", "type" => "text", "required" => 0),
            array("name" => "city", "type" => "text", "required" => 0),
            array("name" => "state", "type" => "text", "required" => 0),
            array("name" => "zip", "type" => "text", "required" => 0),
            array("name" => "country", "type" => "text", "required" => 0),
            array("name" => "bedrooms", "type" => "text", "required" => 0),
            array("name" => "bathrooms", "type" => "text", "required" => 0),
            array("name" => "square_footage", "type" => "text", "required" => 0),
            array("name" => "stories", "type" => "text", "required" => 0),
            array("name" => "lot_size", "type" => "text", "required" => 0),
            array("name" => "parking_spots", "type" => "text", "required" => 0),
            array("name" => "year_built", "type" => "text", "required" => 0),
            array("name" => "currency", "type" => "text", "required" => 0),
            array("name" => "price", "type" => "int", "required" => 0),
            array("name" => "amenities", "type" => "text", "required" => 0),
            array("name" => "description", "type" => "text", "required" => 0),
            array("name" => "listing_time", "type" => "text", "required" => 0),
            array("name" => "expire_time", "type" => "text", "required" => 0),
        ),
        "editable" => 0,
        "status"   => 1,
    );

    /**
     *  Demo data
     *
     *  @return void
     */
    public $demo_data = array(
        'gid_driver'      => 'vast',
        'id_object'       => '2',
        'module'          => 'listings',
        'name'            => 'Test sample',
        'relations'       => 'a:18:{i:0;a:4:{s:4:"name";s:9:"record_id";s:4:"type";s:3:"int";s:4:"link";s:2:"id";s:5:"label";s:14:"ID ref # (int)";}i:1;a:4:{s:4:"name";s:5:"title";s:4:"type";s:4:"text";s:4:"link";s:11:"output_name";s:5:"label";s:12:"Title (text)";}i:2;a:4:{s:4:"name";s:3:"url";s:4:"type";s:4:"text";s:4:"link";s:7:"seolink";s:5:"label";s:15:"View url (text)";}i:3;a:4:{s:4:"name";s:8:"category";s:4:"type";s:4:"text";s:4:"link";s:11:"id_category";s:5:"label";s:15:"Category (text)";}i:4;a:4:{s:4:"name";s:11:"subcategory";s:4:"type";s:4:"text";s:4:"link";s:13:"property_type";s:5:"label";s:20:"Property type (text)";}i:5;a:4:{s:4:"name";s:5:"image";s:4:"type";s:4:"text";s:4:"link";s:7:"uploads";s:5:"label";s:11:"Photo (url)";}i:6;a:4:{s:4:"name";s:7:"address";s:4:"type";s:4:"text";s:4:"link";s:7:"address";s:5:"label";s:14:"Address (text)";}i:7;a:4:{s:4:"name";s:4:"city";s:4:"type";s:4:"text";s:4:"link";s:7:"id_city";s:5:"label";s:11:"City (text)";}i:8;a:4:{s:4:"name";s:5:"state";s:4:"type";s:4:"text";s:4:"link";s:9:"id_region";s:5:"label";s:13:"Region (text)";}i:9;a:4:{s:4:"name";s:3:"zip";s:4:"type";s:4:"text";s:4:"link";s:3:"zip";s:5:"label";s:18:"Postal code (text)";}i:10;a:4:{s:4:"name";s:7:"country";s:4:"type";s:4:"text";s:4:"link";s:10:"id_country";s:5:"label";s:14:"Country (text)";}i:11;a:4:{s:4:"name";s:8:"bedrooms";s:4:"type";s:4:"text";s:4:"link";s:13:"fe_bd_rooms_1";s:5:"label";s:15:"Bedrooms (text)";}i:12;a:4:{s:4:"name";s:9:"bathrooms";s:4:"type";s:4:"text";s:4:"link";s:14:"fe_bth_rooms_1";s:5:"label";s:16:"Bathrooms (text)";}i:17;a:4:{s:4:"name";s:10:"year_built";s:4:"type";s:4:"text";s:4:"link";s:9:"fe_year_2";s:5:"label";s:20:"Building year (text)";}i:18;a:4:{s:4:"name";s:8:"currency";s:4:"type";s:4:"text";s:4:"link";s:12:"gid_currency";s:5:"label";s:15:"Currency (text)";}i:21;a:4:{s:4:"name";s:11:"description";s:4:"type";s:4:"text";s:4:"link";s:13:"fe_comments_1";s:5:"label";s:18:"Description (text)";}i:22;a:4:{s:4:"name";s:12:"listing_time";s:4:"type";s:4:"text";s:4:"link";s:13:"date_modified";s:5:"label";s:14:"Updated (text)";}i:23;a:4:{s:4:"name";s:11:"expire_time";s:4:"type";s:4:"text";s:4:"link";s:11:"date_expire";s:5:"label";s:22:"Expiration date (text)";}}',
        'admin_form_data' => '',
        'output_type'     => 'browser',
        'file_format'     => 'test.xml',
        'append_date'     => '1',
        'scheduler'       => 'a:1:{s:4:"type";s:6:"manual";}',
        'published'       => '0',
        'editable'        => '0',
        'status'          => '1',
        'date_created'    => '',
    );

    /**
     * Not available symbols
     *
     * @var array
     */
    protected $search = array('&', "'", '"', ">", "<");

    /**
     * Symbols for replace
     *
     * @var array
     */
    protected $replace = array('&amp;', '&apos;', '&quot;', "&gt;", "&lt;");

    /**
     * Content type
     *
     * @var object
     */
    public $content_file = "text/xml";

    /**
     * Constructor
     *
     * @return Export_vast_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Return default filename
     *
     * @return string
     */
    public function get_filename()
    {
        return "export.xml";
    }

    /**
     * Generate output file
     *
     * @param string $filename output filename
     * @param array  $data     export data
     * @param array  $settings driver settings
     *
     * @return void
     */
    public function generate($filename, $data, $settings)
    {
        $f = fopen($filename, "wb");
        fputs($f, '<?xml version="1.0" encoding="utf-8"?>' . "\n");
        fputs($f, '<listings>' . "\n");
        foreach ($data as $i => $row) {
            if (!$i) {
                continue;
            }
            fputs($f, '	<listing>' . "\n");

            if (isset($row['record_id']) && !empty($row['record_id'])) {
                fputs($f, '		<record_id>' . $row['record_id'] . '</record_id>' . "\n");
            }

            if (isset($row['title']) && !empty($row['title'])) {
                fputs($f, '		<title>' . $row['title'] . '</title>' . "\n");
            }

            if (isset($row['url']) && !empty($row['url'])) {
                fputs($f, '		<url>' . $row['url'] . '</url>' . "\n");
            }

            if (isset($row['category']) && !empty($row['category'])) {
                fputs($f, '		<category>' . $row['category'] . '</category>' . "\n");
            }

            if (isset($row['subcategory']) && !empty($row['subcategory'])) {
                fputs($f, '		<subcategory>' . $row['subcategory'] . '</subcategory>' . "\n");
            }

            if (isset($row['image']) && !empty($row['image'])) {
                fputs($f, '		<image>' . $row['image'] . '</image>' . "\n");
            }

            if (isset($row['address']) && !empty($row['address'])) {
                fputs($f, '		<address>' . $row['address'] . '</address>' . "\n");
            }

            if (isset($row['city']) && !empty($row['city'])) {
                fputs($f, '		<city>' . $row['city'] . '</city>' . "\n");
            }

            if (isset($row['state']) && !empty($row['state'])) {
                fputs($f, '		<state>' . $row['state'] . '</state>' . "\n");
            }

            if (isset($row['zip']) && !empty($row['zip'])) {
                fputs($f, '		<zip>' . $row['zip'] . '</zip>' . "\n");
            }

            if (isset($row['country']) && !empty($row['country'])) {
                fputs($f, '		<country>' . $row['country'] . '</country>' . "\n");
            }

            if (isset($row['bedrooms']) && !empty($row['bedrooms'])) {
                fputs($f, '		<bedrooms>' . $row['bedrooms'] . '</bedrooms>' . "\n");
            }

            if (isset($row['bathrooms']) && !empty($row['bathrooms'])) {
                fputs($f, '		<bathrooms>' . $row['bathrooms'] . '</bathrooms>' . "\n");
            }

            if (isset($row['square_footage']) && !empty($row['square_footage'])) {
                fputs($f, '		<square_footage>' . $row['square_footage'] . '</square_footage>' . "\n");
            }

            if (isset($row['stories']) && !empty($row['stories'])) {
                fputs($f, '		<stories>' . $row['stories'] . '</stories>' . "\n");
            }

            if (isset($row['lot_size']) && !empty($row['lot_size'])) {
                fputs($f, '		<lot_size>' . $row['lot_size'] . '</lot_size>' . "\n");
            }

            if (isset($row['parking_spots']) && !empty($row['parking_spots'])) {
                fputs($f, '		<parking_spots>' . $row['parking_spots'] . '</parking_spots>' . "\n");
            }

            if (isset($row['year_built']) && !empty($row['year_built'])) {
                fputs($f, '		<year_built>' . $row['year_built'] . '</year_built>' . "\n");
            }

            if (isset($row['currency']) && !empty($row['currency'])) {
                fputs($f, '		<currency>' . $row['currency'] . '</currency>');
            }

            if (isset($row['price']) && !empty($row['price'])) {
                fputs($f, '		<price>' . $row['price'] . '</price>' . "\n");
            }

            if (isset($row['amenities']) && !empty($row['amenities'])) {
                fputs($f, '		<amenities>' . $row['amenities'] . '</amenities>' . "\n");
            }

            if (isset($row['description']) && !empty($row['description'])) {
                fputs($f, '		<description>' . $row['description'] . '</description>' . "\n");
            }

            if (isset($row['listing_time']) && !empty($row['listing_time'])) {
                fputs($f, '		<listing_time>' . $row['listing_time'] . '</listing_time>' . "\n");
            }

            if (isset($row['expire_time']) && !empty($row['expire_time'])) {
                fputs($f, '		<expire_time>' . $row['expire_time'] . '</expire_time>' . "\n");
            }

            fputs($f, '	</listing>' . "\n");
        }
        fputs($f, '</listings>' . "\n");
        fclose($f);
    }

    /**
     * Validate export data
     *
     * @param array $data export data
     *
     * @return array
     */
    public function validate_data($data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['record_id'])) {
            $return['data']['record_id'] = intval($data['record_id']);
        }

        if (isset($data['title'])) {
            $return['data']['title'] = str_replace($this->search, $this->replace, $data['title']);
        }

        if (isset($data['url'])) {
            $return['data']['url'] = str_replace($this->search, $this->replace, $data['url']);
        }

        if (isset($data['category'])) {
            $return['data']['category'] = str_replace($this->search, $this->replace, $data['category']);
        }

        if (isset($data['subcategory'])) {
            $return['data']['subcategory'] = str_replace($this->search, $this->replace, $data['subcategory']);
        }

        if (isset($data['image'])) {
            $return['data']['image'] = str_replace($this->search, $this->replace, $data['image']);
        }

        if (isset($data['image'])) {
            $return['data']['image'] = str_replace($this->search, $this->replace, $data['image']);
            if (!empty($return['data']['image'])) {
                $return['data']['image'] = array_shift(explode('|', $return['data']['image']));
            }
        }

        if (isset($data['address'])) {
            $return['data']['address'] = str_replace($this->search, $this->replace, $data['address']);
        }

        if (isset($data['city'])) {
            $return['data']['city'] = str_replace($this->search, $this->replace, $data['city']);
        }

        if (isset($data['state'])) {
            $return['data']['state'] = str_replace($this->search, $this->replace, $data['state']);
        }

        if (isset($data['zip'])) {
            $return['data']['zip'] = str_replace($this->search, $this->replace, $data['zip']);
        }

        if (isset($data['country'])) {
            $return['data']['country'] = str_replace($this->search, $this->replace, $data['country']);
        }

        if (isset($data['bedrooms'])) {
            $return['data']['bedrooms'] = intval($data['bedrooms']);
        }

        if (isset($data['bathrooms'])) {
            $return['data']['bathrooms'] = intval($data['bathrooms']);
        }

        if (isset($data['square_footage'])) {
            $return['data']['square_footage'] = intval($data['square_footage']);
        }

        if (isset($data['stories'])) {
            $return['data']['stories'] = str_replace($this->search, $this->replace, $data['stories']);
        }

        if (isset($data['lot_size'])) {
            $return['data']['lot_size'] = intval($data['lot_size']);
        }

        if (isset($data['parking_spots'])) {
            $return['data']['parking_spots'] = intval($data['parking_spots']);
        }

        if (isset($data['year_built'])) {
            if (strlen($data['year_built']) == 4) {
                $return['data']['year_built'] = intval($data['year_built']);
            }
        }

        if (isset($data['currency'])) {
            $return['data']['currency'] = str_replace($this->search, $this->replace, $data['currency']);
        }

        if (isset($data['price'])) {
            $return['data']['price'] = intval($data['price']);
        }

        if (isset($data['amenities'])) {
            $return['data']['amenities'] = str_replace($this->search, $this->replace, $data['amenities']);
        }

        if (isset($data['description'])) {
            $return['data']['description'] = str_replace($this->search, $this->replace, $data['description']);
        }

        if (isset($data['listing_time'])) {
            $value = strtotime($data['listing_time']);
            if ($value > 0) {
                $return['data']['listing_time'] = date('Y-m-d H:i:s', $value);
            }
        }

        if (isset($data['expire_time'])) {
            $value = strtotime($data['expire_time']);
            if ($value > 0) {
                $return['data']['expire_time'] = date('Y-m-d H:i:s', $value);
            }
        }

        return $return;
    }
}
