<?php

namespace Pg\Modules\Language_identification\Controllers;

/**
 * Language_identification module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Language_identification user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Language_identification
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Language_identification extends \Controller
{

    /**
     * Class constructor
     *
     * @return Language_identification
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Language_identification_model");
    }

    private function getPoint()
    {
        if (!($this->input->post('point'))) {
            return false;
        }
        $auth_type = $this->session->userdata('auth_type');
        if ($auth_type === 'user') {
            return false;
        }
        $this->load->helper('cookie');
        if (get_cookie('lang_id')) {
            return false;
        }
        return explode(';', $this->input->post('point'));
    }

    private function checkLangs($langs)
    {
        foreach ($langs as $lang) {
            $lang_id = $this->pg_language->get_lang_id_by_code($lang[0] . $lang[1], 1);
            if ($lang_id) {
                return $lang_id;
            }
        }
        return false;
    }

    /**
     * Module management
     *
     * @return void
     */
    public function index()
    {
        $point = $this->getPoint();
        if (empty($point)) {
            return false;
        }
        $latitude = floatval($point[0]);
        $longitude = floatval($point[1]);
        $this->load->model('Countries_model');
        $closest_cities = $this->Countries_model->get_closest_cities($latitude, $longitude, 10000);
        if (empty($closest_cities)) {
            log_message('error', '(language_identification) Empty closest cities');
            return false;
        }
        $city = $closest_cities[0];
        $country = $this->Countries_model->get_country($city['country_code']);
        if (empty($country['languages'])) {
            return false;
        }
        $return = array(
            'lang_id' => $this->checkLangs(explode(',', $country['languages'])),
        );
        if (empty($return['lang_id'])) {
            $return['lang_id'] = $this->checkLangs(explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']));
        }

        if ($this->pg_language->current_lang_id == $return['lang_id']) {
            return false;
        } else {
            $this->view->assign($return);
        }
        
        $this->view->render();
    }

}
