<?php

namespace Pg\Modules\Listings\Controllers;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Listings admin side controller
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	controllers
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Listings extends \Controller
{

    /**
     * Available filters for listings management page
     * 
     * @var array
     */
    protected $filters = array('all', 'not_active', 'active');

    /**
     * Constructor
     *
     * @return Admin_Listings
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'listings_menu_item');
        $this->view->setHeader(l('admin_header_listings', 'listings'));
    }

    /**
     * Block of listings management
     * 
     * @param string $filter filter type
     * @param string $order sorting order 
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    private function getListings($filter = null, $order = null, $order_direction = null, $page = null)
    {
        $this->load->model('Listings_model');

        $this->Menu_model->set_menu_active_item('admin_listings_menu', 'listings-list-item');

        $current_settings = isset($_SESSION['listings_list']) ? $_SESSION['listings_list'] : array();
        if (!isset($current_settings['filter'])) {
            $current_settings['filter'] = $this->filters[0];
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }

        if (!$filter) {
            $filter = $current_settings['filter'];
        }
        if (!in_array($filter, $this->filters)) {
            $filter = $this->filters[0];
        }
        $this->view->assign('filter', $filter);

        $current_settings['page'] = $page;
        $current_settings['filter'] = $filter;

        $filter_data['all'] = $this->Listings_model->get_listings_count();
        $filter_data['active'] = $this->Listings_model->get_listings_count(array('active' => 1));
        $filter_data['not_active'] = $this->Listings_model->get_listings_count(array('not_active' => 1));
        $this->view->assign('filter_data', $filter_data);

        if ($this->input->post('filter-submit')) {
            $_SESSION['listings_filters'] = array();

            $post_data['type'] = strval($this->input->post('type', true));
            if ($post_data['type']) {
                $_SESSION['listings_filters']['type'] = $post_data['type'];
            }

            $post_data['id_country'] = strval($this->input->post('id_country', true));
            if ($post_data['id_country']) {
                $_SESSION['listings_filters']['id_country'] = $post_data['id_country'];
            }

            $post_data['id_region'] = intval($this->input->post('id_region', true));
            if ($post_data['id_region']) {
                $_SESSION['listings_filters']['id_region'] = $post_data['id_region'];
            }

            $post_data['id_city'] = intval($this->input->post('id_city', true));
            if ($post_data['id_city']) {
                $_SESSION['listings_filters']['id_city'] = $post_data['id_city'];
            }

            $post_data['id_district'] = intval($this->input->post('id_district', true));
            if ($post_data['id_district']) {
                $_SESSION['listings_filters']['id_district'] = $post_data['id_district'];
            }

            $post_data['category'] = strval($this->input->post('category', true));
            if ($post_data['category']) {
                $t = explode('_', $post_data['category']);
                $_SESSION['listings_filters']['id_category'] = intval($t[0]);
                if (!empty($t[1])) {
                    $_SESSION['listings_filters']['property_type'] = intval($t[1]);
                }
            }

            $post_data['id_user'] = intval($this->input->post('id_user', true));
            if ($post_data['id_user']) {
                $_SESSION['listings_filters']['id_user'] = $post_data['id_user'];
            }

            $post_data['id'] = intval($this->input->post('id', true));
            if ($post_data['id']) {
                $_SESSION['listings_filters']['id'] = $post_data['id'];
            }

            $post_data['keyword'] = trim($this->input->post('keyword', true));
            if ($post_data['keyword']) {
                $_SESSION['listings_filters']['keyword'] = $post_data['keyword'];
            }
        }

        if ($this->input->post('filter-reset')) {
            $_SESSION['listings_filters'] = array();
        }

        $filters = isset($_SESSION['listings_filters']) ? $_SESSION['listings_filters'] : array();
        if ($filter != 'all') {
            $filters[$filter] = '1';
        }

        if (!$order) {
            $order = $current_settings['order'];
        }
        $this->view->assign('order', $order);
        $current_settings['order'] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);
        $current_settings['order_direction'] = $order_direction;

        $listings_count = $this->Listings_model->get_listings_count($filters);

        if (!$page) {
            $page = $current_settings['page'];
        }
        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');

        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $listings_count, $items_on_page);
        $current_settings['page'] = $page;

        $_SESSION['listings_list'] = $current_settings;

        $sort_links = array(
            'date_created' => site_url() . 'admin/listings/' . $filter . '/date_created/' . (($order != 'date_created' xor $order_direction == 'DESC') ? 'ASC' : 'DESC'),
            'review_count' => site_url() . 'admin/listings/' . $filter . '/review_count/' . (($order != 'review_count' xor $order_direction == 'DESC') ? 'ASC' : 'DESC'),
        );
        $this->view->assign('sort_links', $sort_links);

        switch ($order) {
            case 'date_created':
                $order_array = array('id' => $order_direction);
                break;
            default:
                $order_array = array($order => $order_direction);
                break;
        }

        if ($listings_count > 0) {
            $listings = $this->Listings_model->get_listings_list($filters, $page, $items_on_page, $order_array);
            $this->view->assign('listings', $listings);
        }

        $operation_types = $this->Listings_model->get_operation_types(true);
        $this->view->assign('operation_types', $operation_types);

        $is_operation_types_disabled = $this->Listings_model->is_operation_types_disabled();
        $this->view->assign('is_operation_types_disabled', $is_operation_types_disabled);

        $this->load->helper('navigation');
        $this->load->library("pg_date");

        $page_data = get_admin_pages_data(site_url() . 'admin/listings/' . $filter . '/' . $order . '/' . $order_direction . '/', $listings_count, $items_on_page, $page, 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $page_data['filter'] = $filters;

        $this->view->assign('page_data', $page_data);
        $this->view->render('list_listings');
    }

    /**
     * Listings management
     * 
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function index($order = null, $order_direction = null, $page = null)
    {
        $this->getListings(null, $order, $order_direction, $page);
    }

    /**
     * All listings management
     * 
     * @param string $order sorting order 
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function all($order = null, $order_direction = null, $page = null)
    {
        $this->getListings('all', $order, $order_direction, $page);
    }

    /**
     * Active listings manangement
     * 
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function active($order = null, $order_direction = null, $page = null)
    {
        $this->getListings('active', $order, $order_direction, $page);
    }

    /**
     * Inactive listings manangement
     * 
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function not_active($order = null, $order_direction = null, $page = null)
    {
        $this->getListings('not_active', $order, $order_direction, $page);
    }

    /**
     * Listings for sale manangement
     * 
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function sale($order = null, $order_direction = null, $page = null)
    {
        $this->getListings('sale', $order, $order_direction, $page);
    }

    /**
     * Listings for buy manangement
     * 
     * @param string $order sorting order 
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function buy($order = null, $order_direction = null, $page = null)
    {
        $this->getListings('buy', $order, $order_direction, $page);
    }

    /**
     * Show listing data
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function show($listing_id)
    {
        if (!$listing_id) {
            show_404();
            return;
        }

        $this->load->model('Listings_model');

        $current_settings = isset($_SESSION['listings_list']) ? $_SESSION['listings_list'] : array('filter' => 'sale');
        $this->view->assign('filter', $current_settings['filter']);

        $this->load->helper('navigation');

        $url = site_url() . 'admin/listings/index/' . $order . '/' . $order_direction . '/';
        $page_data = get_admin_pages_data($url, $listings_count, $items_on_page, $page, 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $data = $this->Listings_model->get_sale_by_id($listing_id);
        $this->view->assign('data', $data);

        $this->view->render('listings_view');
    }

    /**
     * Edit listing data
     * 
     * @param integer $listing_id listing identifier
     * @param string $section_gid section guid
     * @return void
     */
    public function edit($listing_id = 0, $section_gid = 'overview')
    {
        $this->load->model('Listings_model');
        $this->load->model('listings/models/Listings_module_model');

        $format_settings = array('get_moderation' => true);
        $format_settings['get_location'] = false;

        switch ($section_gid) {
            case 'overview':
                $format_settings['get_user'] = true;
                $format_settings['get_location'] = true;
                break;
            case 'description':
                $format_settings['get_description'] = true;
                break;
            case 'gallery':
                $format_settings['get_photos_all'] = true;
                $format_settings['get_virtual_tours'] = true;
                $format_settings['get_file'] = true;
                $format_settings['get_video'] = true;
                break;
            case 'map':
                $format_settings['get_location'] = true;
                break;
            case 'calendar':
                $format_settings['get_booking'] = true;
                break;
            case 'seo':
                $format_settings['get_user'] = true;
                $format_settings['get_location'] = true;
                break;
        }

        $this->Listings_model->set_format_settings($format_settings);

        if ($listing_id) {
            $data = $this->Listings_model->get_listing_by_id($listing_id);
        } else {
            $is_operation_types_disabled = $this->Listings_model->is_operation_types_disabled();
            if ($is_operation_types_disabled) {
                show_404();
            }
            $data = array(
                'is_featured' => 0,
                'is_lift_up' => 0,
                'is_lift_up_country' => 0,
                'is_lift_up_region' => 0,
                'is_lift_up_city' => 0,
                'is_highlight' => 0,
                'is_slide_show' => 0,
            );
        }

        $format_settings = array(
            'get_user' => true,
            'get_location' => $format_settings['get_location'],
            'get_moderation' => false,
            'get_description' => false,
            'get_photos_all' => false,
            'get_virtual_tours' => false,
            'get_file' => false,
            'get_video' => false,
            'get_booking' => false,
        );
        $this->Listings_model->set_format_settings($format_settings);

        if ($this->input->post('btn_save')) {
            $section = $this->input->post('property_type_section', true);
            $post_data = $this->input->post('data', true);

            $errors = array();
            switch ($section_gid) {
                case 'overview':
                    $post_data['id_country'] = $this->input->post('id_country', true);
                    $post_data['id_region'] = $this->input->post('id_region', true);
                    $post_data['id_city'] = $this->input->post('id_city', true);
                    $post_data['id_district'] = $this->input->post('id_district', true);
                    $post_data['services'] = $this->input->post('services', true);
                    $post_data['id_wish_lists'] = $this->input->post('id_wish_lists', true);
                    $post_data['id_user'] = $this->input->post('id_user', true);

                    if (!$post_data['id_wish_lists']) {
                        $post_data['id_wish_lists'] = array();
                    }

                    if (!$listing_id) {
                        $operation_types = array_flip($this->Listings_model->get_operation_types());
                        $operation_type = $this->input->post('type', true);
                        if ($operation_type && isset($operation_types[$operation_type])) {
                            $post_data['id_type'] = $operation_types[$operation_type];
                        }

                        if (isset($post_data['category'])) {
                            $pos = strpos($post_data['category'], '_');
                            $post_data['id_category'] = substr($post_data['category'], 0, $pos);
                            $post_data['property_type'] = substr($post_data['category'], $pos + 1);
                            unset($post_data['category']);
                        }
                    } else {
                        if (isset($post_data['category'])) {
                            $pos = strpos($post_data['category'], '_');
                            $post_data['property_type'] = substr($post_data['category'], $pos + 1);
                            unset($post_data['category']);
                        }
                    }

                    $post_data['date_open_alt'] = $this->input->post('date_open_alt', true);
                    if ($post_data['date_open_alt']) {
                        $post_data['date_open'] = $post_data['date_open_alt'];
                    }
                    $post_data['date_available_alt'] = $this->input->post('date_available_alt', true);
                    if ($post_data['date_available_alt']) {
                        $post_data['date_available'] = $post_data['date_available_alt'];
                    }

                    if ($post_data['services']['featured'] != $data['is_featured']) {
                        if ($post_data['services']['featured']) {
                            $post_data['featured_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['featured_date_end'] = '0000-00-00 00:00:00';
                        }
                    }
                    if ($post_data['services']['lift_up'] != $data['is_lift_up']) {
                        if ($post_data['services']['lift_up']) {
                            $post_data['lift_up_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['lift_up_date_end'] = '0000-00-00 00:00:00';
                        }
                    }
                    if ($post_data['services']['lift_up_country'] != $data['is_lift_up_country']) {
                        if ($post_data['services']['lift_up_country']) {
                            $post_data['lift_up_country_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['lift_up_country_date_end'] = '0000-00-00 00:00:00';
                        }
                    }
                    if ($post_data['services']['lift_up_region'] != $data['is_lift_up_region']) {
                        if ($post_data['services']['lift_up_region']) {
                            $post_data['lift_up_region_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['lift_up_region_date_end'] = '0000-00-00 00:00:00';
                        }
                    }
                    if ($post_data['services']['lift_up_city'] != $data['is_lift_up_city']) {
                        if ($post_data['services']['lift_up_city']) {
                            $post_data['lift_up_city_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['lift_up_city_date_end'] = '0000-00-00 00:00:00';
                        }
                    }
                    if ($post_data['services']['highlight'] != $data['is_highlight']) {
                        if ($post_data['services']['highlight']) {
                            $post_data['highlight_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['highlight_date_end'] = '0000-00-00 00:00:00';
                        }
                    }
                    if ($post_data['services']['slide_show'] != $data['is_slide_show']) {
                        if ($post_data['services']['slide_show']) {
                            $post_data['slide_show_date_end'] = date('Y-m-d H:i:s', time() + $this->Listings_module_model->service_period_default * 24 * 60 * 60);
                        } else {
                            $post_data['slide_show_date_end'] = '0000-00-00 00:00:00';
                        }
                    }

                    $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        if ($listing_id) {
                            $this->Listings_model->save_listing($listing_id, $validate_data['data']);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_listing', 'listings'));
                        } else {
                            $listing_id = $this->Listings_model->save_listing(null, $validate_data['data']);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_listing', 'listings'));
                        }

                        if (isset($validate_data['data']['id_wish_lists'])) {
                            if (!empty($validate_data['data']['id_wish_lists'])) {
                                $id_wish_lists = (array) unserialize($validate_data['data']['id_wish_lists']);
                            } else {
                                $id_wish_lists = array();
                            }

                            $this->load->model('listings/models/Wish_list_model');

                            foreach ($id_wish_lists as $wish_id) {
                                $params['where']['id_wish_list'] = $wish_id;
                                $params['where']['id_listing'] = $listing_id;
                                $wish_list_by_id = $this->Wish_list_model->_get_wish_list_content_count($params);
                                if (!$wish_list_by_id) {
                                    $this->Wish_list_model->add_to_wish_list($wish_id, array($listing_id));
                                }
                            }

                            if (isset($data['id_wish_lists']) && !empty($data['id_wish_lists'])) {
                                $delete_from_wish_lists = array_diff($data['id_wish_lists'], $id_wish_lists);
                                foreach ($delete_from_wish_lists as $wish_list_id) {
                                    $this->Wish_list_model->delete_from_wish_list($wish_list_id, array($listing_id));
                                }
                                $add_to_wish_lists = array_diff($id_wish_lists, $data['id_wish_lists']);
                                foreach ($add_to_wish_lists as $wish_list_id) {
                                    $this->Wish_list_model->add_to_wish_list($wish_list_id, array($listing_id));
                                }
                            } else {
                                foreach ($id_wish_lists as $wish_list_id) {
                                    $this->Wish_list_model->add_to_wish_list($wish_list_id, array($listing_id));
                                }
                            }
                        }

                        $url = site_url() . 'admin/listings/edit/' . $listing_id . '/' . $section_gid;
                        $this->view->setRedirect($url);
                    }
                    break;
                case 'description':
                    $property_type_gid = $this->Listings_model->get_field_editor_type($data);
                    $this->load->model('listings/models/Listings_field_editor_model');
                    $fields_for_select = $this->Listings_field_editor_model->get_fields_for_select($property_type_gid);
                    foreach ($fields_for_select as $field) {
                        $post_data[$field] = $this->input->post($field, true);
                    }

                    $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        if ($listing_id) {
                            $this->Listings_model->save_listing($listing_id, $validate_data['data']);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_listing', 'listings'));
                        } else {
                            $listing_id = $this->Listings_model->save_listing(null, $validate_data['data']);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_listing', 'listings'));
                        }

                        if (isset($validate_data['data']['id_wish_lists'])) {
                            $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
                            $listing = $this->Listings_model->format_listing($validate_data['data']);
                            $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));

                            $this->load->model('listings/models/Wish_list_model');
                            if (isset($data['id_wish_lists']) && !empty($data['id_wish_lists'])) {
                                $delete_from_wish_lists = array_diff($data['id_wish_lists'], $listing['id_wish_lists']);
                                foreach ($delete_from_wish_lists as $wish_list_id) {
                                    $this->Wish_list_model->delete_from_wish_list($wish_list_id, array($listing_id));
                                }
                                $add_to_wish_lists = array_diff($listing['id_wish_lists'], $data['id_wish_lists']);
                                foreach ($add_to_wish_lists as $wish_list_id) {
                                    $this->Wish_list_model->add_to_wish_list($wish_list_id, array($listing_id));
                                }
                            } else {
                                foreach ($listing['id_wish_lists'] as $wish_list_id) {
                                    $this->Wish_list_model->add_to_wish_list($wish_list_id, array($listing_id));
                                }
                            }
                        }

                        $url = site_url() . 'admin/listings/edit/' . $listing_id . '/' . $section_gid;
                        $this->view->setRedirect($url);
                    }
                    break;
                case 'gallery':
                    if ($this->input->post('btn_save_file')) {
                        $this->load->model('listings/models/Listings_uploads_model');
                        $validate_data = $this->Listings_uploads_model->save_file($listing_id, 'listing_file', $post_data);
                        // delete listing file
                        if ($this->input->post('listing_file_delete') && $listing_id && $data['listing_file']) {
                            $this->load->model('File_uploads_model');
                            $this->File_uploads_model->delete_upload($this->Listings_uploads_model->file_config_id, $data['prefix'], $data['listing_file']);
                            if (!isset($validate_data['data']['file'])) {
                                $listing_data = array(
                                    'listing_file' => '',
                                    'listing_file_name' => '',
                                    'listing_file_comment' => '',
                                );
                                $this->Listings_model->save_listing($listing_id, $listing_data);
                            }
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_file_deleted', 'listings'));
                        } elseif ($validate_data['data']['file']) {
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_file_uploaded', 'listings'));
                        }
                        $step = 3;
                    } else {
                        unset($post_data['listing_file_name']);
                        unset($post_data['listing_file_comment']);
                    }
                    if ($this->input->post('btn_save_video')) {
                        // delete video
                        if ($this->input->post('listing_video_delete')) {
                            $this->load->model('listings/models/Listings_uploads_model');
                            $this->Listings_uploads_model->delete_video($listing_id);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_video_deleted', 'listings'));
                        } else {
                            $listing_video_embed = $this->input->post('listing_video_embed');
                            $this->load->model('listings/models/Listings_uploads_model');
                            $validate_data = $this->Listings_uploads_model->save_video($listing_id, 'listing_video', $listing_video_embed);
                            if ($validate_data['success']) {
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_video_uploaded', 'listings'));
                            }
                        }
                        $step = 4;
                    }
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        $url = site_url() . 'admin/listings/edit/' . $listing_id . '/' . $section_gid;
                        $this->view->setRedirect($url);
                    }
                    break;
                case 'map':
                    $this->load->model('geomap/models/Geomap_model');
                    $this->load->model('geomap/models/Geomap_settings_model');
                    $post_settings = $this->input->post('map', true);
                    $post_settings['lat'] = $post_data['lat'];
                    $post_settings['lon'] = $post_data['lon'];
                    $map_gid = $this->Geomap_model->get_default_driver_gid();
                    $validate_map = $this->Geomap_settings_model->validate_settings($post_settings);
                    if (!empty($validate_map['errors'])) {
                        break;
                    }
                    $this->Geomap_settings_model->save_settings($map_gid, 0, $listing_id, 'listing_view', $validate_map['data']);

                    $save_data = array(
                        'lat' => $post_data['lat'],
                        'lon' => $post_data['lon'],
                    );
                    $this->Listings_model->save_listing($listing_id, $save_data, false);
                    $url = site_url() . 'admin/listings/edit/' . $listing_id . '/' . $section_gid;
                    $this->view->setRedirect($url);
                    break;
                case 'calendar':
                    $post_data = $this->input->post('data', true);
                    $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        $this->Listings_model->save_listing($listing_id, $validate_data['data'], false);
                        $url = site_url() . 'admin/listings/edit/' . $listing_id . '/' . $section_gid;
                        $this->view->setRedirect($url);
                    }
                    break;
                case 'seo':
                    $this->load->model('Seo_model');
                    $seo_fields = $this->Seo_model->get_seo_fields();
                    foreach ($seo_fields as $key => $section_data) {
                        if ($this->input->post('btn_save_' . $section_data['gid'])) {
                            $post_data = array();
                            $post_data[$section_data['gid']] = $this->input->post($section_data['gid'], true);
                            $validate_data = $this->Seo_model->validate_seo_tags($listing_id, $post_data);
                            if (empty($validate_data['errors'])) {
                                $listing_data['id_seo_settings'] = $this->Seo_model->save_seo_tags($data['id_seo_settings'], $validate_data['data']);
                                if (!$data['id_seo_settings']) {
                                    $data['id_seo_settings'] = $listing_data['id_seo_settings'];
                                    $this->Listings_model->save_listing($listing_id, $listing_data, false);
                                }
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_settings_updated', 'seo'));
                            }
                            break;
                        }
                    }
                    break;
            }

            if (!empty($post_data)) {
                $this->Listings_model->set_format_settings('use_format', false);
                $data = $this->Listings_model->get_listing_by_id($listing_id);
                $this->Listings_model->set_format_settings('use_format', true);

                $post_data = $this->Listings_model->format_listings(array(array_merge($data, $post_data)));
                $data = $post_data[0];
            }
        }

        if ($listing_id) {
            $is_moderation_installed = $this->pg_module->is_module_active('moderation');
            if ($is_moderation_installed) {
                $this->load->model('listings/models/Listings_moderation_model');
                if ($this->Listings_moderation_model->is_listing_moderated($listing_id)) {
                    $this->view->assign('on_moderation', 1);
                    $this->system_messages->addMessage(View::MSG_ERROR, l('error_listing_on_moderation', 'listings'));
                }
            }
        }

        switch ($section_gid) {
            case 'overview':
                $operation_types = $this->Listings_model->get_operation_types();
                $this->view->assign('operation_types', $operation_types);
                $this->view->assign('operation_types_count', count($operation_types));

                if ($this->pg_module->is_module_active('currencies')) {
                    $this->load->model('Currencies_model');
                    $currencies = $this->Currencies_model->get_currency_list();
                    $this->view->assign('currencies', $currencies);
                    if (isset($data['gid_currency']) && !empty($data['gid_currency'])) {
                        $current_price_currency = $this->Currencies_model->get_currency_by_gid($data['gid_currency']);
                        $this->view->assign('current_price_currency', $current_price_currency);
                    } else {
                        $current_price_currency = $this->Currencies_model->get_currency_default();
                        $data['gid_currency'] = $current_price_currency['gid'];
                    }
                }

                $this->view->assign('current_lang_id', $this->pg_language->current_lang_id);
                $this->view->assign('langs', $this->pg_language->languages);

                $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
                $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
                $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
                $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
                $this->view->assign('page_data', $page_data);

                // check pay services exists
                if ($this->pg_module->is_module_active('services')) {
                    $this->load->model('Services_model');
                    $services_params = array(
                        'where_in' => array(
                            'template_gid' => array(
                                'listings_featured_template',
                                'listings_lift_up_template',
                                'listings_lift_up_country_template',
                                'listings_lift_up_region_template',
                                'listings_lift_up_city_template',
                                'listings_highlight_template',
                                'listings_slide_show_template',
                            ),
                        ),
                        'where' => array(
                            'status' => 1,
                        ),
                    );
                    $services_count = $this->Services_model->get_service_count($services_params);
                    if ($services_count > 0) {
                        $listings_services = $this->Services_model->get_service_list($services_params);
                        foreach ($listings_services as $service) {
                            $this->view->assign('use_' . $service['gid'], $service['status']);
                        }
                        $this->view->assign('use_services', true);
                    } else {
                        $this->view->assign('use_services', false);
                    }
                }
                break;
            case 'description':
                $this->load->model('Field_editor_model');
                $property_type_gid = $this->Listings_model->get_field_editor_type($data);
                $this->Field_editor_model->initialize($property_type_gid);
                $property_type_sections = $this->Field_editor_model->get_section_list();

                if (!empty($property_type_sections)) {
                    $sections = array();
                    foreach ($property_type_sections as $section) {
                        $sections[] = $section['gid'];
                    }
                    $this->view->assign('sections_data', array_values($property_type_sections));

                    $params['where_in']['section_gid'] = $sections;
                    $fields_data = $this->Field_editor_model->get_form_fields_list($data, $params);
                    $this->view->assign('fields_data', $fields_data);
                }
                break;
            case 'gallery':
                $this->load->model('listings/models/Listings_uploads_model');

                // photos
                $this->view->assign('gallery_type', $this->Listings_uploads_model->gallery_type);

                // virtual tours
                $this->view->assign('vtour_type', $this->Listings_uploads_model->virtual_tour_type);

                // files
                $file_type_config = $this->Listings_uploads_model->get_file_type();

                $max_size = $file_type_config['max_size'];

                $i = 1;
                while ($max_size > 1000) {
                    $max_size /= 1024;
                    $i++;
                }
                $max_size = round($max_size, 1);
                if ($max_size) {
                    $file_type_config['max_size_str'] = $max_size . ' ' . l('text_filesize_' . $i, 'listings');
                }
                $this->view->assign('file_settings', $file_type_config);

                // videos
                $video_type_config = $this->Listings_uploads_model->get_video_type();

                $max_size = $video_type_config['max_size'];

                $i = 1;
                while ($max_size > 1000) {
                    $max_size /= 1024;
                    $i++;
                }
                $max_size = round($max_size, 1);
                if ($max_size) {
                    $video_type_config['max_size_str'] = $max_size . ' ' . l('text_filesize_' . $i, 'listings');
                }
                $this->view->assign('video_settings', $video_type_config);
                break;
            case 'map':
                $markers = array(
                    array(
                        'gid' => $data['id'],
                        'country' => $data['country'],
                        'region' => $data['region'],
                        'city' => $data['city'],
                        'address' => $data['address'],
                        'postal_code' => $data['zip'],
                        'lat' => (float) $data['lat'],
                        'lon' => (float) $data['lon'],
                        'dragging' => true,
                        'info' => ($data['location'] ? $data['location'] : ''),
                    ),
                );
                $this->view->assign('markers', $markers);

                $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
                $view_settings = array(
                    'disable_smart_zoom' => 1,
                    'zoom_listener' => 'get_listing_zoom_data',
                    'type_listener' => 'get_listing_type_data',
                    'drag_listener' => 'get_listing_drag_data',
                    'panorama_listener' => 'get_listing_panorama',
                    'lang' => $current_language['code'],
                    'lat' => (float) $data['lat'],
                    'lon' => (float) $data['lon'],
                );
                $this->view->assign('map_settings', $view_settings);

                if ($listing_id) {
                    $this->load->model('geomap/models/Geomap_model');
                    $this->load->model('geomap/models/Geomap_settings_model');
                    $map_gid = $this->Geomap_model->get_default_driver_gid();
                    $map_settings = $this->Geomap_settings_model->get_settings($map_gid, 0, $listing_id, 'listing_view');
                    $this->view->assign('listing_map_settings', $map_settings);
                }
                break;
            case 'calendar':

                break;
            case 'seo':
                $this->load->model('Seo_model');
                $seo_fields = $this->Seo_model->get_seo_fields();
                $this->view->assign('seo_fields', $seo_fields);

                $languages = $this->pg_language->languages;
                $this->view->assign('languages', $languages);

                $current_lang_id = $this->pg_language->current_lang_id;
                $this->view->assign('current_lang_id', $current_lang_id);

                if (!empty($data['id_seo_settings'])) {
                    $seo_settings = $this->Seo_model->get_seo_tags($data['id_seo_settings']);
                    $this->view->assign('seo_settings', $seo_settings);
                }
                break;
        }

        $this->view->assign('section_gid', $section_gid);
        $this->view->assign('data', $data);

        $this->Menu_model->set_menu_active_item('admin_listings_menu', 'listings-list-item');
        $this->view->render('edit_listings_form');
    }

    /**
     * Listing services management
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function services($listing_id)
    {
        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('use_format', false);
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        $this->view->assign('section_gid', 'services');
        $this->view->assign('data', $data);
        $this->view->assign('page_data', $page_data);

        if ($this->pg_module->is_module_active('services')) {
            $this->load->model('Services_model');
            // check pay services exists
            $services_params = array(
                'where_in' => array(
                    'template_gid' => array(
                        'listings_featured_template',
                        'listings_lift_up_template',
                        'listings_highlight_template',
                        'listings_slide_show_template',
                    ),
                ),
                'where' => array(
                    'status' => 1,
                ),
            );
            $services_count = $this->Services_model->get_service_count($services_params);
            if ($services_count) {
                $page_data['listings_services'] = $this->Services_model->get_service_list($services_params);
                $page_data['use_services'] = true;
            }
        } else {
            $page_data['use_services'] = false;
        }

        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        $this->view->assign('section_gid', 'services');
        $this->view->assign('data', $data);
        $this->view->assign('page_data', $page_data);
        $this->view->assign('section_gid', 'services');

        $this->view->render('edit_listings_services_form');
    }

    /**
     * Remove listings by identifiers
     * 
     * @param array/integer $listing_ids listing identifier
     * @return void
     */
    public function delete($listing_ids = null)
    {
        if (!$listing_ids) {
            $listing_ids = $this->input->post('ids', true);
        }
        if (!empty($listing_ids)) {
            $this->load->model('Listings_model');
            foreach ((array) $listing_ids as $listing_id)
                $this->Listings_model->delete_listing($listing_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_listings_delete', 'listings'));
        }
        $this->view->setRedirect(site_url() . 'admin/listings/index');
    }

    /**
     * Activate/deactivate listings by identifiers
     * 
     * Available status values: 1 - activate, 0 - deactivate
     * 
     * @param array/integer $listings_ids listing identifier
     * @param integer $status activation status
     * @return void
     */
    public function activate($listings_ids, $status = 0)
    {
        if (!$listings_ids) {
            $listings_ids = $this->input->post('ids', true);
        }
        if (!empty($listings_ids)) {
            $this->load->model('Listings_model');
            
            $this->Listings_model->set_format_settings('use_format', false);
            $listings = $this->Listings_model->get_listings_list_by_key(['ids' => (array)$listings_ids]);
            $this->Listings_model->set_format_settings('use_format', true);
            
            foreach ((array) $listings_ids as $listing_id) {
                if ($status) {
                    $validate_data = $this->Listings_model->validateForActivation($listings[$listing_id]);
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                        $this->view->setRedirect(site_url() . 'admin/listings/index/');
                        return;
                    }
                    
                    $this->Listings_model->set_format_settings('use_format', false);
                    $data = $this->Listings_model->get_listing_by_id($listing_id);
                    $this->Listings_model->set_format_settings('use_format', true);
                    $post_period = $this->pg_module->get_module_config('listings', 'default_activation_period');
                    if (strtotime($data['date_activity']) > time()) {
                        $date_activity = date('Y-m-d H:i:s', strtotime($data['date_activity']) + $post_period * 24 * 60 * 60);
                    } else {
                        $date_activity = date('Y-m-d H:i:s', time() + $post_period * 24 * 60 * 60);
                    }
                    $this->Listings_model->activate_listing($listing_id, $status, $date_activity);
                } else {
                    $this->Listings_model->activate_listing($listing_id, $status);
                }
            }
            if ($status) {
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing', 'listings'));
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_deactivate_listing', 'listings'));
            }
        }
        $url = site_url() . 'admin/listings/index/';
        $this->view->setRedirect($url);
    }

    /**
     * Export form block by ajax
     * 
     * @return void
     */
    public function ajax_get_export_extend_form()
    {
        $operation_type = $this->input->post('type', true);
        $category_id = $this->input->post('id_category');
        $category_id = explode('_', $category_id);

        $this->load->model('Listings_model');

        $data = array('operation_type' => $operation_type, 'id_category' => $category_id[0]);
        $property_type_gid = $this->Listings_model->get_field_editor_type($data);

        $this->load->model('listings/models/Listings_export_model');

        $this->view->output($this->Listings_export_model->get_export_extend_form($property_type_gid, 'admin_export_form_gid'));

        $this->view->render();
    }

    /**
     * Wish lists management
     * 
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function wish_lists($order = 'date_add', $order_direction = 'DESC', $page = 1)
    {
        $this->load->model('listings/models/Wish_list_model');
        $this->Menu_model->set_menu_active_item('admin_listings_menu', 'wish-lists-item');
        $filters = array();
        $current_settings = isset($_SESSION['wish_list']) ? $_SESSION['wish_list'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_add';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }

        $current_settings['page'] = $page;

        if (!$order) {
            $order = $current_settings['order'];
        }
        $this->view->assign('order', $order);
        $current_settings['order'] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);
        $current_settings['order_direction'] = $order_direction;

        $wish_lists_count = $this->Wish_list_model->get_wish_lists_count($filters);

        if (!$page) {
            $page = $current_settings['page'];
        }
        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');

        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $wish_lists_count, $items_on_page);
        $current_settings['page'] = $page;

        $_SESSION['wish_list'] = $current_settings;

        if ($order == 'user') {
            $order = 'id_user';
        }

        if ($wish_lists_count > 0) {
            $wish_lists = $this->Wish_list_model->get_wish_lists_list($filters, $page, $items_on_page, array($order => $order_direction));
            $this->view->assign('wish_lists', $wish_lists);
        }

        $this->load->helper('navigation');

        $url = site_url() . 'admin/listings/wish_lists/' . $order . '/' . $order_direction . '/';
        $page_data = get_admin_pages_data($url, $wish_lists_count, $items_on_page, $page, 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $this->view->render('wish_lists');
    }

    /**
     * Edit wish list data
     * 
     * @param integer $wish_list_id wish list identifier
     * @return void
     */
    public function edit_wish_list($wish_list_id = 0)
    {
        $this->load->model('listings/models/Wish_list_model');

        if ($wish_list_id) {
            $data = $this->Wish_list_model->get_wish_list_by_id($wish_list_id, true);
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = $this->input->post('data', true);
            $post_data['id_user'] = 0;
            $validate_data = $this->Wish_list_model->validate_wish_list($wish_list_id, $post_data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
            } else {
                if (!empty($validate_data['data'])) {
                    $this->Wish_list_model->save_wish_list($wish_list_id, $validate_data['data']);
                    if ($wish_list_id) {
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_wish_list', 'listings'));
                    } else {
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_wish_list', 'listings'));
                    }
                    $url = site_url() . 'admin/listings/wish_lists';
                    $this->view->setRedirect($url);
                }
            }
        }
        $this->view->assign('data', $data);

        $this->view->assign('current_lang_id', $this->pg_language->current_lang_id);
        $this->view->assign('langs', $this->pg_language->languages);

        $this->Menu_model->set_menu_active_item('admin_listings_menu', 'wish-lists-item');
        $this->view->render('edit_wish_list_form');
    }

    /**
     * Remove wish lists by identifiers
     * 
     * @param integer $wish_list_ids wish lists identifiers
     * @return void
     */
    public function delete_wish_list($wish_list_ids = null)
    {
        if (!$wish_list_ids) {
            $wish_list_ids = $this->input->post('ids', true);
        }
        if (!empty($wish_list_ids)) {
            $this->load->model('listings/models/Wish_list_model');
            foreach ((array) $wish_list_ids as $wish_list_id)
                $this->Wish_list_model->delete_wish_list($wish_list_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_wish_lists_delete', 'listings'));
        }
        $this->view->setRedirect(site_url() . 'admin/listings/wish_lists/');
    }

    /**
     * Activate/deactivate wish list
     * 
     * Available status values: 1- activate, 0 - deactivate
     * 
     * @param integer $wish_list_id wish list identifier
     * @param integer $status activation status
     * @return void
     */
    public function activate_wish_list($wish_list_id, $status = 0)
    {
        if (!empty($wish_list_id)) {
            $this->load->model('listings/models/Wish_list_model');
            if ($status) {
                if ($this->Wish_list_model->activate_wish_list($wish_list_id)) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_wish_list', 'listings'));
                } else {
                    $this->system_messages->addMessage(View::MSG_ERROR, l('error_activate_wish_list', 'listings'));
                }
            } else {
                $this->Wish_list_model->deactivate_wish_list($wish_list_id, $status);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_deactivate_wish_list', 'listings'));
            }
        }
        $url = site_url() . 'admin/listings/wish_lists/';
        $this->view->setRedirect($url);
    }

    /**
     * Wish list content management
     * 
     * @param integer $wish_list_id wish list identifier
     * @param integer $page page of results
     * @return void
     */
    public function wish_list_content($wish_list_id, $page = 1)
    {
        $this->load->model('Listings_model');
        $this->load->model('listings/models/Wish_list_model');

        $this->Wish_list_model->set_format_content_settings('get_listings', true);
        $data = $this->Wish_list_model->get_wish_list_by_id($wish_list_id, true);
        $this->Wish_list_model->set_format_content_settings('get_listings', false);
        if (!$data) {
            show_404();
        }
        $this->view->assign('wish_list', $data);

        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');

        $listings_count = $this->Wish_list_model->get_wish_list_content_count($wish_list_id);
        if ($listings_count > 0) {
            $this->Wish_list_model->set_format_content_settings('get_listings', true);
            $listings = $this->Wish_list_model->get_wish_list_content_list($wish_list_id, $page, $items_on_page);
            $this->Wish_list_model->set_format_content_settings('get_listings', false);
            $this->view->assign('listings', $listings);
        }

        $this->load->helper('navigation');

        $url = site_url() . 'admin/listings/wish_list_content/' . $wish_list_id . '/';
        $page_data = get_admin_pages_data($url, $listings_count, $items_on_page, $page, 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->set_menu_active_item('admin_listings_menu', 'wish-lists-item');

        $this->view->setHeader(str_replace('[name]', $data['name'], l('admin_header_wish_list_content', 'listings')));
        $this->view->setBackLink(site_url() . 'admin/listings/wish_lists');

        $this->view->render('wish_list_content');
    }

    /**
     * Add listings to wish list
     * 
     * @param integer $wish_list_id wish list identifier
     * @return void
     */
    public function add_to_wish_list($wish_list_id)
    {
        $listing_ids = $this->input->post('ids', true);
        $this->load->model('listings/models/Wish_list_model');
        $this->Wish_list_model->add_to_wish_list($wish_list_id, $listing_ids);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_to_wish_list', 'listings'));
        $this->view->setRedirect(site_url() . 'admin/listings/wish_list_content/' . $wish_list_id);
    }

    /**
     * Remove listing from wish list
     * 
     * @param integer $wish_list_id wish list identifier
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_from_wish_list($wish_list_id, $listing_id = false)
    {
        $this->load->model('listings/models/Wish_list_model');
        if ($listing_id) {
            $this->Wish_list_model->delete_from_wish_list($wish_list_id, array($listing_id));
        } else {
            $this->Wish_list_model->clear_wish_list($wish_list_id);
        }
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_from_wish_list', 'listings'));
        $this->view->setRedirect(site_url() . 'admin/listings/wish_list_content/' . $wish_list_id);
    }

    /**
     * Resort wish list content by ajax
     * 
     * @param integer $wish_list_id wish list identifier
     * @return void
     */
    public function ajax_save_wish_list_sorting($wish_list_id)
    {
        $this->load->model('listings/models/Wish_list_model');

        $sorter = $this->input->post('sorter', true);
        foreach ($sorter as $item_str => $sort_index) {
            $listing_id = intval(str_replace('pItem', '', $item_str));
            $this->Wish_list_model->save_content_sorter($wish_list_id, $listing_id, $sort_index);
        }

        $this->Wish_list_model->change_logo($wish_list_id);

        $return['success'] = l('success_wish_list_sorter_saved', 'listings');

        $this->view->assign($return);

        $this->view->render();

        return;
    }

    /**
     * Remove listing object from wish list
     * 
     * @param integer $wish_list_id wish list identifier
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_delete_from_wish_list($wish_list_id, $listing_id)
    {
        $this->load->model('listings/models/Wish_list_model');
        $this->Wish_list_model->delete_from_wish_list($wish_list_id, array($listing_id));
        $return['success'] = l('success_delete_from_wish_list', 'listings');

        $this->view->assign($return);

        $this->view->render();
    }

    /**
     * Refresh wish list content
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_reload_wish_list_content($listing_id)
    {
        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        $this->view->assign('listing', $listing);
        $this->view->render('listing_view_block');
    }

    /**
     * Change map settings of listing object
     * 
     * @param integer $listing_id listing identifier
     * @param string $option_name map option name
     * @return void
     */
    public function ajax_set_map($listing_id, $option_name)
    {
        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('use_format' => true));

        if (!$listing) {
            return;
        }

        $this->load->model('Geomap_model');
        $this->load->model('geomap/models/Geomap_settings_model');

        $save_data = array();

        $map_gid = $this->Geomap_model->get_default_driver_gid();
        $settings = $this->Geomap_settings_model->get_settings($map_gid, 0, $listing_id, 'listing_view');
        if (!$settings) {
            $settings = array('gid' => 'listing_view');
        }
        switch ($option_name) {
            case 'type': $settings['view_type'] = $this->input->post('type', true);
                break;
            case 'zoom': $settings['zoom'] = $this->input->post('zoom', true);
                break;
            case 'coordinates':
                $settings['lat'] = $save_data['lat'] = $this->input->post('lat', true);
                $settings['lon'] = $save_data['lon'] = $this->input->post('lon', true);
                break;
        }

        $validate_data = $this->Geomap_settings_model->validate_settings($settings);
        if (empty($validate_data['errors']) && !empty($validate_data['data'])) {
            $this->Geomap_settings_model->save_settings($map_gid, 0, $listing_id, 'listing_view', $validate_data['data']);
        }

        $validate_data = $this->Listings_model->validate_listing($listing_id, $save_data);
        if (empty($validate_data['errors']) && !empty($validate_data['data'])) {
            $this->Listings_model->save_listing($listing_id, $validate_data['data']);
        }
    }

    /**
     * Render calendar block by ajax
     * 
     * @param integer $listing_id listing identifier
     * @param integer $count number of calendars
     * @param integer $year year
     * @param integer $month month
     * @return void
     */
    public function ajax_get_calendar($listing_id, $count, $year, $month = 1)
    {
        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('get_booking', true);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('get_booking', false);

        $params = array(
            'listing' => $listing,
            'count' => $count,
            'year' => $year,
            'month' => $month,
            'template' => 'view',
        );

        $this->load->helper('listings');

        $this->view->output(listings_calendar_block($params, true));

        $this->view->render();
    }

    /**
     * Listings objects by user
     * 
     * @param string $user_id user identifier
     * @return void
     */
    public function user($user_id)
    {
        if (!isset($_SESSION['listings_filters'])) {
            $_SESSION['listings_filters'] = array();
        }
        $_SESSION['listings_filters']['id_user'] = $user_id;
        $this->view->setRedirect(site_url() . 'admin/listings/index');
    }

    /**
     * Show form to choise listings by ajax
     * 
     * @param integer $max_select max selected listings 
     * @param string $template template name
     * @return void
     */
    public function ajax_get_listings_form($max_select = 1, $template = 'default')
    {
        $this->load->model('Listings_model');
        
        $selected = $this->input->post('selected', true);
        if (!empty($selected)) {
            $params['ids'] = $selected;
            $data['selected'] = $this->Listings_model->get_listings_list($params, null, null, array('date_modified' => 'DESC'));
        } else {
            $data['selected'] = array();
        }
        $data['max_select'] = $max_select ? $max_select : 0;

        $this->view->assign('select_data', $data);
        $this->view->render('ajax_listing_select_form_' . $template);
    }

    /**
     * Form to choose wish lists by ajax
     * 
     * @param integer $max_select max wish list for selected
     * @param string $template template name
     * @return void
     */
    public function ajax_get_wish_lists_form($max_select = 1, $template = 'default')
    {
        $selected = $this->input->post('selected', true);

        $this->load->model('listings/models/Wish_list_model');

        if (!empty($selected)) {
            $filters['ids'] = $selected;
            $data['selected'] = $this->Wish_list_model->get_wish_lists_list($filters, null, null, array('name' => 'asc'));
        } else {
            $data['selected'] = array();
        }
        $data['max_select'] = $max_select ? $max_select : 0;

        $this->view->assign('select_data', $data);
        $this->view->render('ajax_wish_list_select_form_' . $template);
    }

    /**
     * Listings moderation pending moderation management
     * 
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function moderation($order = 'date_created', $order_direction = 'DESC', $page = 1)
    {
        $this->load->model('listings/models/Listings_moderation_model');

        $current_settings = isset($_SESSION['listings_list']) ? $_SESSION['listings_list'] : array();

        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $current_settings['page'] = $page;

        if (!$order) {
            $order = $current_settings['order'];
        }
        $this->view->assign('order', $order);
        $current_settings['order'] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);
        $current_settings['order_direction'] = $order_direction;

        $listings_count = $this->Listings_moderation_model->get_listings_count();

        if (!$page) {
            $page = $current_settings['page'];
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');

        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $listings_count, $items_on_page);
        $current_settings['page'] = $page;

        $_SESSION['listings_list'] = $current_settings;

        $sort_links = array(
            'date_created' => site_url() . 'admin/listings/moderation/date_created/' . (($order != 'date_created' xor $order_direction == 'DESC') ? 'ASC' : 'DESC'),
        );

        $this->view->assign('sort_links', $sort_links);

        if ($listings_count > 0) {
            $listings = $this->Listings_moderation_model->get_listings_list($page, $items_on_page, array($order => $order_direction));
            $this->view->assign('listings', $listings);
        }

        $this->load->helper('navigation');

        $url = site_url() . 'admin/listings/moderation/' . $order . '/' . $order_direction . '/';
        $page_data = get_admin_pages_data($url, $listings_count, $items_on_page, $page, 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->set_menu_active_item('admin_menu', 'moderation-items');
        $this->view->render('list_listings_moderation');
    }

    /**
     * Edit moderated listing data
     * 
     * @param integer $listing_id listing identifier
     * @param string $section_gid view section
     * @return void
     */
    public function moderation_edit($listing_id, $section_gid = 'overview')
    {
        $this->load->model('listings/models/Listings_model');

        switch ($section_gid) {
            case 'gallery':
                $this->Listings_model->set_format_settings(array('get_file' => true, 'get_video' => true));
                break;
        }
        $this->load->model('listings/models/Listings_moderation_model');
        $data = $this->Listings_moderation_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_file' => false, 'get_video' => false));

        $property_type_gid = $this->Listings_model->get_field_editor_type($data);

        $this->load->model('Field_editor_model');
        $this->Field_editor_model->initialize($property_type_gid);

        $fields_for_select = $this->Field_editor_model->get_fields_for_select();
        $this->Listings_moderation_model->set_dop_fields($fields_for_select);

        if (isset($_POST['btn_save']) || isset($_POST['btn_save_approve']) || isset($_POST['btn_decline'])) {
            $post_data = $this->input->post('data', true);

            if (!is_array($post_data)) {
                $post_data = array();
            }

            if ($section_gid == 'gallery' && empty($data['listing_file'])) {
                unset($post_data['listing_file_name']);
                unset($post_data['listing_file_comment']);
            }

            foreach ($fields_for_select as $field) {
                $post_data[$field] = $this->input->post($field, true);
            }

            $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);

            $validate_data['data']['admin_alert'] = $this->input->post('admin_alert', true);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
            } else {
                $this->Listings_moderation_model->save_listing($listing_id, $validate_data["data"]);

                // delete listing file
                if ($this->input->post('listing_file_delete') && $listing_id && $data['listing_file']) {
                    $this->Listings_moderation_model->delete_file($listing_id);
                }

                // delete video
                if ($this->input->post('listing_video_delete')) {
                    $this->Listings_moderation_model->delete_video($listing_id);
                }

                if (isset($_POST['btn_save'])) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_moderation', 'listings'));
                    $this->view->setRedirect(site_url() . 'admin/listings/moderation_edit/' . $listing_id . '/' . $section_gid);
                }

                if (isset($_POST['btn_save_approve'])) {
                    $this->Listings_moderation_model->approve_listing($listing_id);

                    $this->load->helper('date_format');

                    $this->load->model('Notifications_model');
                    $this->load->model('Users_model');

                    $this->Users_model->set_format_settings('get_safe', false);
                    $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
                    $this->Users_model->set_format_settings('get_safe', true);

                    $email = $listing_data['user']['email'];
                    $lang_id = $listing_data['user']['lang_id'];
                    $listing_data['user'] = $listing_data['user']['fname'];
                    $listing_data['listing'] = l('field_id', 'listings') . $listing_data['id'] . ', ' . $listing_data['location'];
                    $listing_data['status'] = $listing_data['status'] ? l('text_approve_active', 'listings', $lang_id) . ' ' .
                        tpl_date_format($listing_data['date_expire'], $this->pg_date->get_format('date_literal', 'st')) :
                        l('text_approve_inactive', 'listings', $lang_id);
                    $this->Notifications_model->send_notification($email, 'listing_status_updated', $listing_data, '', $lang_id);

                    $this->view->addMessage(View::MSG_SUCCESS, l('success_approve_moderation', 'listings'));
                    $this->view->setRedirect(site_url() . 'admin/listings/moderation/');
                }

                if (isset($_POST['btn_decline'])) {
                    $this->Listings_moderation_model->decline_listing($listing_id);
                    $this->load->model('Notifications_model');
                    $this->load->model('Users_model');
                    $this->Users_model->set_format_settings('get_safe', false);
                    $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
                    $this->Users_model->set_format_settings('get_safe', true);

                    $email = $listing_data['user']['email'];
                    $lang_id = $listing_data['user']['lang_id'];
                    $listing_data['user'] = $listing_data['user']['fname'];
                    $listing_data['listing'] = l('field_id', 'listings') . $listing_data['id'] . ', ' . $listing_data['location'];
                    $listing_data['status'] = l('text_decline', 'listings');
                    $this->Notifications_model->send_notification($email, 'listing_status_updated', $listing_data, '', $lang_id);

                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_decline_moderation', 'listings'));
                    $this->view->setRedirect(site_url() . 'admin/listings/moderation/');
                }
            }

            $data = array_merge($data, $post_data);
        }

        $params = array();
        $params['where']['editor_type_gid'] = $property_type_gid;
        $params['where_in']['field_type'] = array('text', 'textarea');
        $fields_data = $this->Field_editor_model->get_form_fields_list($data, $params);
        $this->view->assign('fields_data', $fields_data);

        $compare_data = $this->Listings_moderation_model->compare($listing_id);
        $this->view->assign('compare_data', $compare_data);

        $this->view->assign('section_gid', $section_gid);
        $this->view->assign('data', $data);

        $this->Menu_model->set_menu_active_item('admin_menu', 'moderation-items');
        $this->view->render('edit_listings_moderation_form');
    }

    /**
     * fix stats
     */
    public function fixStats()
    {
        $this->load->model('Listings_model');
        $this->load->model('listings/models/Listings_stat_model');
        $this->load->model('Properties_model');

        $lang_id = $this->pg_language->current_lang_id;

        $categories_gid = current($this->Properties_model->categories);
        $categories = $this->Properties_model->get_categories($categories_gid, $lang_id);

        $all_properties = $this->Properties_model->get_all_properties($categories_gid, $lang_id);

        $columns = array('sale_cnt' => '1', 'buy_cnt' => '2', 'rent_cnt' => '3', 'lease_cnt' => '4');

        $a = &$columns;
        foreach ($columns as $type_name => $id_type) {
            $b = &$categories;
            foreach ($categories as $id_category => $cat_name) {
                $c = &$all_properties;
                foreach ($all_properties[$id_category] as $property_type => $property_name) {
                    $params['id_type'] = $id_type;
                    $params['category'] = $id_category;
                    $params['property_type'] = $property_type;
                    $params['active'] = 1;

                    if ($id_type == 1) {
                        $params['sold'] = 0;
                    }

                    $cnt = $this->Listings_model->get_listings_count($params);

                    $stat = $this->Listings_stat_model->get_stat_by_id('property_type_' . $id_category . '_' . $property_type);
                    if (!empty($stat)) {
                        $save_data[$type_name] = $cnt;
                        $validate_data = $this->Listings_stat_model->validate_stat($stat['id'], $save_data);
                        $this->Listings_stat_model->save_stat($stat['id'], $save_data);
                        unset($validate_data['data'][$type_name]);
                        unset($validate_data['data']);
                        unset($save_data);
                    }
                }
                unset($c);
            }
            unset($b);
        }
        unset($a);
        
        $this->view->output('Complete');
        
        $this->view->render();
    }

    /**
     * fix users stats
     */
    public function fixUsersStats()
    {

        $this->load->model('Listings_model');
        $this->load->model('Users_model');

        $users = $this->Users_model->get_users_list();

        $columns = array(
            'listings_for_sale_count' => '1',
            'listings_for_buy_count' => '2',
            'listings_for_rent_count' => '3',
            'listings_for_lease_count' => '4',
        );

        $a = &$users;
        foreach ($users as $key => $user) {
            $b = &$columns;
            foreach ($columns as $type_name => $id_type) {
                $params['id_type'] = $id_type;
                $params['id_user'] = $user['id'];
                $params['active'] = 1;
                if ($id_type == 1) {
                    $params['sold'] = 0;
                }

                $stats[$type_name] = $this->Listings_model->get_listings_count($params);
            }
            unset($b);
            $this->Users_model->save_user($user['id'], $stats);
        }
        unset($a);

        $this->view->output('<pre>Complite fix users stats</pre>');
        
        $this->view->render();
    }

    /**
     * Settings page
     */
    public function settings()
    {
        $this->load->model('Listings_model');
        $new_settings = $this->input->post('settings');
        if (!empty($new_settings)) {
			$operation_types_enabled = false;
            $operation_types = array_reverse($this->Listings_model->operations_arr);
            foreach ($operation_types as $operation_type) {
                if ($new_settings['for_' . $operation_type . '_enabled']) {
                    $operation_types_enabled = true;
					break;
                }
            }
            if (!$operation_types_enabled) {
                $this->system_messages->addMessage(View::MSG_ERROR, l("error_empty_operation_type", "start"));
            } else {				
				$this->Listings_model->setModuleConfig($new_settings);
			}
        }
        $config = $this->Listings_model->getModuleConfig();
        $this->additionalSettings();
        $this->view->setHeader(l('admin_header_settings_edit', 'listings'));
        $this->view->assign('listings_config', $config);
    }

    private function additionalSettings()
    {
        if ($this->pg_module->is_module_installed('listings_booking')) {
            $booking_config = ['use' => $this->pg_module->is_module_active('listings_booking')];
            $new_booking_config = $this->input->post('booking_config');
            if (!empty($new_booking_config) && ($booking_config['use'] != $new_booking_config['use'])) {
                $this->pg_module->setModuleDisabled('listings_booking', !(bool) $new_booking_config['use']);
                $booking_config['use'] = $new_booking_config['use'];
            }
            $this->view->assign('booking_config', $booking_config);
        }
        if ($this->pg_module->is_module_active('moderation')) {
            $this->load->model('Listings_model');
            $new_moderation_config = $this->input->post('moderation_config');
            $moderation_config_items = [
                'use_moderation' => [
                    'key' => 'use_moderation',
                    'type' => 'checkbox',
                    'name' => l('settings_use_moderation', 'listings'),
                    'value' => isset($new_moderation_config['use_moderation']) ? $new_moderation_config['use_moderation'] : null,
                ],
                'moderation_send_mail' => [
                    'key' => 'moderation_send_mail',
                    'type' => 'checkbox',
                    'name' => l('settings_moderation_send_mail', 'listings'),
                    'value' => isset($new_moderation_config['use_moderation']) ? $new_moderation_config['use_moderation'] : null,
                ],
                'admin_moderation_emails' => [
                    'key' => 'admin_moderation_emails',
                    'type' => 'text',
                    'name' => l('settings_admin_moderation_emails', 'listings'),
                    'value' => isset($new_moderation_config['use_moderation']) ? $new_moderation_config['use_moderation'] : null,
                ],
            ];
            if (!empty($new_moderation_config)) {
                $this->Listings_model->setModuleConfig($new_moderation_config);
                
                if ($this->pg_module->is_module_active('upload_gallery')) {
                    $this->load->model('Upload_gallery_model');
                    $photo_type = $this->Upload_gallery_model->get_type_by_gid('listings-photo');
                    if (!empty($photo_type)) {
                        $this->Upload_gallery_model->set_type($photo_type['id'], ['use_moderation' => $moderation_config_items['use_moderation']['value']]);
                    }
                }
            }
            $this->view->assign('listings_moderation_config', $this->Listings_model->getModuleConfig($moderation_config_items));
        }
        return true;
    }

}
