<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('LISTINGS_RESIDENTIAL_SALE_TABLE', DB_PREFIX . 'listings_residential');
define('LISTINGS_COMMERCIAL_SALE_TABLE', DB_PREFIX . 'listings_commercial');
define('LISTINGS_LOT_AND_LAND_SALE_TABLE', DB_PREFIX . 'listings_lot_and_land');
define('LISTINGS_RESIDENTIAL_BUY_TABLE', DB_PREFIX . 'listings_residential');
define('LISTINGS_COMMERCIAL_BUY_TABLE', DB_PREFIX . 'listings_commercial');
define('LISTINGS_LOT_AND_LAND_BUY_TABLE', DB_PREFIX . 'listings_lot_and_land');
define('LISTINGS_RESIDENTIAL_RENT_TABLE', DB_PREFIX . 'listings_residential');
define('LISTINGS_COMMERCIAL_RENT_TABLE', DB_PREFIX . 'listings_commercial');
define('LISTINGS_LOT_AND_LAND_RENT_TABLE', DB_PREFIX . 'listings_lot_and_land');
define('LISTINGS_RESIDENTIAL_LEASE_TABLE', DB_PREFIX . 'listings_residential');
define('LISTINGS_COMMERCIAL_LEASE_TABLE', DB_PREFIX . 'listings_commercial');
define('LISTINGS_LOT_AND_LAND_LEASE_TABLE', DB_PREFIX . 'listings_lot_and_land');
define('LOT_AND_LAND', DB_PREFIX . 'listings_lot_and_land');
define('COMMERCIAL', DB_PREFIX . 'listings_commercial');
define('RESIDENTIAL', DB_PREFIX . 'listings_residential');

/**
 * Listings Field Editor Model
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_field_editor_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;

    /**
     * Fields from field editor by types
     * 
     * @var array
     */
    public $field_editor_fields = array();

    /**
     * Sections from field editor by types
     * 
     * @var array
     */
    public $field_editor_sections = array();
    public $listing_types = array('lot_and_land', 'residential', 'commercial');

    /**
     * Constructor
     *
     * return Listings_field_editor_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return data of fields from field editor
     * 
     * @param string $property_type_gid field editor type GUID
     * @return array
     */
    public function get_fields_for_select($property_type_gid)
    {
        if (!isset($this->field_editor_fields[$property_type_gid])) {
            $this->ci->load->model('Field_editor_model');
            $this->ci->Field_editor_model->initialize($property_type_gid);
            $property_type_sections = $this->ci->Field_editor_model->get_section_list();
            $property_type_sections_gids = array_keys($property_type_sections);
            if (!empty($property_type_sections_gids)) {
                $property_type_fields = $this->ci->Field_editor_model->get_fields_for_select($property_type_sections_gids, array());
            } else {
                $property_type_fields = array();
            }
            $this->field_editor_fields[$property_type_gid] = $property_type_fields;
            $this->field_editor_sections[$property_type_gid] = $property_type_sections_gids;
        }
        return $this->field_editor_fields[$property_type_gid];
    }

    /**
     * Return data of fields from field editor (internal)
     * 
     * @param string $property_type_gid property type GUID
     * @param array $fields_for_select fields from field editor
     * @param array $listing_ids listings identificators
     * @return array
     */
    public function get_all_fields_listing()
    {
        foreach ($this->listing_types as $key => $value) {
            $this->ci->db->select();
            $this->ci->db->from(constant(strtoupper($value)));
            $listings_fields[$value] = $this->ci->db->get()->result_array();
        }
        return $listings_fields;
    }

    public function get_field_editor_data($property_type_gid, $fields_for_select, $listing_ids)
    {
        array_unshift($fields_for_select, 'id_listing');
        $this->ci->db->select(implode(',', $fields_for_select))
            ->from(constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE'))
            ->where_in('id_listing', (array) $listing_ids);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[$r['id_listing']] = $r;
                unset($data[$r['id_listing']]['id_listing']);
            }
            return $data;
        }
        return array();
    }

    /**
     * Return field editor types as array
     * 
     * @return array
     */
    public function get_field_editor_types()
    {
        $operation_types = $this->ci->Listings_model->get_operation_types();

        $field_editor_types = array();
        foreach ($operation_types as $operation_type) {
            $field_editor_types = array_merge($field_editor_types, $this->ci->Listings_model->field_editor_types[$operation_type]);
        }

        return array_unique($field_editor_types);
    }

    /**
     * Remove field editor data of listings from data source
     * 
     * @param integer $listing_id listing identifier
     * @param string $property_type_gid property type guid
     * @return void
     */
    public function delete_listing($listing_id, $property_type_gid)
    {
        $this->ci->db->where('id_listing', $listing_id);
        $this->ci->db->delete(constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE'));
    }

    /**
     * Return search criteria of field editor data
     * 
     * @param array $filters filters data
     * @return array
     */
    public function get_search_criteria($filters)
    {
        $boolean_mode = isset($filters['keyword_mode']);

        $filters['keyword'] = trim($filters['keyword']);

        $this->ci->load->model('Field_editor_model');

        $fe_criteria = array();
        $property_type_gids = $this->get_field_editor_types();
        foreach ($property_type_gids as $property_type_gid) {
            $this->ci->Field_editor_model->initialize($property_type_gid);
            $fe_criteria_type = array();
            $temp_criteria = $this->ci->Field_editor_model->return_fulltext_criteria($filters['keyword'], $boolean_mode);
            $field_editor_table = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
            $fe_table_name = substr($field_editor_table, strlen(DB_PREFIX));
            if (!empty($temp_criteria[$fe_table_name]['where_sql'])) {
                $fe_criteria[] = str_replace('search_field', $field_editor_table . '.search_field', $temp_criteria[$fe_table_name]['where_sql']);
            }
        }

        return $fe_criteria;
    }

    public function get_search_tables($filter_data)
    {
        foreach ($filter_data as $key => $property_type_gid) {
            $filter_data[$key] = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
        }
        return array_unique($filter_data);
    }

    /**
     * Return listings identifiers from field_editor
     * 
     * @param string $property_type_gid property type GUID
     * @param string $params search criteria
     * @return array
     */
    public function get_listings_ids_from_field_editor($property_type_gid, $params)
    {
        $this->ci->db->select('id_listing');
        $this->ci->db->from(constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE'));

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r['id_listing'];
            }
            return $data;
        }
        return array();
    }

    /**
     * Create field editor data of listing
     * 
     * @param integer $listing_id listing identifier
     * @param array $fields_data fields data
     * @param string $property_type_gid property type guid
     * @return void
     */
    public function create_listing($listing_id, $fields_data, $property_type_gid)
    {
        $fields_data['id_listing'] = $listing_id;
        $this->ci->db->insert(constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE'), $fields_data);
    }

    /**
     * Save field editor data of listing
     * 
     * @param integer $listing_id listing identifier
     * @param array $fields_data fields data
     * @param string $property_type_gid property type guid
     * @return void
     */
    public function save_listing($listing_id, $fields_data, $property_type_gid)
    {
        $this->ci->db->where('id_listing', $listing_id);
        $this->ci->db->update(constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE'), $fields_data);
    }

}
