<?php

namespace Pg\Modules\Listings_booking\Controllers;

use Pg\Modules\Listings_booking\Models\Listings_booking_model;
use Pg\Libraries\View;

/**
 * Listings booking module
 *
 * @package     PG_RealEstate
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings booking user side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Listings booking
 * @category    controller
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_booking extends \Controller
{

    /**
     * Constructor
     *
     * @return Listings booking
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
    }

    /**
     * Bookings requests to my listings
     *
     * @param string $status order status
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function orders($status = 'wait', $order = 'created', $order_direction = 'DESC', $page = 1)
    {
        $data = $this->input->post('data', true);
        $block = $this->ordersBlock($status, $data, $order, $order_direction, $page);
        $this->view->assign('block', $block);
        $this->view->assign('status', $status);
        $user_id = $this->session->userdata('user_id');
        $this->view->assign('user_id', $user_id);

        $user_type = $this->session->userdata('user_type');
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_booking_item');

        $current_settings = isset($_SESSION['listings_booking']) ? $_SESSION['listings_booking'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'created';
        }
        $this->view->assign('order', $current_settings['order']);

        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);

        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);

        if (isset($current_settings['filters']['listings'])) {
            $this->view->assign('selected', $current_settings['filters']['listings']);
        }

        $this->load->model('listings_booking/models/Listings_booking_model');
        $statuses = [
            Listings_booking_model::STATUS_WAIT,
            Listings_booking_model::STATUS_APPROVE,
            Listings_booking_model::STATUS_DECLINE,
            Listings_booking_model::STATUS_BLACKLISTED
        ];
        $orders_count = $this->Listings_booking_model->getPeriodsCounts($statuses, $user_id);

        $this->view->assign('orders_count', $orders_count);
        $this->view->assign('orders_count_sum', array_sum($orders_count));

        $this->view->render('my_orders');
    }

    /**
     * Booking requests to my listings by ajax
     *
     * @param string $status order status
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     */
    public function ajax_orders($status = 'wait', $order = 'created', $order_direction = 'DESC', $page = 1)
    {
        $data = $this->input->post('data', true);
        $this->view->output($this->ordersBlock($status, $data, $order, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Block of booking requests to my listings
     *
     * @param string $status order status
     * @param array $data filter data
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    protected function ordersBlock($status = 'wait', $data = array(), $order = 'created', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_booking']) ? $_SESSION['listings_booking'] : array();
        if (!isset($current_settings['data'])) {
            $current_settings['data'] = array();
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'created';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = '1';
        }
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array();
        }
        if (!isset($current_settings['filters']['status'])) {
            $current_settings['filters']['status'] = 'wait';
        }

        $user_id = $this->session->userdata('user_id');

        $this->load->model('listings_booking/models/Listings_booking_model');

        if (!empty($data)) {
            $current_settings['filters'] = $data;
        }
        if (!isset($current_settings['filters']['status'])) {
            $current_settings['filters']['status'] = 'wait';
        }
        if ($status) {
            if ($status != $current_settings['filters']['status']) {
                $page = 1;
            }
            $current_settings['filters']['status'] = $status;
        }

        $filters = $current_settings['filters'];

        $filters['not_user'] = $user_id;
        $filters['owners'] = $user_id;

        $order = trim(strip_tags($order));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];
        $this->view->assign('order', $order);

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        $page = intval($page);
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];
        $this->view->assign('page', $page);

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');
        $orders_count = $this->Listings_booking_model->getPeriodsCount($filters);
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $orders_count, $items_on_page);
        $_SESSION['listings_booking'] = $current_settings;

        // sorting
        $sort_data = array(
            'url' => site_url() . 'listings_booking/orders/',
            'order' => $current_settings['order'],
            'direction' => $current_settings['order_direction'],
            'links' => array('created' => l('field_date_created', 'listings')),
        );
        $this->view->assign('sort_data', $sort_data);

        if ($orders_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'created':
                    $order_array['date_created'] = $order_direction;
                    break;
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }
            $this->Listings_booking_model->set_format_settings('get_listing', true);
            $orders = $this->Listings_booking_model->get_periods_list(
                $filters, $current_settings['page'], $items_on_page, $order_array
            );
            $this->Listings_booking_model->set_format_settings('get_listing', false);
            $this->view->assign('orders', $orders);
        }
        $this->view->assign('orders_count', $orders_count);
        $this->load->helper('navigation');
        $url = site_url() . 'listings_booking/orders/' . $order . '/' . $order_direction . '/';

        $page_data = get_user_pages_data($url, $orders_count, $items_on_page, $current_settings['page'], 'briefPage');

        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        return $this->view->fetch('my_orders_block', 'user', 'listings_booking');
    }

    /**
     * My bookings requests
     *
     * Available status values: open, book, wait, decline, close
     *
     * @param string $status request status
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function requests($status = 'wait', $order = 'created', $order_direction = 'DESC', $page = 1)
    {
        $data = $this->input->post('data', true);

        $block = $this->_requests_block($status, $data, $order, $order_direction, $page);
        $this->view->assign('block', $block);

        $this->view->assign('status', $status);

        $user_id = $this->session->userdata('user_id');
        $this->view->assign('user_id', $user_id);

        $user_type = $this->session->userdata('user_type');
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_requests_item');

        $current_settings = isset($_SESSION['listings_requests']) ? $_SESSION['listings_requests'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'created';
        }
        $this->view->assign('order', $current_settings['order']);

        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);

        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);

        $statuses = [
            Listings_booking_model::STATUS_WAIT,
            Listings_booking_model::STATUS_APPROVE,
            Listings_booking_model::STATUS_DECLINE,
            Listings_booking_model::STATUS_BLACKLISTED
        ];
        $requests_count = $this->Listings_booking_model->getRequestsCounts($statuses, $user_id);

        $this->view->assign('requests_count', $requests_count);
        $this->view->assign('requests_count_sum', array_sum($requests_count));

        $this->load->model('menu/models/Indicators_model');
        $this->Indicators_model->delete('booking_status_item', $user_id, true);
        $this->Indicators_model->delete('all_booking_item', 'user_' . $user_id, true);

        $this->view->render('my_requests');
    }

    /**
     * My booking requests by ajax
     *
     * Available status values: open, book, wait, decline, close
     *
     * @param string $status request status
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function ajax_requests($status = 'wait', $order = 'created', $order_direction = 'DESC', $page = 1)
    {
        $data = $this->input->post('data', true);
        $this->view->output($this->_requests_block($status, $data, $order, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Block of my booking requests
     *
     * Available status values: open, book, wait, decline, close
     *
     * @param string $status request status
     * @param array $data filter data
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    protected function _requests_block($status = 'wait', $data = array(), $order = 'created', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_requests']) ? $_SESSION['listings_requests'] : array();
        if (!isset($current_settings['data'])) {
            $current_settings['data'] = array();
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'created';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = '1';
        }
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array('status' => 'approve');
        }
        $this->load->model('listings_booking/models/Listings_booking_model');

        if (!empty($data)) {
            $current_settings['filters'] = $data;
        }
        if ($status) {
            if ($status != $current_settings['filters']['status']) {
                $page = 1;
            }
            $current_settings['filters']['status'] = $status;
        }
        $filters = $current_settings['filters'];
        $user_id = $this->session->userdata('user_id');

        $filters['users'] = $user_id;
        $filters['not_owner'] = $user_id;

        $order = trim(strip_tags($order));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];
        $this->view->assign('order', $order);

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        $page = intval($page);
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];
        $this->view->assign('page', $page);

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');
        $requests_count = $this->Listings_booking_model->getPeriodsCount($filters);
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $requests_count, $items_on_page);
        $_SESSION['listings_requests'] = $current_settings;

        // sorting
        $sort_data = array(
            'url' => site_url() . 'listings_booking/requests/',
            'order' => $current_settings['order'],
            'direction' => $current_settings['order_direction'],
            'links' => array('created' => l('field_date_created', 'listings')),
        );
        $this->view->assign('sort_data', $sort_data);

        if ($requests_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }
            $this->Listings_booking_model->set_format_settings('get_listing', true);
            $requests = $this->Listings_booking_model->get_periods_list($filters, $current_settings['page'], $items_on_page, $order_array);
            $this->Listings_booking_model->set_format_settings('get_listing', false);
            $this->view->assign('requests', $requests);
        }
        $this->view->assign('requests_count', $requests_count);
        $this->load->helper('navigation');

        $url = site_url() . 'listings_booking/requests/' . $order . '/' . $order_direction . '/';
        $page_data = get_user_pages_data($url, $requests_count, $items_on_page, $current_settings['page'], 'briefPage');

        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        return $this->view->fetch('my_requests_block', 'user', 'listings_booking');
    }

    /**
     * Remove booking request to my listing
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function order_delete($order_id)
    {
        $user_id = $this->session->userdata('user_id');
        $this->load->model('listings_booking/models/Listings_booking_model');
        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if (!$order_id || $user_id != $order['listing']['id_user']) {
            show_404();
            return;
        }

        $this->Listings_booking_model->delete_period($order_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_deleted', 'listings_booking'));
        $this->view->setRedirect(site_url() . 'listings_booking/orders');
    }

    /**
     * Remove booking request to my listing by ajax
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_order_delete($order_id)
    {
        $return = array(View::MSG_ERROR => '', View::MSG_SUCCESS => '');
        $user_id = $this->session->userdata('user_id');
        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($user_id != $order['listing']['id_user']) {
            $return[View::MSG_ERROR] = l('error_invalid_order', 'listings_booking');
        } else {
            $this->Listings_booking_model->delete_period($order_id);
            $return[View::MSG_SUCCESS] = l('success_order_deleted', 'listings_booking');
        }

        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Remove my booking request
     *
     * @param integer $request_id request identifier
     * @return void
     */
    public function request_delete($request_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $request = $this->Listings_booking_model->get_period_by_id($request_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if (!$request_id || $user_id != $request['id_user']) {
            show_404();
            return;
        }

        $this->Listings_booking_model->delete_period($request_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_request_deleted', 'listings_booking'));
        $this->view->setRedirect(site_url() . 'listings_booking/requests');
    }

    /**
     * Remove my booking request by ajax
     *
     * @param integer $request_id request identifier
     * @return void
     */
    public function ajax_request_delete($request_id)
    {
        $return = array(View::MSG_ERROR => array(), View::MSG_SUCCESS => '');
        $user_id = $this->session->userdata('user_id');
        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $request = $this->Listings_booking_model->get_period_by_id($request_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if (!$request_id || $user_id != $request['id_user']) {
            $return[View::MSG_ERROR] = l('error_invalid_order', 'listings_booking');
        } else {
            $this->Listings_booking_model->delete_period($request_id);
            $return[View::MSG_SUCCESS] = l('success_request_deleted', 'listings_booking');
        }

        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Approve booking request to my listing
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function order_approve($order_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if (!$order || $user_id != $order['listing']['id_user']) {
            show_404();
            return;
        }

        $is_intersect = $this->Listings_booking_model->is_intersect_exists_orders($listing_id, 0, $order['date_start'], $order['date_end'], $order_id);
        if ($is_intersect) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_intersect_calendar_periods', 'listings_booking'));
        } else {
            $this->Listings_booking_model->approve_period($order_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_approved', 'listings_booking'));
        }
        $this->view->setRedirect(site_url() . 'listings_booking/orders');
    }

    public function ajax_get_periods()
    {
        $params = $this->input->post('params', true);
        $order_by = $this->input->post('order_by', true);
        $this->load->model('listings_booking/models/Listings_booking_model');
        $return['periods'] = $this->Listings_booking_model->get_periods_list_by_key($params, null, null, $order_by);
        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Approve booking request to my listing by ajax
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_order_approve($order_id)
    {
        $order_id = intval($order_id);
        $return = array(View::MSG_ERROR => array(), View::MSG_SUCCESS => '');
        $user_id = $this->session->userdata('user_id');

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($order['id_user'] == '0') {
            $post_data = $this->input->post('period');
            $post_data['status'] = Listings_booking_model::STATUS_APPROVE;
            $validate_data = $this->Listings_booking_model->validate_period($order_id, $post_data);
            $this->Listings_booking_model->save_period($order_id, $validate_data['data']);
            $this->Listings_booking_model->approve_guest_period($order_id);
            $return[View::MSG_SUCCESS] = l('success_order_approved', 'listings_booking');
        } else {
            if (!$order || $user_id != $order['listing']['id_user']) {
                $return[View::MSG_ERROR] = l('error_invalid_order', 'listings_booking');
            } else {
                $post_data = $this->input->post('period');
                $validate_data = $this->Listings_booking_model->validate_period($order_id, $post_data);
                $is_intersect = $this->Listings_booking_model->is_intersect_exists_periods($order['id_listing'], 0, $order['date_start'], $order['date_end'], $order_id);
                if ($is_intersect) {
                    $validate_data['errors'][] = l('error_intersect_calendar_periods', 'listings_booking');
                }
                if (!empty($validate_data['errors'])) {
                    $return[View::MSG_ERROR] = implode('<br>', $validate_data['errors']);
                } else {
                    if (!empty($validate_data['data'])) {
                        $this->Listings_booking_model->save_period($order_id, $validate_data['data']);
                    }
                    $this->Listings_booking_model->approve_period($order_id);
                    $return[View::MSG_SUCCESS] = l('success_order_approved', 'listings_booking');
                }
            }
        }
        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Add email from guest booking request to blacklist
     *
     * @param type $period_id
     * @return type
     */
    public function add_to_blacklist($period_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');

        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        if ($period['id_owner'] == $this->session->userdata('user_id')) {
            if ($this->Listings_booking_model->toBlacklist($period_id)) {
                $this->system_messages->add_message(
                    View::MSG_SUCCESS, l('success_request_blacklisted', 'listings_booking')
                );
            } else {
                // Unknown error
            }
        } else {
            // Not owner
            $this->system_messages->add_message(
                View::MSG_SUCCESS, l('error_403_forbidden', 'start')
            );
        }
        $this->view->setRedirect(site_url() . 'listings_booking/orders/');
    }

    /**
     * Cancel booking
     *
     * @param type $period_id
     * @param type $confirmation_code
     */
    public function guest_cancel_booking($period_id, $confirmation_code)
    {
        $period_id = intval($period_id);

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($period['confirmation_code'] == $confirmation_code) {
            $this->Listings_booking_model->delete_period($period_id);
            // дописать нотифай собственнику
        }
    }

    /**
     * Decline booking request to my listing
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function order_decline($order_id)
    {
        $user_id = $this->session->userdata('user_id');
        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if (!$order_id || $user_id != $order['listing']['id_user']) {
            show_404();
            return;
        }

        $this->Listings_booking_model->decline_period($order_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_declined', 'listings_booking'));

        $this->view->setRedirect(site_url() . 'listings_booking/orders');
    }

    /**
     * Decline booking request to my listing by ajax
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_order_decline($order_id)
    {
        $order_id = intval($order_id);
        $return = array(View::MSG_ERROR => array(), View::MSG_SUCCESS => '');
        $user_id = $this->session->userdata('user_id');

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        $post_data = $this->input->post('period');

        if ($order['id_user'] == '0') {
            $post_data['status'] = 'decline';
            $validate_data = $this->Listings_booking_model->validate_period($order_id, $post_data);
            $this->Listings_booking_model->save_period($order_id, $validate_data['data']);
            $this->Listings_booking_model->decline_guest_period($order_id);
            $return[View::MSG_SUCCESS] = l('success_order_declined', 'listings_booking');
        } else {
            if (!$order_id || $user_id != $order['listing']['id_user']) {
                $return[View::MSG_ERROR] = l('error_invalid_order', 'listings_booking');
            } else {
                if ($order['status'] == 'guest_wait') {
                    $this->Listings_booking_model->decline_guest_period($order_id);
                } else {
                    $this->Listings_booking_model->decline_period($order_id);
                }
                $return[View::MSG_SUCCESS] = l('success_order_declined', 'listings_booking');
            }
        }
        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Answer form on booking request to my listing by ajax
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_order_decline_form($order_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');
        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);
        $this->view->assign('view_type', 'decline');
        $this->view->assign('order', $order);
        $this->view->output($this->view->fetch('ajax_order_approve_form', 'user', 'listings_booking'));
        $this->view->render();
    }

    /**
     * Answer form on booking request to my listing by ajax
     *
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_order_approve_form($order_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');
        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        if (!empty($order)) {
            $is_intersect_exists_orders = $this->Listings_booking_model->is_intersect_exists_orders($order['id_listing'], $order['id_user'], $order['date_start'], $order['date_end'], $order['id']);
            if ($is_intersect_exists_orders !== false) {
                $this->view->assign('is_intersect_exists_orders', true);
            }
        }
        $this->Listings_booking_model->set_format_settings('get_listing', false);
        $this->view->assign('order', $order);
        $this->view->output($this->view->fetch('ajax_order_approve_form', 'user', 'listings_booking'));
        $this->view->render();
    }

    /**
     * Booking request form to my listing by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_order_form($listing_id, $order_id = null, $edit = true)
    {
        $user_id = $this->session->userdata('user_id');

        $this->load->model('Listings_model');
        $this->Listings_model->applyFormatSettings(['use_format' => true, 'get_photos' => true]);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->revertFormatSettings();

        if (!$listing) {
            return '';
        }

        $form_id = $this->input->post('form_id', true);
        if ($form_id) {
            $this->view->assign('formId', $form_id);
        }
        if ($user_id == $listing['id_user'] && $edit == true) {
            $this->view->assign('my_listing_edit', '1');
        }

        if ($order_id) {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $this->Listings_booking_model->set_format_settings('use_format', true);
            $order = $this->Listings_booking_model->get_period_by_id($order_id);
            $this->view->assign('order', $order);
        }

        if ($this->pg_module->is_module_active('payments')) {
            $this->load->model('Currencies_model');
            $currencies = $this->Currencies_model->get_currency_list();
            $this->view->assign('currencies', $currencies);
            if (isset($listing['gid_currency']) && !empty($listing['gid_currency'])) {
                $current_price_currency = $this->Currencies_model->get_currency_by_gid($listing['gid_currency']);
                $this->view->assign('current_price_currency', $current_price_currency);
            }
        }

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                if ($order_id) {
                    $date_start = date('Y-m-d', strtotime($order['date_start']));
                    $date_end = date('Y-m-d', strtotime($order['date_end']));
                } else {
                    $time = time();
                    $date_start = date('Y-m-d', $time);
                    $date_end = date('Y-m-d', $time + (max($listing['calendar_period_min'], 1)) * 86400);
                }
                $this->view->assign('current_date_start', $date_start);
                $this->view->assign('current_date_end', $date_end);
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                if ($order_id) {
                    $start = strtotime($order['date_start']);
                    $month_start = date('m', $start);
                    $year_start = date('Y', $start);

                    $end = strtotime($order['date_end']);
                    $month_end = date('m', $end);
                    $year_end = date('Y', $end);
                } else {
                    $month_start = date('m');
                    $year_start = date('Y');

                    $month_end = ($month_start + max($listing['calendar_period_min'], 1)) % 12;
                    $year_end = $year_start + intval(($month_start + max($listing['calendar_period_min'], 1) - 2) / 12);

                    if (!$month_end) {
                        $month_end = 12;
                    }
                }
                
                $monthNames = ld('month_full', 'start');
                ksort($monthNames['option']);
                $this->view->assign('month_names', $monthNames);
                
                $this->view->assign('current_month_start', $month_start);
                $this->view->assign('current_year_start', $year_start);
                $this->view->assign('current_month_end', $month_end);
                $this->view->assign('current_year_end', $year_end);

                $date_start = $year_start . '-' . $month_start . '-01';
                $date_end = $year_end . '-' . $month_end . '-01';
                break;
        }

        $this->view->assign('auth_type', $this->session->userdata('auth_type'));
        $this->view->assign('is_guest', $this->session->userdata('auth_type') !== 'user');

        if ($listing['id_user'] != $user_id) {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $price_value = $this->Listings_booking_model->get_period_price($listing, $date_start, $date_end);
            $this->load->helper('start');
            $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
            $return['price'] = currency_format_output($price_arr);
            $this->view->assign('current_price', currency_format_output($price_arr));
            $this->view->assign('show_price', 1);
        }
        $page_data['min_date'] = date($this->pg_date->get_format('date_numeric', 'date'));
        $this->view->assign('listing', $listing);
        $this->view->assign('page_data', $page_data);
        $this->view->render('order_form');
    }

    /**
     * Save booking request to my listing by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $order_id order identifier
     * @return void
     */
    public function ajax_save_order($listing_id, $order_id = null)
    {
        $result = array(View::MSG_ERROR => array(), View::MSG_SUCCESS => array(), 'id' => 0, 'data' => '');

        $user_id = $this->session->userdata('user_id');

        $this->load->model('Listings_model');
        $listing = $this->Listings_model->get_listing_by_id($listing_id);

        $booking = $listing['id_user'] != $user_id;

        $this->load->model('listings_booking/models/Listings_booking_model');

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $user_id;
        if (!$booking) {
            $post_period['status'] = Listings_booking_model::STATUS_APPROVE;
        }

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                $date_start_alt = $this->input->post('date_start_alt', true);
                if ($date_start_alt) {
                    $post_period['date_start'] = $date_start_alt;
                }

                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_end_alt) {
                    $post_period['date_end'] = $date_end_alt;
                }
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $date_start_month = $this->input->post('date_start_month', true);
                $date_start_year = $this->input->post('date_start_year', true);
                $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';

                $date_end_month = $this->input->post('date_end_month', true);
                $date_end_year = $this->input->post('date_end_year', true);
                $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                break;
        }

        $validate_data = $this->Listings_booking_model->validate_period($order_id, $post_period, $booking);
        if (!empty($validate_data['errors'])) {
            $result[View::MSG_ERROR] = implode('<br>', $validate_data['errors']);
        } else {
            $guests = $validate_data['data']['guests'] ? $validate_data['data']['guests'] : 1;

            $validate_data['data']['price'] = $this->Listings_booking_model->get_period_price($listing, $validate_data['data']['date_start'], $validate_data['data']['date_end'], $guests);

            if ($order_id) {
                $result[View::MSG_SUCCESS] = l('success_order_updated', 'listings');
            } else {
                $result[View::MSG_SUCCESS] = l('success_order_added', 'listings');
            }

            $order_id = $this->Listings_booking_model->save_period($order_id, $validate_data['data'], $booking);

            $result['id'] = $validate_data['data']['id'] = $order_id;

            if (!$booking) {
                $this->view->assign('listing', $listing);

                $validate_data['data']['id_owner'] = $user_id;

                $period = $this->Listings_booking_model->format_period($validate_data['data']);
                $this->view->assign('period', $period);

                $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
                $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
                $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
                $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
                $this->view->assign('page_data', $page_data);

                $this->view->assign('edit', 1);
                if ($this->pg_module->is_module_active('payments')) {
                    $this->load->model('Currencies_model');
                    $current_price_currency = $this->Currencies_model->get_currency_default();
                }
                $this->view->assign('current_price_currency', $current_price_currency);

                $result['data'] = $this->view->fetch('calendar_period', 'user', 'listings_booking');
            }
            if ($booking) {
                $result['message_for_user'] = $this->view->fetch('booking_message', 'user', 'listings_booking');
            }
        }
        $this->view->assign($result);
        $this->view->render();
    }

    /**
     * Add period form of listing calendar by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $period_id period identifier
     * @return void
     */
    public function ajax_period_form($listing_id, $period_id = null)
    {
        $user_id = $this->session->userdata('user_id');

        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));

        if (!$listing || $listing['id_user'] != $user_id) {
            return '';
        }

        $form_id = $this->input->post('form_id', true);
        if ($form_id) {
            $this->view->assign('formId', $form_id);
        }

        $this->view->assign('data', $listing);

        if ($period_id) {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $this->Listings_booking_model->set_format_settings('get_user', false);
            $this->Listings_booking_model->set_format_settings('get_listing', false);
            $period = $this->Listings_booking_model->get_period_by_id($period_id);
            $this->view->assign('period', $period);
        }

        if ($this->pg_module->is_module_active('payments')) {
            $this->load->model('Currencies_model');
            $currencies = $this->Currencies_model->get_currency_list();
            $this->view->assign('currencies', $currencies);
            $this->view->assign('base_currency', $this->Currencies_model->get_currency_default(true));
            $this->view->assign('cur_currency', $this->Currencies_model->get_currency_default());
            if (isset($listing['gid_currency']) && !empty($listing['gid_currency'])) {
                $current_price_currency = $this->Currencies_model->get_currency_by_gid($listing['gid_currency']);
                $this->view->assign('current_price_currency', $current_price_currency);
            }
        }

        $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_numeric', 'ISO8601');
        $page_data['min_date'] = date($this->pg_date->get_format('date_numeric', 'date'));

        $this->view->assign('page_data', $page_data);
        $this->view->assign('rand', rand(100000, 999999));
        $this->view->render('ajax_period_form');
    }

    /**
     * Save listing calendar period by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $period_id period identifier
     * @return void
     */
    public function ajax_save_period($listing_id, $period_id = null, $is_not_return = false)
    {
        $result = array(View::MSG_ERROR => array(), View::MSG_SUCCESS => array(), 'id' => 0, 'data' => '', 'intersect_periods' => '');

        $user_id = $this->session->userdata('user_id');

        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (!$listing || $listing['id_user'] != $user_id) {
            return '';
        }

        $this->load->model('listings_booking/models/Listings_booking_model');
        if (!$is_not_return) {
            $post_period = $this->input->post('period', true);
        } else {
            $post_period = $_SESSION['period_price_data'];
            unset($_SESSION['period_price_data']);
        }
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $user_id;

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                $date_start_alt = $this->input->post('date_start_alt', true);
                if ($date_start_alt) {
                    $post_period['date_start'] = $date_start_alt;
                }

                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_end_alt) {
                    $post_period['date_end'] = $date_end_alt;
                }
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $date_start_month = $this->input->post('date_start_month', true);
                $date_start_year = $this->input->post('date_start_year', true);
                if ($date_start_year && $date_start_month) {
                    $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';
                }
                $date_end_month = $this->input->post('date_end_month', true);
                $date_end_year = $this->input->post('date_end_year', true);
                if ($date_end_year && $date_end_month) {
                    $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                }
                break;
        }

        $validate_data = $this->Listings_booking_model->validate_period($period_id, $post_period, false, false, true);
        if ($validate_data['intersect_periods']) {
            $result['intersect_periods'] = $validate_data['intersect_periods'];
            $result['data'] = $validate_data['data'];
        } else {
            if (!empty($validate_data['errors'])) {
                $result[View::MSG_ERROR] = implode('<br>', $validate_data['errors']);
            } else {
                if ($period_id) {
                    $result[View::MSG_SUCCESS] = l('success_period_updated', 'listings_booking');
                } else {
                    $result[View::MSG_SUCCESS] = l('success_period_added', 'listings_booking');
                }
                $period_id = $this->Listings_booking_model->save_period($period_id, $validate_data['data']);
                $result['id'] = $validate_data['data']['id'] = $period_id;
                $this->view->assign('listing', $listing);
                $validate_data['data']['id_owner'] = $user_id;
                $period = $this->Listings_booking_model->format_period($validate_data['data']);
                $this->view->assign('period', $period);
                $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
                $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
                $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
                $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
                $this->view->assign('page_data', $page_data);
                //for reload listing periods in calendar
                $this->view->assign('is_get_periods', true);
                $this->view->assign('edit', 1);

                if ($this->pg_module->is_module_active('payments')) {
                    $this->load->model('Currencies_model');
                    $current_price_currency = $this->Currencies_model->get_currency_default();
                }
                $this->view->assign('current_price_currency', $current_price_currency);
                $result['data'] = $this->view->fetch('calendar_period', 'user', 'listings_booking');
            }
        }
        if (!$is_not_return) {
            $this->view->assign($result);
            $this->view->render();
        } else {
            return $result;
        }
    }

    /**
     * Remove listing calendar period
     *
     * @param integer $period_id period identifier
     * @return void
     */
    public function period_delete($period_id)
    {
        $user_id = $this->session->userdata('user_id');

        if (!$period_id) {
            show_404();
            return;
        }

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('use_format', false);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        $this->Listings_booking_model->set_format_settings('use_format', true);

        if ($user_id != $period['id_user']) {
            show_404();
            return;
        }

        $this->Listings_booking_model->delete_period($period_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_period_deleted', 'listings_booking'));

        $this->view->setRedirect(site_url() . 'listings/edit/' . $period['id_listing'] . '/calendar/2');
    }

    /**
     * Remove listing calendar period by ajax
     *
     * @param integer $period_id period identifier
     * @return void
     */
    public function ajax_rewrite_periods()
    {
        $return = array('data' => array(), View::MSG_ERROR => '', View::MSG_SUCCESS => '');
        $return['data'] = $data = $this->input->post('data', true);
        
        if (isset($data['delete_periods']) && $data['delete_periods']) {
            foreach ($data['delete_periods'] as $value) {
                $result = $this->ajax_period_delete($value, true);
                if (!empty($result[View::MSG_ERROR])) {
                    $return[View::MSG_ERROR] = $result[View::MSG_ERROR];
                    $this->view->assign($return);
                    $this->view->render();
                    return;
                }
            }
        }

        $id_listing = $data['new_data']['id_listing'];

        if (isset($data['rewrite_periods']) && $data['rewrite_periods']) {
            $return['data']['rewrite_periods'] = array();
            foreach ($data['rewrite_periods'] as $key => $value) {
                $period_id = $value['id'];
                unset($value['id']);
                $_SESSION['period_price_data'] = $value;
                $result = $this->ajax_save_period($id_listing, $period_id, true);
                if (!empty($result[View::MSG_ERROR])) {
                    $return[View::MSG_ERROR] = $result[View::MSG_ERROR];
                    $this->view->assign($return);
                    $this->view->render();
                    return;
                } else {
                    array_push($return['data']['rewrite_periods'], $result);
                }
            }
        }

        if (isset($data['new_periods']) && $data['new_periods']) {
            $return['data']['new_periods'] = array();
            foreach ($data['new_periods'] as $key => $value) {
                $_SESSION['period_price_data'] = $value;
                $result = $this->ajax_save_period($id_listing, null, true);
                if (!empty($result[View::MSG_ERROR])) {
                    $return[View::MSG_ERROR] = $result[View::MSG_ERROR];
                    $this->view->assign($return);
                    $this->view->render();
                    return;
                } else {
                    array_push($return['data']['new_periods'], $result);
                }
            }
        }

        $_SESSION['period_price_data'] = $data['new_data'];
        $id = null;
        if (isset($data['new_data']['id']) && $data['new_data']['id']) {
            $id = $data['new_data']['id'];
        }
        $result = $this->ajax_save_period($id_listing, $id, true);
        $return['data']['new_data'] = array();
        if (!empty($result[View::MSG_ERROR])) {
            $return[View::MSG_ERROR] = $result[View::MSG_ERROR];
            $this->view->assign($return);
            $this->view->render();
            return;
        } else {
            $return['data']['new_data'] = $result;
        }

        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_period_delete($period_id, $is_not_return = false)
    {
        $return = array(View::MSG_ERROR => '', View::MSG_SUCCESS => '');

        $user_id = $this->session->userdata('user_id');

        if (!$period_id) {
            $return[View::MSG_ERROR] = 'Not access';
            $this->view->assign($return);
            $this->view->render();
            return;
        }

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('use_format', false);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        $this->Listings_booking_model->set_format_settings('use_format', true);


        if ($user_id != $period['id_user']) {
            if (!$is_not_return) {
                $return[View::MSG_ERROR] = 'Not access';

                $this->view->assign($return);

                $this->view->render();
                return;
            } else {
                return $return;
            }
        }

        $this->Listings_booking_model->delete_period($period_id);
        if (!$is_not_return) {
            $return[View::MSG_SUCCESS] = l('success_period_deleted', 'listings_booking');

            $this->view->assign($return);

            $this->view->render();
        } else {
            return $return;
        }
    }

    /**
     * Calculate price of booking period
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_get_booking_price($listing_id)
    {
        $return = array('price' => 0);

        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $date_start = $this->input->post('date_start', true);
        $date_start_alt = $this->input->post('date_start_alt', true);
        if (!empty($date_start_alt)) {
            $date_start = $date_start_alt;
        }

        $date_end = $this->input->post('date_end', true);
        $date_end_alt = $this->input->post('date_end_alt', true);
        if (!empty($date_end_alt)) {
            $date_end = $date_end_alt;
        }
        $guests = intval($this->input->post('guests', true));
        if (!$guests) {
            $guests = 1;
        }

        if (!$date_start || !$date_end) {
            if ($listing['price_negotiated']) {
                $return['price'] = l('text_negotiated_price_rent', 'listings_booking');
            } else {
                if ($listing['price_reduced'] > 0) {
                    $price_value = $listing['price_reduced'];
                } else {
                    $price_value = $listing['price'];
                }

                $this->load->helper('start');
                $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
                $return['price'] = l('text_price_from', 'listings_booking') . ' ' . currency_format_output($price_arr);
            }
        } else {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $price_value = $this->Listings_booking_model->get_period_price($listing, $date_start, $date_end, $guests);
            if ($price_value) {
                $this->load->helper('start');
                $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
                $return['price'] = currency_format_output($price_arr);
            } else {
                $return['price'] = l('text_negotiated_price_rent', 'listings_booking');
            }
        }

        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Guest booking order
     *
     * @param type $listing_id
     */
    public function ajax_send_order_guest($listing_id)
    {
        $return = array();

        $this->load->model('Listings_model');
        $listing = $this->Listings_model->get_listing_by_id($listing_id);

        $this->load->model('listings_booking/models/Listings_booking_model');

        $user_data = $this->input->post('guest_data', true);

        $validate_user = array();
        if ($user_data['email'] != $user_data['email_confirm']) {
            $validate_user['errors'][] = 'Емэйлы не совпадают';
        } else {
            $validate_user = $this->Listings_booking_model->validateGuest($user_data, $listing['id_user']);
        }

        if (!empty($validate_user['errors'])) {
            $return[View::MSG_ERROR] = implode('<br>', $validate_user['errors']);
        } else {

            $booking = true;

            $post_period = $this->input->post('period', true);
            $post_period['id_listing'] = $listing_id;
            $post_period['id_user'] = 0;

            switch ($listing['price_period']) {
                case Listings_booking_model::PRICE_PERIOD_DAY:
                    $date_start_alt = $this->input->post('date_start_alt', true);
                    if ($date_start_alt) {
                        $post_period['date_start'] = $date_start_alt;
                    }
                    $date_end_alt = $this->input->post('date_end_alt', true);
                    if ($date_end_alt) {
                        $post_period['date_end'] = $date_end_alt;
                    }
                    break;
                case Listings_booking_model::PRICE_PERIOD_MONTH:
                    $date_start_month = $this->input->post('date_start_month', true);
                    $date_start_year = $this->input->post('date_start_year', true);
                    $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';

                    $date_end_month = $this->input->post('date_end_month', true);
                    $date_end_year = $this->input->post('date_end_year', true);
                    $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                    break;
            }

            $validate_data = $this->Listings_booking_model->validate_period(null, $post_period, $booking, true);

            if (!empty($validate_data['errors'])) {
                $return[View::MSG_ERROR] = implode('<br>', $validate_data['errors']);
            } else {
                $guests = $validate_data['data']['guests'] ? $validate_data['data']['guests'] : 1;

                $validate_data['data']['price'] = $this->Listings_booking_model->get_period_price($listing, $validate_data['data']['date_start'], $validate_data['data']['date_end'], $guests);

                $return[View::MSG_SUCCESS] = l('success_order_added', 'listings');

                if ($this->Listings_booking_model->save_guest_period($validate_data['data'], $validate_user['data'])) {
                    $return[View::MSG_SUCCESS] = l('success_order_added_guest', 'listings_booking');
                } else {
                    $return[View::MSG_ERROR] = 'Undefined error';
                }
            }
        }

        exit(json_encode($return));
    }

    public function ajax_check_period($listing_id)
    {
        $this->load->model('Listings_model');
        $this->load->model('listings_booking/models/Listings_booking_model');

        $listing = $this->Listings_model->get_listing_by_id($listing_id);

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = 0;

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                $date_start_alt = $this->input->post('date_start_alt', true);
                if ($date_start_alt) {
                    $post_period['date_start'] = $date_start_alt;
                }
                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_end_alt) {
                    $post_period['date_end'] = $date_end_alt;
                }
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $date_start_month = $this->input->post('date_start_month', true);
                $date_start_year = $this->input->post('date_start_year', true);
                $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';

                $date_end_month = $this->input->post('date_end_month', true);
                $date_end_year = $this->input->post('date_end_year', true);
                $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                break;
        }

        $validate_data = $this->Listings_booking_model->validate_period(null, $post_period, true, true);

        if (empty($validate_data)) {
            $return[View::MSG_ERROR] = 'Undefined error';
        } elseif (!empty($validate_data['errors'])) {
            $return[View::MSG_ERROR] = implode('<br>', $validate_data['errors']);
        } else {
            $return[View::MSG_SUCCESS] = l('success_order_added', 'listings');
        }

        exit(json_encode($return));
    }

    public function guest_confirm_booking($period_id, $confirmation_code)
    {
        $site_url = site_url();

        $period_id = intval($period_id);
        $confirmation_code = trim(strip_tags($confirmation_code));
        $this->load->model('listings_booking/models/Listings_booking_model');

        $confirmation_status = $this->Listings_booking_model->confirm_guest_period($period_id, $confirmation_code);

        switch ($confirmation_status) {
            case 'confirm': {
                    $this->system_messages->add_message(View::MSG_SUCCESS, l('info_status_confirm', 'listings_booking'));
                    $this->view->setRedirect($site_url);
                    break;
                }
            case 'already_confirmed': {
                    $this->system_messages->add_message(View::MSG_SUCCESS, l('info_status_already_confirmed', 'listings_booking'));
                    $this->view->setRedirect($site_url);
                    break;
                }
            default: {
                    show_404();
                    return;
                }
        }
    }

    public function guest_order_view($period_id = null, $confirmation_code = '')
    {
        $period_id = intval($period_id);
        $confirmation_code = trim(strip_tags($confirmation_code));

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);

        if ($period['booking_data']['confirmation_code'] == $confirmation_code && $period['status'] != 'decline') {
            if ($this->input->post('save_guest_booking')) {
                $return = array();

                $input_period = $this->input->post('period', true);

                $post_data = array(
                    'date_start' => $input_period['date_start'],
                    'date_end' => $input_period['date_end'],
                    'guests' => $input_period['guests'],
                );

                $validate_data = $this->Listings_booking_model->validate_period($period_id, $post_data);

                if (!empty($validate_data['errors'])) {
                    $return[View::MSG_ERROR] = implode('<br>', $validate_data['errors']);
                } else {
                    if ($this->Listings_booking_model->is_data_changed($period_id, $period, $validate_data['data']) === true) {
                        $validate_data['data']['status'] = 'wait';
                        if ($this->Listings_booking_model->save_period($period_id, $validate_data['data'], true)) {
                            $return[View::MSG_SUCCESS] = $input_period;
                        }
                    } else {
                        $return['no_data_change'] = 1;
                    }
                }

                exit(json_encode($return));
            } else {
                $this->view->assign('period', $period);
                $this->view->render('guest_order_view');
            }
        } else {
            show_404();
        }
    }

    /* rename to booking_form */

    public function order_form($listing_id)
    {
        $input_period = $this->input->post('period', true);

        $this->load->model('listings/models/Listings_model');
        $this->load->model('field_editor/models/Field_editor_model');

        $this->Listings_model->set_format_settings(array('get_photos' => true));
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $property_type_gid = $listing['field_editor_type'];
        $this->Field_editor_model->initialize($property_type_gid);
        $property_type_sections = $this->Field_editor_model->get_section_list();

        $this->view->assign('sections_data', array_values($property_type_sections));
        $this->view->assign('sections_data_count', count($property_type_sections));

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                $date_start_alt = $this->input->post('date_start_alt', true);
                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_start_alt) {
                    $date_start_alt = strftime("%Y-%m-%d", strtotime($date_start_alt));
                } else {
                    $date_start_alt = strftime("%Y-%m-%d");
                }
                if ($date_end_alt) {
                    $date_end_alt = strftime("%Y-%m-%d", strtotime($date_end_alt));
                } else {
                    $date_end_alt = strftime("%Y-%m-%d", strtotime("+1 day"));
                }
                $this->view->assign('period_date', array(
                    'date_start' => $date_start_alt,
                    'date_end' => $date_end_alt
                    )
                );
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $date_start_month = intval($this->input->post('date_start_month', true));
                $date_start_year = intval($this->input->post('date_start_year', true));
                $date_end_month = intval($this->input->post('date_end_month', true));
                $date_end_year = intval($this->input->post('date_end_year', true));
                $this->view->assign('period_date', array(
                    'month_start' => $date_start_month,
                    'year_start' => $date_start_year,
                    'month_end' => $date_end_month,
                    'year_end' => $date_end_year,
                ));
                break;
        }

        $this->view->assign('guests', $input_period['guests']);

        if (!$listing) {
            show_404();
            return;
        }

        if ($this->pg_module->is_module_active("users")) {
            $this->load->model('Users_model');
            $this->Users_model->set_format_settings(array('get_contact' => true, 'get_company' => true, 'get_field_editor' => true, 'get_microdata' => true));
            $listing['user'] = $this->Users_model->get_user_by_id($listing['id_user']);
            $this->Users_model->set_format_settings(array('get_contact' => false, 'get_company' => false, 'get_field_editor' => false, 'get_microdata' => false));
        } else {
            $this->load->model('Start_model');
            $listing['user'] = $this->Start_model->getUserDefaultOnSite();
        }

        $this->view->assign('listing_data', $listing);

        $this->load->model('menu/models/Menu_model');
        $this->Menu_model->set_menu_active_item('guest_main_menu', 'guest-main-buyers-properties-item');
        $this->Menu_model->breadcrumbs_set_active($listing['output_name'], site_url() . 'listings/view/' . $listing['id']);
        $this->Menu_model->breadcrumbs_set_active(l('header_guest_booking', 'listings_booking'));
        $this->view->render('booking_form');
    }

}
