<?php

namespace Pg\Modules\Mobile\Controllers;

/**
 * Mobile module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Mobile version controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mobile
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Mobile extends \Controller
{

    /**
     * Class constructor
     *
     * @return Mobile
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Get index page data
     *
     * @return void
     */
    public function index()
    {
        
    }

    /**
     * Output mobile logo path
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function init($lang_id = '')
    {
        $theme_data = $this->pg_theme->format_theme_settings($this->router->class);
        exit($theme_data['mobile_logo']['path']);
    }

    /**
     * Authorize user by oauth
     *
     * @param string $service_id service identifier
     * @param string $redirect   redirect url
     *
     * @return void
     */
    public function oauth_login($service_id, $redirect = null)
    {
        if (!$this->pg_module->is_module_active('users_connections')) {
            show404();
        }

        $user_type = 0;

        if ($redirect) {
            $redirect = base64_decode($redirect);
        } else {
            $redirect = $this->input->get('redirect');
        }

        // Грузим модели
        $this->load->model('social_networking/models/Social_networking_services_model');
        $this->load->model('social_networking/models/Social_networking_connections_model');
        $this->load->model('Users_connections_model');

        // Данные
        $service = $this->Social_networking_services_model->get_service_by_id($service_id);

        // Проверка подключения
        if ($service['oauth_version'] == 2) {
            $result = $this->Social_networking_connections_model->_check_oauth2_connection($service, site_url('mobile/oauth_login/' . $service_id . '/' . base64_encode($redirect)));
        } else {
            $result = $this->Social_networking_connections_model->_check_oauth_connection($service, site_url('mobile/oauth_login/' . $service_id . '/' . base64_encode($redirect)));
        }

        // Авторизуем или посылаем на авторизацию
        if ($result['result']) {
            // Если получен ключ ответа
            if (isset($result['result']['oauth_token'])) {
                $result['result']['access_token'] = $result['result']['oauth_token'];
            }
            if (isset($result['result']['access_token'])) {
                $result['result']['access_secret'] = isset($result['result']['oauth_token_secret']) ? $result['result']['oauth_token_secret'] : '';
                $result['result']['expires_in'] = isset($result['result']['expires_in']) ? $result['result']['expires_in'] : 0;
                $user_id = $this->session->userdata('user_id');
                $service_user_id = isset($result['result']['user_id']) ? $result['result']['user_id'] : false;
                $service_user_fname = '';
                $service_user_sname = '';
                $service_user_email = '';
                $service_model = $service['gid'] . '_service_model';
                $service_file = APPPATH . 'modules/social_networking/models/services/' . $service_model . '.php';
                if (file_exists($service_file)) {
                    include_once $service_file;
                    $this->service = new $service_model();
                    if (method_exists($this->service, 'get_user_data')) {
                        $user_data = $this->service->get_user_data($service_user_id, $result['result']['access_token'], $result['result']['access_secret']);
                        if (($user_data) && isset($user_data['id'])) {
                            $service_user_id = $user_data['id'];
                            $service_user_fname = $user_data['fname'];
                            $service_user_sname = $user_data['sname'];
                            $service_user_email = $user_data['email'];
                        }
                    }
                }

                if ($service_user_id) {
                    $connection = $this->Users_connections_model->get_connection_by_data($service_id, $service_user_id);

                    if ($connection && isset($connection['id'])) {
                        $this->Users_connections_model->delete_connection($connection['id']);
                        $user_id = $connection['user_id'];

                        $this->load->model("users/models/Auth_model");
                        $auth_data = $this->Auth_model->login($user_id);
                        if (empty($auth_data["errors"])) {
                            $connection = array(
                                'service_id' => $service_id,
                                'user_id' => $user_id,
                                'access_token' => $result['result']['access_token'],
                                'access_token_secret' => $result['result']['access_secret'],
                                'data' => $service_user_id,
                                'date_end' => date("Y-m-d H:i:s", time() + $result['result']['expires_in']),
                            );
                            $this->Users_connections_model->save_connection(null, $connection);
                            $user_type = $auth_data['user_data']['user_type'];
                        }
                    }
                }
            }
        }

        $url = $redirect . '#!/account/oauth/' . $service_id . '/';

        if ($service_user_id) {
            $url .= $service_user_id;
        } else {
            $url .= 0;
        }

        $url .= '/';

        if ($user_type) {
            $url .= $user_type;
        } else {
            $url .= 0;
        }

        redirect($url);
    }

    /**
     * Register user by oauth
     *
     * @param string $service_id service identifier
     * @param string $user_type  user type
     * @param string $redirect   redirect url
     *
     * @return void
     */
    public function oauth_register($service_id, $user_type, $redirect = null)
    {
        if (!$this->pg_module->is_module_active('users_connections')) {
            show404();
        }

        if ($redirect) {
            $redirect = base64_decode($redirect);
        } else {
            $redirect = $this->input->get('redirect');
        }

        // Грузим модели
        $this->load->model('social_networking/models/Social_networking_services_model');
        $this->load->model('social_networking/models/Social_networking_connections_model');
        $this->load->model('Users_connections_model');
        // Данные
        $service = $this->Social_networking_services_model->get_service_by_id($service_id);

        // Проверка подключения
        if ($service['oauth_version'] == 2) {
            $result = $this->Social_networking_connections_model->_check_oauth2_connection($service, site_url('mobile/oauth_register/' . $service_id . '/' . $user_type . '/' . base64_encode($redirect)));
        } else {
            $result = $this->Social_networking_connections_model->_check_oauth_connection($service, site_url('mobile/oauth_register/' . $service_id . '/' . $user_type . '/' . base64_encode($redirect)));
        }

        // Авторизуем или посылаем на авторизацию
        if ($result['result']) {
            // Если получен ключ ответа
            if (isset($result['result']['oauth_token'])) {
                $result['result']['access_token'] = $result['result']['oauth_token'];
            }
            if (isset($result['result']['access_token'])) {
                $result['result']['access_secret'] = isset($result['result']['oauth_token_secret']) ? $result['result']['oauth_token_secret'] : '';
                $result['result']['expires_in'] = isset($result['result']['expires_in']) ? $result['result']['expires_in'] : 0;
                $user_id = $this->session->userdata('user_id');
                $service_user_id = isset($result['result']['user_id']) ? $result['result']['user_id'] : false;
                $service_user_fname = '';
                $service_user_sname = '';
                $service_user_email = '';
                $service_model = $service['gid'] . '_service_model';
                $service_file = APPPATH . 'modules/social_networking/models/services/' . $service_model . '.php';
                if (file_exists($service_file)) {
                    include_once $service_file;
                    $this->service = new $service_model();
                    if (method_exists($this->service, 'get_user_data')) {
                        $user_data = $this->service->get_user_data($service_user_id, $result['result']['access_token'], $result['result']['access_secret']);
                        if (($user_data) && isset($user_data['id'])) {
                            $service_user_id = $user_data['id'];
                            $service_user_fname = $user_data['fname'];
                            $service_user_sname = $user_data['sname'];
                            $service_user_email = $user_data['email'];
                        }
                    }
                }

                if ($service_user_id) {
                    $connection = $this->Users_connections_model->get_connection_by_data($service_id, $service_user_id);
                    if ($connection && isset($connection['id'])) {
                        $this->Users_connections_model->delete_connection($connection['id']);
                        $user_id = $connection['user_id'];

                        $this->load->model("users/models/Auth_model");
                        $auth_data = $this->Auth_model->login($user_id);
                        if (empty($auth_data["errors"])) {
                            $connection = array(
                                'service_id' => $service_id,
                                'user_id' => $user_id,
                                'access_token' => $result['result']['access_token'],
                                'access_token_secret' => $result['result']['access_secret'],
                                'data' => $service_user_id,
                                'date_end' => date("Y-m-d H:i:s", time() + $result['result']['expires_in']),
                            );
                            $this->Users_connections_model->save_connection(null, $connection);
                        }
                    }

                    $params = array();
                    $params["where"]["email"] = $service_user_email;
                    $this->load->model("Users_model");
                    $count = $this->Users_model->get_users_count($params);
                    if ($count > 0) {
                        $service_user_email = rand(100, 300) . $service_user_email;
                    }

                    $lang_id = $this->pg_language->current_lang_id;
                    $this->load->model('users/models/Users_groups_model');
                    $group_id = $this->Users_groups_model->get_default_group_id();

                    $user_data = array(
                        'fname' => $service_user_fname,
                        'sname' => $service_user_sname,
                        'email' => $service_user_email,
                        'confirm' => 1,
                        'user_type' => $user_type,
                        'status' => '1',
                        'lang_id' => $lang_id,
                        'group_id' => $group_id,
                    );

                    $user_data['user_type'] = $user_type;

                    if ($service_id && $result['result']['access_token'] && $service_user_id) {
                        // Грузим модели
                        $this->load->model('social_networking/models/Social_networking_services_model');
                        $this->load->model('users_connections/models/Users_connections_model');
                        // Данные
                        $service = $this->Social_networking_services_model->get_service_by_id($service_id);
                        if ($service) {
                            $user_id = $this->Users_model->save_user(null, $user_data);

                            $this->load->model("users/models/Auth_model");
                            $auth_data = $this->Auth_model->login($user_id);

                            if (!empty($auth_data["errors"])) {
                                //$this->system_messages->addMessage(View::MSG_ERROR, $auth_data["errors"]);
                                $service_user_id = 0;
                            } else {
                                $connection = array(
                                    'service_id' => $service_id,
                                    'user_id' => $user_id,
                                    'access_token' => $result['result']['access_token'],
                                    'access_token_secret' => $result['result']['access_secret'],
                                    'data' => $service_user_id,
                                    'date_end' => date("Y-m-d H:i:s", time() + $result['result']['expires_in']),
                                );
                                $this->Users_connections_model->save_connection(null, $connection);
                                //$this->system_messages->addMessage(View::MSG_SUCCESS, l('please_set_email', 'users_connections'));
                            }
                        }
                    }
                }
            }
        }

        redirect($redirect . '#!/account/oauth/' . $service_id . '/' . ($user_id ? $service_user_id : 0) . '/' . ($user_id ? $user_type : 0));
    }

    /**
     * Get data of mobile theme
     *
     * @param string $lang_code language code
     *
     * @return void
     */
    public function theme($lang_code = null)
    {
        $lang_id = $this->pg_language->get_lang_id_by_code($lang_code);
        if ($lang_id) {
            $this->pg_language->current_lang_id = $lang_id;
        }

        $theme = $this->pg_theme->format_theme_settings();
        $theme_data = $this->pg_theme->return_active_settings();

        $data = array(
            'logo_width' => $theme['mobile_logo']['width'],
            'logo_height' => $theme['mobile_logo']['height'],
            'logo_url' => site_url() . $theme['mobile_logo']['path'],
            'background' => $theme_data['user']['scheme_data']['color_settings']['main_bg'],
        );

        $this->view->assign($data);
        
        $this->view->render();
    }

    /**
     * Process backend request
     *
     * @return void
     */
    public function backend()
    {
        $data = array();

        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
        } elseif ($this->pg_module->is_module_active('users')) {
            $user_id = 0;

            $login_user_id = $this->input->post('user_id', true);
            $login_user_secret = $this->input->post('user_secret', true);
            if ($login_user_id && $login_user_secret) {
                $this->load->model('Mobile_model');
                $is_user_secret = $this->Mobile_model->is_user_secret($login_user_id, $login_user_secret);
                if ($is_user_secret) {
                    $this->load->model('Users_model');
                    $this->Users_model->set_format_settings('get_safe', false);
                    $user = $this->Users_model->get_user_by_id($login_user_id);
                    $this->Users_model->set_format_settings('get_safe', true);
                    if (!empty($user)) {
                        $this->load->model('users/models/Auth_model');
                        $login_return = $this->Auth_model->login_by_email_password($user['email'], $user['password']);
                        if (empty($login_return["errors"])) {
                            $user_id = $login_user_id;
                        }
                    }
                }
            }
        }

        if ($user_id) {
            if ($this->pg_module->is_module_active('Mailbox_model')) {
                $this->load->model('Mailbox_model');
                $data['mailbox']['new_messages_count'] = 0;
                $chats = $this->Mailbox_model->get_chats_list(array('where' => array('id_com_user' => $user_id)));
                foreach ($chats as $chat) {
                    $data['mailbox']['new_messages_count'] += $chat['new'];
                }
            }
        }

        $this->view->assign('data', $data);
        
        $this->view->render();
    }

    /**
     * Share listing by social networks
     *
     * @param integer $listing_id listing identifier
     *
     * @return void
     */
    public function social_share_listing($listing_id)
    {
        

        $this->load->helper('social_networking');
        show_social_networks_head();

        $this->load->model('Listings_model');
        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);

        $this->load->helper('seo');
        $seolink = rewrite_link('listings', 'view', $listing_data);

        $this->load->helper('start');
        $description = currency_format_output(array('value' => $listing_data['price'], 'cur_gid' => $listing_data['gid_currency']));

        $this->view->output("<html><head>
		<script>
			window.onload = function(){
				FB.ui({
					method: 'feed',
					redirect_uri: '" . $seolink . "',
					link: '" . $seolink . "',
					picture: '" . $listing_data['media']['photo']['thumbs']['small'] . "',
					name: '" . addslashes($listing_data['output_name']) . "',
					caption: '" . addslashes($listing_data['property_type_str']) . ' ' . addslashes($listing_data['operation_type_str']) . "',
					description: '" . addslashes(strip_tags($description)) . "'
				}, function(response){});
			}
		</script></head><body></body></html>");
        
        $this->view->render();
    }

}
