<?php

namespace Pg\Modules\Mobile\Models;

/**
 * Mobile module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('MOBILE_USERS_TABLE', DB_PREFIX . 'mobile_users');

/**
 * Mobile main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mobile
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Mobile_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    public $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    public $DB;

    /**
     * Available view modes of index block
     *
     * @var array
     */
    protected $index_view_arr = array('gallery', 'list', 'map');

    /**
     * Upload image (GUID) of applications page background
     *
     * @var string
     */
    protected $download_apps_background_config_gid = 'mobile-app-background';

    /**
     * Constructor
     *
     * @return Mobile_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;
    }

    /**
     * Return user secret
     *
     * @param integer $user_id user identifier
     *
     * @return
     */
    public function get_user_secret($user_id)
    {
        $this->DB->select('secret')->from(MOBILE_USERS_TABLE);
        $this->DB->where('id_user', $user_id);
        $result = $this->DB->get()->result_array();
        if (empty($result)) {
            return false;
        } else {
            return $result[0]['secret'];
        }
    }

    /**
     * Check user secret
     *
     * @param integer $user_id     user identifier
     * @param string  $user_secret user secret
     *
     * @return
     */
    public function is_user_secret($user_id, $user_secret)
    {
        $this->DB->select('COUNT(*) as count')->from(MOBILE_USERS_TABLE);
        $this->DB->where('id_user', $user_id);
        $this->DB->where('secret', $user_secret);
        $result = $this->DB->get()->result_array();
        if (empty($result)) {
            return false;
        } else {
            return $result[0]['count'] > 0;
        }
    }

    /**
     * Save user secret
     *
     * @param integer $user_id user identifier
     *
     * @return string
     */
    public function save_user_secret($user_id)
    {
        $secret = md5(date('Y-m-d H:i:s'));
        $data['last_visit_date'] = date('Y-m-d H:i:s');
        $this->DB->where('id_user', $user_id);
        $this->DB->update(MOBILE_USERS_TABLE, $data);
        $sql = "INSERT INTO " . MOBILE_USERS_TABLE . " (id_user, secret, last_visit_date) "
                . "VALUES (" . $this->DB->escape($user_id) . ", " . $this->DB->escape($secret) . ", " . $this->DB->escape(date('Y-m-d H:i:s')) . ") "
                . "ON DUPLICATE KEY UPDATE id_user=" . $this->DB->escape($user_id) . ", secret=" . $this->DB->escape($secret) . ", last_visit_date=" . $this->DB->escape(date('Y-m-d H:i:s'));
        $this->DB->query($sql);

        return $secret;
    }

    /**
     * Validate module settings
     *
     * @param string $data settings data
     *
     * @return array
     */
    public function validate_settings($data)
    {
        $return = array('errors' => array(), 'data' => array(), 'langs' => array());

        $this->CI->config->load("reg_exps", true);

        if (isset($data['index_type_block'])) {
            if (in_array($data['index_type_block'], array('featured', 'latest', 'rent', 'sale'))) {
                $return['data']['index_type_block'] = $data['index_type_block'];
            }
        }

        if (isset($data['index_view_block'])) {
            if (in_array($data['index_view_block'], $this->index_view_arr)) {
                $return['data']['index_view_block'] = $data['index_view_block'];
            }
        }

        if (isset($data['index_count_block'])) {
            $return['data']['index_count_block'] = intval($data['index_count_block']);
        }

        if (isset($data['geomap_driver'])) {
            if (in_array($data['geomap_driver'], array('google', 'yandex'))) {
                $return['data']['geomap_driver'] = $data['geomap_driver'];
            }
        }

        if (isset($data['download_apps_used'])) {
            $return['data']['download_apps_used'] = $data['download_apps_used'] ? 1 : 0;
        }

        if (isset($data['download_apps_android'])) {
            $return['data']['download_apps_android'] = trim($data['download_apps_android']);
            if (!empty($return['data']['download_apps_android'])) {
                $url_expr = $this->CI->config->item("url", "reg_exps");
                if (!preg_match($url_expr, $return["data"]["download_apps_android"])) {
                    $return['errors'][] = l('error_download_apps_android_invalid', 'mobile');
                }
            }
        }

        if (isset($data['download_apps_ios'])) {
            $return['data']['download_apps_ios'] = trim($data['download_apps_ios']);
            if (!empty($return['data']['download_apps_ios'])) {
                $url_expr = $this->CI->config->item("url", "reg_exps");
                if (!preg_match($url_expr, $return["data"]["download_apps_ios"])) {
                    $return['errors'][] = l('error_download_apps_ios_invalid', 'mobile');
                }
            }
        }

        if (isset($data['download_apps_title'])) {
            foreach ($this->CI->pg_language->languages as $lang_id => $lang_data) {
                if (isset($data['download_apps_title'][$lang_id])) {
                    $return['langs']['download_apps_title'][$lang_id] = trim(strip_tags($data['download_apps_title'][$lang_id]));
                }
            }
        }

        if (isset($data['download_apps_text'])) {
            foreach ($this->CI->pg_language->languages as $lang_id => $lang_data) {
                if (isset($data['download_apps_text'][$lang_id])) {
                    $return['langs']['download_apps_text'][$lang_id] = trim(strip_tags($data['download_apps_text'][$lang_id]));
                }
            }
        }

        return $return;
    }

    /**
     * Remove upload background of applications page
     *
     * @return array
     */
    public function delete_download_apps_background()
    {
        $download_apps_background = $this->CI->pg_module->get_module_config('mobile', 'download_apps_background');

        if (empty($download_apps_background)) {
            return;
        }

        $this->CI->load->model('Uploads_model');
        $this->CI->Uploads_model->delete_upload($this->download_apps_background_config_gid, '', $download_apps_background);

        $this->CI->pg_module->set_module_config('mobile', 'download_apps_background', '');
    }

    /**
     * Validate upload background of applications page
     *
     * @param string $file_name file name
     *
     * @return array
     */
    public function validate_download_apps_background($file_name)
    {
        $this->CI->load->model('Uploads_model');
        $result = $this->CI->Uploads_model->validate_upload($this->download_apps_background_config_gid, $file_name);

        return $result;
    }

    /**
     * Upload background of applications page
     *
     * @param string $file_name file name
     *
     * @return array
     */
    public function upload_download_apps_background($file_name)
    {
        $this->CI->load->model('Uploads_model');
        $result = $this->CI->Uploads_model->upload($this->download_apps_background_config_gid, '', $file_name);
        if ($result['file']) {
            $this->CI->pg_module->set_module_config('mobile', 'download_apps_background', $result['file']);
        }

        return $result;
    }

    /**
     * Format mobile settings
     *
     * @param array $data page data
     *
     * @return array
     */
    public function format_settings($data)
    {
        if (!empty($data['download_apps_background'])) {
            $this->CI->load->model('Uploads_model');
            $result = $this->CI->Uploads_model->format_upload($this->download_apps_background_config_gid, '', $data['download_apps_background']);
            $data['media']['download_apps_background'] = $result;
        }

        return $data;
    }

}
