<?php

namespace Pg\Modules\Moderation\Controllers;

/**
 * Moderation module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Moderation admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Moderation
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Moderation extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Moderation
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Moderation_model');
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'moderation-items');
        $this->view->setHeader(l('admin_header_moderation', 'moderation'));
    }

    /**
     * Manage moderation objects
     *
     * @param string  $type_name type name
     * @param integer $page      page of results
     *
     * @return void
     */
    public function index($type_name = "all", $page = 1)
    {
        $page = intval($page) ? intval($page) : 1;
        $type_name = (empty($type_name)) ? "all" : $type_name;
        $moder_type_name = (!empty($type_name) && $type_name != 'all') ? $type_name : "";

        $count = $this->Moderation_model->get_moderation_list_count($moder_type_name);
        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $count, $items_on_page);

        $_SESSION["mobjects_list"]["type_name"] = $type_name;
        $_SESSION["mobjects_list"]["page"] = $page;

        if ($count > 0) {
            // get tags
            $list = $this->Moderation_model->get_moderation_list($moder_type_name, $page, $items_on_page);
            $this->view->assign('list', $list);
        }

        $this->load->helper("navigation");
        $page_data = get_admin_pages_data(site_url() . "admin/moderation/index/" . $type_name . "/", $count,
                                          $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->view->assign('type_name', $type_name);

        $moder_types = $this->Moderation_type_model->get_types();
        if (is_array($moder_types) && count($moder_types)) {
            foreach ($moder_types as $key => $mtype) {
                $moder_types[$key]["count"] = $this->Moderation_model->get_moderation_list_count($mtype["name"]);
            }
        }
        $this->view->assign('moder_types', $moder_types);
        $this->view->render('admin_moder_list');
    }

    /**
     * Remove object by record in moderation table
     *
     * Удаляет объект по записи в таблице модераций
     *
     * @param integer $id_entry entry identifier
     *
     * @return void
     */
    public function delete_object($id_entry)
    {
        if (intval($id_entry)) {
            $item_data = $this->Moderation_model->get_moderation_item($id_entry);
            $object_id = $item_data["id_object"];
            $type_data = $this->Moderation_model->get_moderation_type_by_id($item_data["id_type"]);

            if ($object_id && $type_data["model"] && $type_data["module"] && $type_data["method_delete_object"]) {
                /// подключаем модель
                $model_name = ucfirst($type_data["model"]);
                $model_path = strtolower($type_data["module"] . "/models/") . $model_name;
                $this->load->model($model_path);

                /// удаляем объект
                $this->$model_name->$type_data["method_delete_object"]($object_id);

                //// удаляем запись в модерации (вдруг осталась после делейта объекта)
                $this->Moderation_model->delete_moderation_item_by_id($id_entry);
            }
        }
        redirect(site_url() . "admin/moderation/index/" . $_SESSION["mobjects_list"]["type_name"] . "/" . $_SESSION["mobjects_list"]["page"]);

        return;
    }

    /**
     * Remove moderation object
     *
     * Удаляет запись в таблице модераций
     *
     * @param integer $id_entry entry identifier
     *
     * @return void
     */
    public function delete($id_entry)
    {
        if (intval($id_entry)) {
            $this->Moderation_model->delete_moderation_item_by_id($id_entry);
        }
        redirect(site_url() . "admin/moderation/index/" . $_SESSION["mobjects_list"]["type_name"] . "/" . $_SESSION["mobjects_list"]["page"]);

        return;
    }

    /**
     * Approve moderation object
     *
     * @param integer $id_entry entry identifier
     *
     * @return void
     */
    public function approve($id_entry)
    {
        if (intval($id_entry)) {
            $item_data = $this->Moderation_model->get_moderation_item($id_entry);
            $object_id = $item_data["id_object"];
            $type_data = $this->Moderation_model->get_moderation_type_by_id($item_data["id_type"]);

            if ($object_id && $type_data["model"] && $type_data["module"] && $type_data["method_set_status"]) {
                /// подключаем модель
                $model_name = ucfirst($type_data["model"]);
                $model_path = strtolower($type_data["module"] . "/models/") . $model_name;
                $this->load->model($model_path);

                /// апрувим объект
                $this->$model_name->$type_data["method_set_status"]($object_id, 1);

                //// удаляем запись в модерации
                $this->Moderation_model->delete_moderation_item_by_id($id_entry);
            }
        }
        redirect(site_url() . "admin/moderation/index/" . $_SESSION["mobjects_list"]["type_name"] . "/" . $_SESSION["mobjects_list"]["page"]);

        return;
    }

    /**
     * Decline moderation object
     *
     * @param integer $id_entry entry identifier
     *
     * @return void
     */
    public function decline($id_entry)
    {
        if (intval($id_entry)) {
            $item_data = $this->Moderation_model->get_moderation_item($id_entry);
            $object_id = $item_data["id_object"];
            $type_data = $this->Moderation_model->get_moderation_type_by_id($item_data["id_type"]);

            if ($object_id && $type_data["model"] && $type_data["module"] && $type_data["method_set_status"] && $type_data["allow_to_decline"]) {
                /// подключаем модель
                $model_name = ucfirst($type_data["model"]);
                $model_path = strtolower($type_data["module"] . "/models/") . $model_name;
                $this->load->model($model_path);

                /// деклайним объект
                $this->$model_name->$type_data["method_set_status"]($object_id, 0);

                //// удаляем запись в модерации
                $this->Moderation_model->delete_moderation_item_by_id($id_entry);
            }
        }
        redirect(site_url() . "admin/moderation/index/" . $_SESSION["mobjects_list"]["type_name"] . "/" . $_SESSION["mobjects_list"]["page"]);

        return;
    }

    /**
     * Manage moderation settings
     *
     * @return void
     */
    public function settings()
    {
        $moder_types = $this->Moderation_type_model->get_types();
        if (is_array($moder_types) && count($moder_types)) {
            foreach ($moder_types as $key => $mtype) {
                $moder_types[$key]["count"] = $this->Moderation_model->get_moderation_list_count($mtype["name"]);
            }
        }

        if ($this->input->post("btn_save")) {
            $mtype = $this->input->post("mtype");
            $types_id = $this->input->post("type_id");
            $check_badwords = $this->input->post("check_badwords");

            if (is_array($moder_types)) {
                foreach ($moder_types as $key => $value) {
                    $moder_types[$key]['mtype'] = isset($mtype[$value['id']]) ? strval(intval($mtype[$value['id']])) : -1;
                    $moder_types[$key]['check_badwords'] = isset($check_badwords[$value['id']]) ? strval(intval($check_badwords[$value['id']])) : 0;
                }
            }

            $post_data["moderation_send_mail"] = $this->input->post("moderation_send_mail");
            $post_data["admin_moderation_emails"] = $this->input->post("admin_moderation_emails");

            $validate_data = $this->Moderation_model->validate_settings($post_data);          
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = $post_data;
            } else {
                foreach ($validate_data["data"] as $setting => $value) {
                    $this->pg_module->set_module_config("moderation", $setting, $value);
                }
                $data = $validate_data["data"];

                if (is_array($mtype)) {
                    foreach ($types_id as $type_id) {
                        $type_id = intval($type_id);
                        if ($type_id) {
                            $attrs["mtype"] = isset($mtype[$type_id]) ? strval(intval($mtype[$type_id])) : -1;
                            $attrs["check_badwords"] = isset($check_badwords[$type_id]) ? intval($check_badwords[$type_id]) : 0;
                            $this->Moderation_type_model->save_type($type_id, $attrs);
                        }
                    }
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "moderation"));
            }
        } else {
            $data["moderation_send_mail"] = $this->pg_module->get_module_config("moderation", "moderation_send_mail");
            $data["admin_moderation_emails"] = $this->pg_module->get_module_config("moderation",
                                                                                   "admin_moderation_emails");
        }
        $this->view->assign("data", $data);
        $this->view->assign('moder_types', $moder_types);

        $this->view->render('admin_moder_settings');
    }

    /**
     * Manage bad words
     *
     * @param string $type action name
     *
     * @return void
     */
    public function badwords($type = "")
    {
        if ($type == "add" && isset($_POST) && !empty($_POST)) {
            $word = $this->input->post("word");
            $errors = $this->Moderation_badwords_model->set_badword($word);
            if (count($errors)) {
                $this->system_messages->addMessage(View::MSG_ERROR, $errors);
            }
        }

        if ($type == "check_text" && isset($_POST) && !empty($_POST)) {
            $check_data["text"] = $this->input->post("text");
            $check_data["modified"] = $this->Moderation_badwords_model->search_in_text($check_data["text"]);
            $this->view->assign('check_data', $check_data);
        }
        $this->view->assign('type', $type);

        $badwords = $this->Moderation_badwords_model->get_badwords();
        $this->view->assign('badwords', $badwords);
        $this->view->render('admin_moder_badwords');
    }

    /**
     * Remove bad word
     *
     * @param integer $id bad word identifier
     *
     * @return void
     */
    public function delete_badword($id)
    {
        $id = intval($id);
        if ($id) {
            $this->Moderation_badwords_model->delete_badword($id);
        }
        redirect(site_url() . "admin/moderation/badwords");
    }

    /**
     * Aprrove moderation objects
     *
     * @param integer $id_entries entries identifiers
     *
     * @return void
     */
    public function few_approve($id_entries)
    {
        $id_entries = explode('-', $id_entries);

        foreach ($id_entries as $id_entry) {
            if (intval($id_entry)) {
                $item_data = $this->Moderation_model->get_moderation_item($id_entry);
                $object_id = $item_data["id_object"];
                $type_data = $this->Moderation_model->get_moderation_type_by_id($item_data["id_type"]);

                if ($object_id && $type_data["model"] && $type_data["module"] && $type_data["method_set_status"]) {
                    /// подключаем модель
                    $model_name = ucfirst($type_data["model"]);
                    $model_path = strtolower($type_data["module"] . "/models/") . $model_name;
                    $this->load->model($model_path);

                    /// апрувим объект
                    $this->$model_name->$type_data["method_set_status"]($object_id, 1);

                    //// удаляем запись в модерации
                    $this->Moderation_model->delete_moderation_item_by_id($id_entry);
                }
            }
        }

        redirect(site_url() . "admin/moderation/index/" . $_SESSION["mobjects_list"]["type_name"] . "/" . $_SESSION["mobjects_list"]["page"]);

        return;
    }

}
