<?php

namespace Pg\Modules\Moderation\Models;

/**
 * Moderation module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Moderation install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Moderation
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Moderation_install_model extends \Model
{

    /**
     * Link to Code Igniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'admin_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'main_items' => array(
                    'action' => 'none',
                    'items' => array(
                        'moderation-items' => array(
                            "action" => "none",
                            "items" => array(
                                "objects_list_item" => array('action' => 'create', 'link' => 'admin/moderation/index', 'status' => 1, 'sorter' => 2),
                                "moder_settings_item_2" => array('action' => 'create', 'link' => 'admin/moderation/settings', 'status' => 1, 'sorter' => 3),
                                "badwords_item" => array('action' => 'create', 'link' => 'admin/moderation/badwords', 'status' => 1, 'sorter' => 4),
                            ),
                        ),
                    ),
                ),
            ),
        ),
        'admin_moderation_menu' => array(
            'action' => 'create',
            'name' => 'Admin mode - Moderation',
            'items' => array(
                'object_list_item' => array('action' => 'create', 'link' => 'admin/moderation/index', 'status' => 1),
                'moder_setting_item' => array('action' => 'create', 'link' => 'admin/moderation/settings', 'status' => 1),
                'badwords_item' => array('action' => 'create', 'link' => 'admin/moderation/badwords', 'status' => 1),
            ),
        ),
    );

    /**
     * Indicators configuration
     *
     * @var array
     */
    protected $menu_indicators = array(
        array(
            'gid' => 'new_item_for_moderation',
            'delete_by_cron' => false,
            'auth_type' => 'admin',
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators_methods = array(
        array('module' => 'moderation', 'method' => 'index', 'is_default' => 1),
        array('module' => 'moderation', 'method' => 'settings', 'is_default' => 0),
        array('module' => 'moderation', 'method' => 'badwords', 'is_default' => 0),
    );

    /**
     * Notifications configuration
     *
     * @var array
     */
    protected $notifications = array(
        "templates" => array(
            array("gid" => "auser_need_moderate", "name" => "New object awaiting moderation", "vars" => array("type"), "content_type" => "text"),
        ),
        "notifications" => array(
            array("gid" => "auser_need_moderate", "template" => "auser_need_moderate", "send_type" => "simple"),
        ),
    );

    /**
     * Constructor
     *
     * @return Moderation_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        //// load langs
        $this->ci->load->model('Install_model');
    }

    /**
     * Install data of menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
        if (!empty($this->menu_indicators)) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->menu_indicators as $data) {
                $this->ci->Indicators_model->save_type(null, $data);
            }
        }
    }

    /**
     * Install languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read('moderation', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');

            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid,
                                              $langs_file);
        }

        // Indicators
        if (!empty($this->menu_indicators)) {
            $langs_file = $this->ci->Install_model->language_file_read('moderation', 'indicators', $langs_ids);
            if (!$langs_file) {
                log_message('info', '(resumes) Empty indicators langs data');

                return false;
            } else {
                $this->ci->load->model('menu/models/Indicators_model');
                $this->ci->Indicators_model->update_langs($this->menu_indicators, $langs_file, $langs_ids);
            }
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->helper('menu');

        $return = array();

        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        if (!empty($this->menu_indicators)) {
            $this->ci->load->model('menu/models/Indicators_model');
            $indicators_langs = $this->ci->Indicators_model->export_langs($this->menu_indicators, $langs_ids);
        }

        return array("menu" => $return, "indicators" => $indicators_langs);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
        if (!empty($this->menu_indicators)) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->menu_indicators as $data) {
                $this->ci->Indicators_model->delete_type($data['gid']);
            }
        }
    }

    /**
     * Install data of moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model('Moderators_model');

        foreach ($this->moderators_methods as $method) {
            $this->ci->Moderators_model->save_method(null, $method);
        }
    }

    /**
     * Install languages of moderators module
     *
     * @param array $langs_ids languages idnetifiers
     *
     * @return void
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read('moderation', 'moderators', $langs_ids);

        // install moderators permissions
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'moderation';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method['method']])) {
                $this->ci->Moderators_model->save_method($method['id'], array(), $langs_file[$method['method']]);
            }
        }
    }

    /**
     * Export languages of moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'moderation';
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method['method']] = $method['langs'];
        }

        return array('moderators' => $return);
    }

    /**
     * Uninstall data of moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        // delete moderation methods in moderators
        $this->ci->load->model('Moderators_model');
        $params['where']['module'] = 'moderation';
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of notifications module
     *
     * @return void
     */
    public function install_notifications()
    {
        // add notification
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        $templates_ids = array();

        foreach ((array) $this->notifications["templates"] as $template_data) {
            if (is_array($template_data["vars"])) {
                $template_data["vars"] = implode(", ", $template_data["vars"]);
            }
            $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $templates_ids[$template_data['gid']] = $this->ci->Templates_model->save_template(null,
                                                                                              $validate_data["data"]);
        }

        foreach ((array) $this->notifications["notifications"] as $notification_data) {
            if (!isset($templates_ids[$notification_data["template"]])) {
                $template = $this->ci->Templates_model->get_template_by_gid($notification_data["template"]);
                $templates_ids[$notification_data["template"]] = $template["id"];
            }
            $notification_data["id_template_default"] = $templates_ids[$notification_data["template"]];
            $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Notifications_model->save_notification(null, $validate_data["data"]);
        }
    }

    /**
     * Import languages of notifiactions module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_notifications_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Notifications_model");

        $langs_file = $this->ci->Install_model->language_file_read("moderation", "notifications", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty notifications langs data");

            return false;
        }

        $this->ci->Notifications_model->update_langs($this->notifications, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export languages of notifications module
     *
     * @param array $langs_ids languages idnetifiers
     *
     * @return array
     */
    public function install_notifications_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Notifications_model");
        $langs = $this->ci->Notifications_model->export_langs($this->notifications, $langs_ids);

        return array("notifications" => $langs);
    }

    /**
     * Unistall data of notifacations module
     *
     * @return void
     */
    public function deinstall_notifications()
    {
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach ((array) $this->notifications["notifications"] as $notification_data) {
            $this->ci->Notifications_model->delete_notification_by_gid($notification_data["gid"]);
        }

        foreach ((array) $this->notifications["templates"] as $template_data) {
            $this->ci->Templates_model->delete_template_by_gid($template_data["gid"]);
        }
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        if ($this->pg_module->is_module_active('moderators')) {
            $this->ci->load->model("Moderators_model");
            $users = $this->ci->Moderators_model->get_users_list(null, 1, array('id' => 'DESC'),
                                                                 array('where' => array('user_type' => 'admin')));
            if (!empty($users)) {
                $this->ci->pg_module->set_module_config("moderation", 'admin_moderation_emails', $users[0]["email"]);
            }
        }
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        
    }

}
