<?php

namespace Pg\Modules\News\Controllers;

/**
 * News module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * News api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	News
 *
 * @category	controollers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_News extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_News
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('News_model');
    }

    /**
     * Get news list
     *
     * @param integer $page page of results (post)
     *
     * @return void
     */
    public function news_list()
    {
        $attrs = array();
        //$attrs['id_lang'] = $this->pg_language->current_lang_id;
        $attrs['status'] = '1';
        $news_count = $this->News_model->get_filtered_news_count($attrs);
        $page = $this->input->post('page', true);
        $items_on_page = $this->pg_module->get_module_config('news', 'userside_items_per_page');
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $news_count, $items_on_page);

        if ($news_count > 0) {
            $data['news'] = $this->News_model->get_filtered_news_list($attrs, $page, $items_on_page,
                                                                      array('id' => 'DESC'));
            $data['date_format'] = $this->pg_date->get_format('date_time_literal', 'st');
            $data['count'] = $news_count;
            if ($news_count > 0) {
                $date_formats = $this->pg_date->get_format('date_literal', 'st');
                $this->load->helper('date_format');
                foreach ($data['news'] as $i => $news) {
                    $data['news'][$i]['date_add_output'] = tpl_date_format($news['date_add'], $date_formats);
                }
            }
            $this->set_api_content('data', $data);
        } else {
            $this->set_api_content('messages', l('api_error_news_not_found', 'news'));
        }
    }

    /**
     * Get news by id
     *
     * @return void
     */
    public function view()
    {
        $id = (int) $this->input->post('news_id', true);
        if (!$id) {
            log_message('error', 'realestate API: Empty news id');
            $this->set_api_content('error', l('api_error_empty_news_id', 'news'));

            return false;
        }
        $news = $this->News_model->get_news_by_id($id);
        if (!$news) {
            log_message('error', 'realestate API: News wаith id "' . $id . '" not found');
            $this->set_api_content('error', l('api_error_news_not_found', 'news'));

            return false;
        }
        //$this->News_model->get_last_news(2);
        $news = $this->News_model->format_single_news($news);

        $date_formats = $this->pg_date->get_format('date_literal', 'st');
        $this->load->helper('date_format');
        $news['date_add_output'] = tpl_date_format($news['date_add'], $date_formats);

        $date_format = $this->pg_date->get_format('date_time_literal', 'st');
        $this->set_api_content('data', array('news' => $news, 'date_format' => $date_format));
    }

}
