<?php

namespace Pg\Modules\Payments\Controllers;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Payments user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Payments extends \Controller
{

    /**
     * Constructor
     *
     * @return Payment
     */
    public function __construct()
    {
        parent::__construct();
    }

    public function index()
    {
        $this->view->setRedirect('users/account');
        //$this->view->render('index');
    }

    /**
     * Get payment statistics
     *
     * @param integer $page page of results
     *
     * @return void
     */
    public function statistic($page = 1)
    {
        if ($this->session->userdata('auth_type') != 'user') {
            show_404();

            return;
        }
        $user_id = $this->session->userdata('user_id');
        $this->load->model("Payments_model");
        $params["where"]["id_user"] = $user_id;

        $payments_count = $this->Payments_model->get_payment_count($params);

        $items_on_page = $this->pg_module->get_module_config('payments', 'items_per_page');
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $payments_count, $items_on_page);

        $payments = $this->Payments_model->get_payment_list($page, $items_on_page, array("date_add" => "DESC"), $params);
        $this->view->assign('payments', $payments);

        $base_currency_gid = $this->pg_module->get_module_config("start", "base_currency");
        $this->view->assign('base_currency_gid', $base_currency_gid);

        $this->load->helper('navigation');

        $page_data = get_user_pages_data(site_url() . "payments/statistic/", $payments_count, $items_on_page, $page, 'briefPage');
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_active(l('header_my_payments_statistic', 'payments'));

        $this->view->render('statistic');
    }

    /**
     * Response from payment system
     *
     * @param string $system_gid system guid
     *
     * @return void
     */
    public function response($system_gid)
    {
        if (empty($system_gid)) {
            return false;
        }

        $data = $_REQUEST;
        $this->load->helper('payments');
        $return = receive_payment($system_gid, $data);

        if ($return["type"] == "html") {
            if (!empty($return["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $return["errors"]);
            }
            if (!empty($return["info"])) {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $return["info"]);
            }

            $this->load->model("Payments_model");
            $payment = $this->Payments_model->get_payment_by_id($return["data"]["id_payment"]);
            $temp = $this->Payments_model->format_payments(array(0 => $payment));
            $this->view->assign('payment', $temp[0]);

            $this->load->model("Payment_systems_model");
            $payment_system = $this->Payment_systems_model->get_system_by_gid($payment['system_gid']);
            $this->view->assign('payment_system', $payment_system);

            $this->load->model('Menu_model');
            $this->Menu_model->breadcrumbs_set_active(l('header_payment_form', 'payments'), site_url() . 'payments/form');
            $this->Menu_model->breadcrumbs_set_active(l('header_payment_return', 'payments'));

            $this->view->render('payment_return');
        }
    }

    /**
     * Show payment form
     *
     * @return void
     */
    public function form()
    {
        $data["payment_type_gid"] = $this->input->post('payment_type_gid', true);
        $data["amount"] = $this->input->post('amount', true);
        // $data["id_user"] = $this->input->post('id_user', true);
        $data["id_user"] = $this->session->userdata('user_id');
        $data["currency_gid"] = $this->input->post('currency_gid', true);
        $data["system_gid"] = $this->input->post('system_gid', true);
        $data["payment_data"] = $this->input->post('payment_data', true);
        $this->load->model("payments/models/Payment_systems_model");
        $data["system"] = $this->Payment_systems_model->get_system_by_gid($data["system_gid"]);
        $this->Payment_systems_model->load_driver($data["system_gid"]);
        $data["map"] = $this->Payment_systems_model->get_html_data_map();

        $this->load->model("Payments_model");
        if (!$this->Payments_model->validate_payment_form($data)) {
            $this->view->setRedirect(site_url() . 'payments/statistic');
        }

        if ($this->input->post('btn_save')) {
            $data["payment_data"] = $_SESSION["saved_payment_data"];
            $map = $this->input->post('map', true);
            if (!empty($map)) {
                foreach ($map as $key => $value) {
                    $data["payment_data"][$key] = $value;
                }
                $this->load->helper('payments');
                $return = send_payment($data["payment_type_gid"], $data["id_user"], $data["amount"], $data["currency_gid"], $data["system_gid"], $data["payment_data"], "validate");
                $data["payment_data"] = $return["data"];

                if (!empty($return["info"])) {
                    $this->system_messages->addMessage(View::MSG_INFO, $return["info"]);
                }

                if (!empty($return["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $return["errors"]);
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_payment_send', 'payments'));
                    $this->view->setRedirect(site_url() . 'payments/statistic');
                }
            }
        } else {
            // payment_data may content diff data from post
            $_SESSION["saved_payment_data"] = $data["payment_data"];
        }

        $this->load->model("Currencies_model");
        $data["currency"] = $this->Currencies_model->get_currency_by_gid($data["currency_gid"]);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_active(l('header_my_payments_statistic', 'payments'));

        $this->view->assign('data', $data);
        $this->view->render('payment_form');
    }

    /**
     * Get javascript form
     *
     * @param integer $payment_id payment identifier
     *
     * @return void
     */
    public function js($payment_id)
    {
        $this->load->model("Payments_model");
        $payment = $this->Payments_model->get_payment_by_id($payment_id);

        $this->load->model("payments/models/Payment_systems_model");
        $payment['system'] = $this->Payment_systems_model->get_system_by_gid($payment["system_gid"]);

        $this->Payment_systems_model->load_driver($payment['system_gid']);

        $js = $this->Payment_systems_model->get_js($payment, $payment['system']);
        $this->view->assign('js', $js);

        if (!isset($payment['js'])) {
            $payment['js'] = '';
        }
        $this->view->assign('data', $payment);

        if (!empty($_SERVER['HTTP_REFERER']) && preg_match('#http(s)?://#', $_SERVER['HTTP_REFERER'])) {
            $this->view->assign('back', $_SERVER['HTTP_REFERER']);
        }

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_active(l('header_my_payments_statistic', 'payments'));

        $this->view->render('payment_js');
    }

    /**
     * Get payment form by mobile
     *
     * @param integer $payment_id payment identifier
     *
     * @return void
     */
    public function mobile($payment_id)
    {
        $this->load->model("Payments_model");
        $payment = $this->Payments_model->get_payment_by_id($payment_id);

        $this->load->model("payments/models/Payment_systems_model");
        $payment['system'] = $this->Payment_systems_model->get_system_by_gid($payment["system_gid"]);

        if ($this->Payment_systems_model->action_js($payment["system_gid"])) {
            $this->Payment_systems_model->load_driver($payment['system_gid']);

            $js = $this->Payment_systems_model->get_js($payment, $payment['system']);
            $this->view->assign('js', $js);

            if (!isset($payment['js'])) {
                $payment['js'] = '';
            }
            $this->view->assign('data', $payment);

            if (preg_match('#http(s)?://#', $_SERVER['HTTP_REFERER'])) {
                $this->view->assign('back', $_SERVER['HTTP_REFERER']);
            }
        } else {
            $payment["id_payment"] = $payment["id"];
            $this->Payment_systems_model->action_request($payment["system_gid"], $payment);
        }

        $this->view->render('payment_mobile');
    }

    /**
     * Change currency
     *
     * @param integer $currency_id currency identifier
     *
     * @return void
     */
    public function change_currency($currency_id)
    {
        $this->session->set_userdata("currency_id", intval($currency_id));
        if (strpos($_SERVER["HTTP_REFERER"], site_url()) !== false) {
            $this->view->setRedirect($_SERVER["HTTP_REFERER"]);
        } else {
            $this->view->setRedirect();
        }
    }

}
