<?php

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Payments management
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('admin_home_payments_block')) {

    /**
     * Get homepage payments block in admin mode
     *
     * @return string
     */
    function admin_home_payments_block()
    {
        $ci = &get_instance();

        $auth_type = $ci->session->userdata("auth_type");
        if ($auth_type != "admin") {
            return '';
        }

        $user_type = $ci->session->userdata("user_type");

        $show = true;
        if ($user_type == 'moderator') {
            $show = false;
            $ci->load->model('Moderators_model');
            $methods = $ci->Moderators_model->get_module_methods('payments');
            if (is_array($methods) && !in_array('index', $methods)) {
                $show = true;
            } else {
                $permission_data = $ci->session->userdata("permission_data");
                if (isset($permission_data['payments']['index']) && $permission_data['payments']['index'] == 1) {
                    $show = true;
                }
            }
        }

        if (!$show) {
            return '';
        }

        $ci->load->model('Payments_model');
        $stat_payments = array(
            "all"     => $ci->Payments_model->get_payment_count(),
            "wait"    => $ci->Payments_model->get_payment_count(array("where" => array('status' => 0))),
            "approve" => $ci->Payments_model->get_payment_count(array("where" => array('status' => 1))),
            "decline" => $ci->Payments_model->get_payment_count(array("where" => array('status' => -1))),
        );

        $ci->load->model("Currencies_model");
        $ci->view->assign('currency', $ci->Currencies_model->default_currency);

        $ci->view->assign("stat_payments", $stat_payments);

        return $ci->view->fetch('helper_admin_home_block', 'admin', 'payments');
    }
}

if (!function_exists('send_payment')) {

    /**
     * Send request to payment system
     *
     * @param string  $payment_type_gid  payment type guid
     * @param integer $id_user           user identifier
     * @param float   $amount            payment amount
     * @param string  $currency_gid      currency guid
     * @param string  $system_gid        system guid
     * @param array   $payment_data      payment data
     * @param boolean $check_html_action action name
     *
     * @return array/void
     */
    function send_payment($payment_type_gid, $id_user, $amount, $currency_gid, $system_gid, $payment_data = array(), $check_html_action = false)
    {
        $return = array("errors" => array(), "info" => array());

        $ci = &get_instance();
        $ci->load->model('payments/models/Payment_systems_model');

        if ($check_html_action == "form" && $ci->Payment_systems_model->action_html($system_gid)) {
            $post_data = array(
                "payment_type_gid" => $payment_type_gid,
                "id_user"          => $id_user,
                "amount"           => $amount,
                "currency_gid"     => $currency_gid,
                "system_gid"       => $system_gid,
                "payment_data"     => $payment_data,
            );
            post_location_request(site_url() . "payments/form", $post_data);

            return;
        }

        if ($check_html_action == "form" && $ci->Payment_systems_model->action_js($system_gid)) {
            $post_data = array(
                "payment_type_gid" => $payment_type_gid,
                "id_user"          => $id_user,
                "amount"           => $amount,
                "currency_gid"     => $currency_gid,
                "system_gid"       => $system_gid,
                "payment_data"     => $payment_data,
            );

            $ci->load->model('Payments_model');
            $validate_data = $ci->Payments_model->validate_payment(null, $post_data);
            if (!empty($validate_data["errors"])) {
                $return['errors'] = $validate_data["errors"];

                return $return;
            } else {
                $payment_id = $ci->Payments_model->save_payment(null, $validate_data['data']);
                redirect(site_url() . 'payments/js/' . $payment_id);

                return;
            }
        }

        if ($check_html_action == "validate" && $ci->Payment_systems_model->action_html($system_gid)) {
            $validate = $ci->Payment_systems_model->action_validate($system_gid, $payment_data);
            if (!empty($validate["errors"])) {
                $return = $validate;

                return $return;
            } else {
                $payment_data = $validate["data"];
            }
        }

        $ci->load->model('Payments_model');

        $for_validate = array(
            "payment_type_gid" => $payment_type_gid,
            "id_user"          => $id_user,
            "amount"           => $amount,
            "currency_gid"     => $currency_gid,
            "system_gid"       => $system_gid,
            "payment_data"     => $payment_data,
        );
        $validate_data = $ci->Payments_model->validate_payment(null, $for_validate);
        if (!empty($validate_data["errors"])) {
            $return["errors"] = $validate_data["errors"];
        } else {
            $id_payment = $ci->Payments_model->save_payment(null, $validate_data["data"]);

            $payment = $ci->Payments_model->get_payment_by_id($id_payment);
            $payment["id_payment"] = $payment["id"];

            $return = $ci->Payment_systems_model->action_request($payment["system_gid"], $payment);
        }

        return $return;
    }
}

if (!function_exists('receive_payment')) {

    /**
     * Get response from payment system
     *
     * @param string $system_gid   system guid
     * @param array  $request_data response data
     *
     * @return array
     */
    function receive_payment($system_gid, $request_data)
    {
        $return = array("errors" => array(), "info" => array());

        $ci = &get_instance();
        $ci->load->model('payments/models/Payment_systems_model');

        $payment_data = $ci->Payment_systems_model->action_responce($system_gid, $request_data);

        $data = $payment_data["data"];
        if (!empty($data["id_payment"])) {
            $ci->load->model('Payments_model');
            $ci->Payments_model->change_payment_status($data["id_payment"], $data["status"]);
        }

        return $payment_data;
    }
}

if (!function_exists('post_location_request')) {

    /**
     * Send post request
     *
     * @param string $url  request url
     * @param array  $data request data
     *
     * @return void
     */
    function post_location_request($url, $data)
    {
        if (!empty($data)) {
            $params = explode("&", urldecode(http_build_query($data)));
        } else {
            $params = array();
        }

        $retHTML = "<html>\n<body onLoad=\"document.send_form.submit();\">\n";
        $retHTML .= "<form method=\"post\" name=\"send_form\" action=\"" . $url . "\">\n";
        foreach ($params as $string) {
            list($key, $value) = explode("=", $string);
            $retHTML .= "<input type=\"hidden\" name=\"" . $key . "\" value=\"" . addslashes($value) . "\">\n";
        }
        $retHTML .= "</form>\n</body>\n</html>";
        print $retHTML;
        exit();
    }
}
