<?php

namespace Pg\Modules\Payments\Models;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('PAYMENTS_TABLE', DB_PREFIX . 'payments');
define('PAYMENTS_TYPES_TABLE', DB_PREFIX . 'payments_type');

/**
 * Payments main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Payments_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Payments object properties in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'payment_type_gid',
        'id_user',
        'amount',
        'currency_gid',
        'status',
        'system_gid',
        'date_add',
        'date_update',
        'payment_data',
    );

    /**
     * Fields will be formatted on
     *
     * @var array
     */
    protected $payment_data_registered_fields = array("name", "comment");

    /**
     * Constructor
     *
     * @return Payments_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return payment object by identifier
     *
     * @param integer $id payment identifier
     *
     * @return array
     */
    public function get_payment_by_id($id)
    {
        $result = $this->ci->db->select(implode(", ", $this->fields))->from(PAYMENTS_TABLE)->where("id", $id)->get()->result_array();
        $return = (!empty($result)) ? $result[0] : array();
        if (!empty($return["payment_data"])) {
            $return["payment_data"] = unserialize($return["payment_data"]);
        }
        $return["id_payment"] = $return["id"];

        return $return;
    }

    /**
     * Return payments objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param array   $order_by          sorting data
     * @param array   $params            sql criteria of query to data source
     * @param array   $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_payment_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(", ", $this->fields));
        $this->ci->db->from(PAYMENTS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                if (!empty($r["payment_data"])) {
                    $r["payment_data"] = unserialize($r["payment_data"]);
                }
                $data[] = $r;
            }

            return $this->format_payments($data);
        }

        return array();
    }

    /**
     * Format payments objects
     *
     * @param array $data payments data
     *
     * @return array
     */
    public function format_payments($data)
    {
        if (empty($data)) {
            return $data;
        }

        $user_ids = $currency_gids = array();
        foreach ($data as $key => $payment) {
            if (empty($user_ids) || !in_array($payment["id_user"], $user_ids)) {
                $user_ids[] = $payment["id_user"];
            }
            if (empty($currency_gids) || !in_array($payment["currency_gid"], $currency_gids)) {
                $currency_gids[] = $payment["currency_gid"];
            }

            if (!empty($payment["payment_data"])) {
                foreach ($payment["payment_data"] as $param_id => $param_value) {
                    if (in_array($param_id, $this->payment_data_registered_fields)) {
                        $payment["payment_data_formatted"][$param_id] = array(
                            "name" => l('html_field_' . $param_id, 'payments'),
                            "value" => nl2br($param_value),
                        );
                    }
                }
            }
            $data[$key] = $payment;
        }

        if (!empty($user_ids)) {
            $this->ci->load->model('Users_model');
            $users = $this->ci->Users_model->get_users_list_by_key(null, null, null, array(), $user_ids);
        }

        if (!empty($currency_gids)) {
            $this->ci->load->model("Currencies_model");
            $params["where_in"]["gid"] = $currency_gids;
            $currencies = $this->ci->Currencies_model->get_currency_list_by_key($params);
        }

        foreach ($data as $key => $payment) {
            if (!empty($users[$payment["id_user"]])) {
                $payment["user"] = $users[$payment["id_user"]];
            } else {
                $payment["user"] = $this->ci->Users_model->format_default_user($payment["id_user"]);
            }
            if (!empty($currencies[$payment["currency_gid"]])) {
                $payment["currency"] = $currencies[$payment["currency_gid"]];
            }
            $data[$key] = $payment;
        }

        return $data;
    }

    /**
     * Return number of payments objects
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_payment_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt");
        $this->ci->db->from(PAYMENTS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Validate payment object for saving to data source
     *
     * @param integer $id   payment identifier
     * @param array   $data payment data
     *
     * @return array
     */
    public function validate_payment($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["payment_type_gid"])) {
            $return["data"]["payment_type_gid"] = $data["payment_type_gid"];
        }

        if (isset($data["id_user"])) {
            $return["data"]["id_user"] = intval($data["id_user"]);
        }

        if (isset($data["amount"])) {
            $return["data"]["amount"] = floatval($data["amount"]);
        }

        if (isset($data["currency_gid"])) {
            $return["data"]["currency_gid"] = $data["currency_gid"];
        }

        if (isset($data["status"])) {
            $return["data"]["status"] = intval($data["status"]);
        }

        if (isset($data["system_gid"])) {
            $return["data"]["system_gid"] = $data["system_gid"];
        }

        if (isset($data["payment_data"])) {
            $return["data"]["payment_data"] = serialize($data["payment_data"]);
        }

        return $return;
    }

    /**
     * Save payment object to data source
     *
     * @param integer $id   payment identifier
     * @param array   $data payment data
     *
     * @return integer
     */
    public function save_payment($id, $data)
    {
        if (is_null($id)) {
            $data["date_add"] = $data["date_update"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(PAYMENTS_TABLE, $data);
            $id = $this->ci->db->insert_id();

            $this->ci->load->model('menu/models/Indicators_model');
            $this->ci->Indicators_model->add('new_payment_item', $id);
        } else {
            $data["date_update"] = date("Y-m-d H:i:s");
            $this->ci->db->where('id', $id);
            $this->ci->db->update(PAYMENTS_TABLE, $data);
        }

        return $id;
    }

    /**
     * Remove payment object by identifier
     *
     * @param integer $id payment identifier
     *
     * @return void
     */
    public function delete_payment($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(PAYMENTS_TABLE);

        return;
    }

    ////// Payment types

    /**
     * Return payment type object by identifier
     *
     * @param integer $id type identifier
     *
     * @return array
     */
    public function get_payment_type_by_id($id)
    {
        $result = $this->ci->db->select("id, gid, callback_module, callback_model, callback_method")->from(PAYMENTS_TYPES_TABLE)->where("id",
                                                                                                                                    $id)->get()->result_array();
        $return = (!empty($result)) ? $result[0] : array();

        return $return;
    }

    /**
     * Return payment type object by GUID
     *
     * @param string $gid type GUID
     *
     * @return array
     */
    public function get_payment_type_by_gid($gid)
    {
        $result = $this->ci->db->select("id, gid, callback_module, callback_model, callback_method")->from(PAYMENTS_TYPES_TABLE)->where("gid",
                                                                                                                                    $gid)->get()->result_array();
        $return = (!empty($result)) ? $result[0] : array();

        return $return;
    }

    /**
     * Return payment types objects as array
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     * @param array $order_by          sorting data
     *
     * @return array
     */
    public function get_payment_type_list($params = array(), $filter_object_ids = null, $order_by = null)
    {
        $this->ci->db->select('id, gid, callback_module, callback_model, callback_method')->from(PAYMENTS_TYPES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                $this->ci->db->order_by($field . " " . $dir);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results;
        }

        return array();
    }

    /**
     * Return number of payment types object
     *
     * @param array $params            sql criteria of query to data source
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_payment_type_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt")->from(PAYMENTS_TYPES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Validate payment type object for saving to data source
     *
     * @param integer $id   type identifier
     * @param array   $data type data
     *
     * @return array
     */
    public function validate_payment_type($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["gid"])) {
            $data["gid"] = strip_tags($data["gid"]);
            $data["gid"] = preg_replace("/[^a-z0-9]+/i", '', $data["gid"]);

            $return["data"]["gid"] = $data["gid"];

            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_payment_type_code_incorrect', 'payments');
            }
        }

        if (isset($data["callback_module"])) {
            $return["data"]["callback_module"] = $data["callback_module"];
        }

        if (isset($data["callback_model"])) {
            $return["data"]["callback_model"] = $data["callback_model"];
        }

        if (isset($data["callback_method"])) {
            $return["data"]["callback_method"] = $data["callback_method"];
        }

        return $return;
    }

    /**
     * Save payment type object to data source
     *
     * @param integer $id   type identifier
     * @param array   $data type data
     * @param array   $name name by languages
     */
    public function save_payment_type($id, $data, $name = null)
    {
        if (is_null($id)) {
            $this->ci->db->insert(PAYMENTS_TYPES_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(PAYMENTS_TYPES_TABLE, $data);
        }

        if (!empty($name)) {
            $languages = $this->ci->pg_language->languages;
            if (!empty($languages)) {
                $lang_ids = array_keys($languages);
                $this->ci->pg_language->pages->set_string_langs('payments', "payment_type_name_" . $id, $name, $lang_ids);
            }
        }

        return $id;
    }

    /**
     * Return payment type object by identifier
     *
     * @param integer $id type identifier
     *
     * @return void
     */
    public function delete_payment_type($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(PAYMENTS_TYPES_TABLE);
        $this->ci->pg_language->pages->delete_string("payments", "payment_type_name_" . $id);

        return;
    }

    /**
     * Return payment type object by GUID
     *
     * @param integer $gid type GUID
     *
     * @return void
     */
    public function delete_payment_type_by_gid($gid)
    {
        $payment_type_data = $this->get_payment_type_by_gid($gid);
        $this->delete_payment_type($payment_type_data["id"]);

        return;
    }

    /**
     * Change status of payment object
     *
     * @param integer $id_payment payment identifier
     * @param integer $status     payment status
     *
     * @return void
     */
    public function change_payment_status($id_payment, $status)
    {
        $payment_data = $this->get_payment_by_id($id_payment);
        if (empty($payment_data)) {
            return false;
        }

        if ($payment_data["status"] != $status) {
            $payment_type = $this->get_payment_type_by_gid($payment_data["payment_type_gid"]);

            $model_name = ucfirst($payment_type["callback_model"]);
            $model_path = strtolower($payment_type["callback_module"] . "/models/") . $model_name;
            $this->ci->load->model($model_path);
            $this->ci->$model_name->{$payment_type["callback_method"]}($payment_data, $status);

            $this->ci->load->model('menu/models/Indicators_model');
            $this->ci->Indicators_model->delete('new_payment_item', $id_payment, true);

            if ($payment_data["payment_type_gid"] == 'account' && $status == 1) {
                $this->ci->Indicators_model->add('new_account_item', $payment_data['id_user'], $payment_data['id_user']);
                $this->ci->Indicators_model->add('all_account_item', 'account_' . $payment_data['id_user'],
                                                 $payment_data['id_user']);
            }

            $this->save_payment($id_payment, array("status" => $status));
        }

        return;
    }

    /**
     * Validate data of payment form
     *
     * @param array $data payment data
     *
     * @return boolean
     */
    public function validate_payment_form($data)
    {
        $good = true;
        if (!$data["payment_type_gid"] || !$data["system_gid"]) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_system_gid_incorrect', 'payments'));
            $good = false;
        }
        if (!$data["amount"]) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_format_required', 'payments'));
            $good = false;
        }
        if (!$data["currency_gid"]) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_currency_code_incorrect', 'payments'));
            $good = false;
        }

        return $good;
    }

    /**
     * Import languages to payment types
     *
     * @param array $data       payment types data
     * @param array $langs_file data by languages
     * @param array $langs_ids  languages identifiers
     *
     * @return void
     */
    public function update_langs($data, $langs_file, $langs_ids)
    {
        foreach ($data as $item) {
            $payment_type = $this->get_payment_type_by_gid($item['gid']);
            $this->ci->pg_language->pages->set_string_langs('payments', "payment_type_name_" . $payment_type['id'],
                                                            $langs_file[$item['gid']], (array) $langs_ids);
        }
    }

    /**
     * Export languages from payment types
     *
     * @param array $data      payment types data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_langs($data, $langs_ids)
    {
        foreach ($data as $item) {
            $payment_type = $this->get_payment_type_by_gid($item['gid']);
            $gids[$item['gid']] = 'payment_type_name_' . $payment_type['id'];
        }
        $langs_data = $this->ci->pg_language->export_langs('payments', $gids, $langs_ids);

        return array_combine(array_keys($gids), $langs_data);
    }

}
