<?php

namespace Pg\Modules\Payments\Models\Systems;

use Pg\Modules\Payments\Models\Payment_driver_model;

/**
 * Payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Barclays payment system driver model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Barclays_model extends Payment_driver_model
{

    /**
     *  Payment settings
     *
     *  @return void
     */
    public $payment_data = array();

    /**
     * System settings
     *
     * @var array
     */
    public $settings = array(
        "seller_id" => array("type" => "text", "content" => "string", "size" => "middle"),
        "secret_word" => array("type" => "text", "content" => "string", "size" => "middle"),
    );

    /**
     * Response variables
     *
     * @var array
     */
    protected $variables = array(
        "oid" => "id_payment",
        "total" => "amount",
        "transactionstatus" => "transaction_status",
    );

    /**
     * Checkout url
     *
     * @var string
     */
    protected $checkout_url = 'https://secure2.epdq.co.uk/cgi-bin/CcxBarclaysEpdq.e';

    /**
     * Checkout url for test mode
     *
     * @var string
     */
    protected $test_checkout_url = '';

    /**
     * Do request to payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_request($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => $payment_data);

        $currency = $this->get_currency_num($payment_data["currency_gid"]);

        $params = "clientid=" . $system_settings["settings_data"]["seller_id"];
        $params .= "&password=" . $system_settings["settings_data"]["secret_word"];
        $params .= "&oid=" . $payment_data["id_payment"];
        $params .= "&chargetype=Auth";
        $params .= "&currencycode=" . $currency;
        $params .= "&total=" . $payment_data["amount"];
        $response = $this->pullpage("secure2.epdq.co.uk", "/cgi-bin/CcxBarclaysEpdqEncTool.e", $params);
        $response_lines = explode("\n", $response);
        $response_line_count = count($response_lines);
        for ($i = 0; $i < $response_line_count; ++$i) {
            if (preg_match('/epdqdata/', $response_lines[$i])) {
                $strEPDQ = $response_lines[$i];
            }
        }
        $tmp = explode("\"", $strEPDQ);
        $strEPDQ = $tmp[1];

        $send_data = array(
            "clientid" => $system_settings["settings_data"]["seller_id"],
            "secret_word" => $system_settings["settings_data"]["secret_word"],
            "total" => $payment_data["amount"],
            "currencycode" => $currency,
            "oid" => $payment_data["id_payment"],
            "merchantdisplayname" => $payment_data["payment_data"]["name"],
            "chargetype" => 'Auth',
            "epdqdata" => $strEPDQ,
            "returnurl" => site_url() . "payments/responce/barclays",
        );

        if (isset($system_settings['is_test']) && $system_settings['is_test'] == 2) {
            $checkout_url = $this->test_checkout_url;
        } else {
            $checkout_url = $this->checkout_url;
        }

        $this->send_data($checkout_url, $send_data, "post");

        return $return;
    }

    /**
     * Process response from payment system
     *
     * @param array $payment_data    payment data
     * @param array $system_settings system settings
     *
     * @return array
     */
    public function func_responce($payment_data, $system_settings)
    {
        $return = array("errors" => array(), "info" => array(), "data" => array(), "type" => "exit");

        foreach ($this->variables as $payment_var => $site_var) {
            $return["data"][$site_var] = isset($payment_data[$payment_var]) ? $this->ci->input->xss_clean($payment_data[$payment_var]) : "";
        }

        //// get status
        $return["data"]["status"] = 0;
        if (isset($return["data"]['transaction_status'])) {
            switch ($return["data"]['transaction_status']) {
                case "success": $return["data"]["status"] = 1;
                    break;
                default: $return["data"]["status"] = -1;
                    break;
            }
        }

        return $return;
    }

    /**
     * Return formatted system settings
     *
     * @return array
     */
    public function get_settings_map()
    {
        foreach ($this->settings as $param_id => $param_data) {
            $this->settings[$param_id]["name"] = l('system_field_' . $param_id, 'payments');
        }

        return $this->settings;
    }

    /**
     * Pull page of payment status
     *
     * @param string $host     system host
     * @param string $usepath  system path
     * @param string $postdata post data
     *
     * @return string
     */
    public function pullpage($host, $usepath, $postdata = "")
    {
        $fp = fsockopen($host, 80, $errno, $errstr, 60);
        if (!$fp) {
            print "$errstr ($errno)<br>\n";
        } else {
            fputs($fp, "POST $usepath HTTP/1.0\n");
            $strlength = strlen($postdata);
            fputs($fp, "Content-type: application/x-www-form-urlencoded\n");
            fputs($fp, "Content-length: " . $strlength . "\n\n");
            fputs($fp, $postdata . "\n\n");
            $output = "";
            while (!feof($fp)) {
                $output .= fgets($fp, 1024);
            }
            #close the socket connection
            fclose($fp);
        }

        return $output;
    }

    /**
     * Return currency code in payment system
     *
     * @param string $code currency code
     *
     * @return string
     */
    public function get_currency_num($code)
    {
        $codes = array(
            "AED" => "784", "AFN" => "971", "ALL" => "008", "AMD" => "051",
            "ANG" => "532", "AOA" => "973", "ARS" => "032", "AUD" => "036",
            "AWG" => "533", "AZN" => "944", "BAM" => "977", "BBD" => "052",
            "BDT" => "050", "BGN" => "975", "BHD" => "048", "BIF" => "108",
            "BMD" => "060", "BND" => "096", "BOB" => "068", "BOV" => "984",
            "BRL" => "986", "BSD" => "044", "BTN" => "064", "BWP" => "072",
            "BYR" => "974", "BZD" => "084", "CAD" => "124", "CDF" => "976",
            "CHF" => "756", "CLF" => "990", "CLP" => "152", "CNY" => "156",
            "COP" => "170", "COU" => "970", "CRC" => "188", "CVE" => "132",
            "CUC" => "931", "CUP" => "192", "CZK" => "203", "DJF" => "262",
            "DKK" => "208", "DOP" => "214", "DZD" => "012", "EGP" => "818",
            "ERN" => "232", "ETB" => "230", "EUR" => "978", "FJD" => "242",
            "FKP" => "238", "GBP" => "826", "GEL" => "981", "CHE" => "947",
            "CHF" => "756", "GHS" => "936", "CHW" => "948", "GIP" => "292",
            "GMD" => "270", "GNF" => "324", "GTQ" => "320", "GYD" => "328",
            "HKD" => "344", "HNL" => "340", "HRK" => "191", "HTG" => "332",
            "HUF" => "348", "INR" => "356", "IDR" => "360", "ILS" => "376",
            "IQD" => "368", "IRR" => "364", "ISK" => "352", "JMD" => "388",
            "JOD" => "400", "JPY" => "392", "KES" => "404", "KGS" => "417",
            "KHR" => "116", "KMF" => "174", "KPW" => "408", "KRW" => "410",
            "KWD" => "414", "KYD" => "136", "KZT" => "398", "LAK" => "418",
            "LBP" => "422", "LKR" => "144", "LRD" => "430", "LSL" => "426",
            "LTL" => "440", "LVL" => "428", "LYD" => "434", "MAD" => "504",
            "MDL" => "498", "MGA" => "969", "MKD" => "807", "MMK" => "104",
            "MNT" => "496", "MOP" => "446", "MRO" => "478", "MUR" => "480",
            "MVR" => "462", "MWK" => "454", "MXN" => "484", "MXV" => "979",
            "MYR" => "458", "MZN" => "943", "NAD" => "516", "NGN" => "566",
            "NIO" => "558", "NOK" => "578", "NPR" => "524", "NZD" => "554",
            "OMR" => "512", "PAB" => "590", "PEN" => "604", "PGK" => "598",
            "PHP" => "608", "PKR" => "586", "PLN" => "985", "PYG" => "600",
            "QAR" => "634", "RON" => "946", "RSD" => "941", "RUB" => "643",
            "RWF" => "646", "SAR" => "682", "SBD" => "090", "SCR" => "690",
            "SDG" => "938", "SEK" => "752", "SGD" => "702", "SHP" => "654",
            "SLL" => "694", "SOS" => "706", "SRD" => "968", "STD" => "678",
            "SVC" => "222", "SYP" => "760", "SZL" => "748", "THB" => "764",
            "TJS" => "972", "TMT" => "934", "TND" => "788", "TOP" => "776",
            "TRY" => "949", "TTD" => "780", "TWD" => "901", "TZS" => "834",
            "UAH" => "980", "UGX" => "800", "USD" => "840", "USN" => "997",
            "USS" => "998", "UYI" => "940", "UYU" => "858", "UZS" => "860",
            "VEF" => "937", "VND" => "704", "VUV" => "548", "WST" => "882",
            "XAF" => "950", "XAG" => "961", "XAU" => "959", "XBA" => "955",
            "XBB" => "956", "XBC" => "957", "XBD" => "958", "XCD" => "951",
            "XDR" => "960", "XFU" => "Nil", "XOF" => "952", "XPD" => "964",
            "XPF" => "953", "XPT" => "962", "XSU" => "994", "XTS" => "963",
            "XUA" => "965", "XXX" => "999", "YER" => "886", "ZAR" => "710",
            "ZMK" => "894", "ZWL" => "932",
        );

        return $codes[$code];
    }

}
