<?php

namespace Pg\Modules\Properties\Models;

/**
 * Properties module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Properties main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Properties
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Properties_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var array
     */
    protected $ci;

    /**
     * Properties data
     *
     * @var array
     */
    protected $attrs = array();

    /**
     * Categories (GUIDs)
     *
     * @var array
     */
    public $categories = array('property_types');

    /**
     * Properties (GUIDs)
     *
     * @var array
     */
    public $properties = array(
        'property_type_1',
        'property_type_2',
        'property_type_3',
    );

    /**
     * Prefix of property type
     *
     * @var string
     */
    public $property_prefix = 'property_type_';

    /**
     * Module GUID
     *
     * @var string
     */
    public $module_gid = 'data_properties';

    /**
     * Class constructor
     *
     * @return Properties_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return property name by GUID
     *
     * @param string  $ds_gid  property GUID in data source
     * @param integer $lang_id language idnetifier
     *
     * @return string/null
     */
    public function get_property($ds_gid, $lang_id = null)
    {
        if (!$ds_gid) {
            return;
        }

        if (!$lang_id) {
            $lang_id = $this->ci->session->userdata('lang_id');
        }

        return $this->ci->pg_language->ds->get_reference($this->module_gid, $ds_gid, $lang_id);
    }

    /**
     * Return categories properties as array
     *
     * @param array   $categories_gid categories GUIDs
     * @param integer $lang_id        language identifier
     *
     * @return array
     */
    public function get_categories($categories_gid, $lang_id = null)
    {
        if (!in_array($categories_gid, $this->categories)) {
            $categories_gid = current($this->categories);
        }
        if (!$lang_id) {
            $lang_id = $this->ci->session->userdata('lang_id');
        }
        $categories = $this->ci->pg_language->ds->get_reference($this->module_gid, $categories_gid, $lang_id);
        foreach ($categories['option'] as $category_gid => $category_name) {
            $category_enabled = $this->ci->pg_module->get_module_config('listings',
                                                                        'property_type_' . $category_gid . '_enabled');
            if ($category_enabled) {
                continue;
            }
            unset($categories['option'][$category_gid]);
        }

        return $categories['option'];
    }

    /**
     * Return properties of category
     *
     * @param string  $num     category value
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function get_properties($num, $lang_id = null)
    {
        if (!$lang_id) {
            $lang_id = $this->ci->session->userdata('lang_id');
        }
        $properties = $this->ci->pg_language->ds->get_reference($this->module_gid, $this->property_prefix . $num,
                                                                $lang_id);

        return $properties['option'];
    }

    /**
     * Return all properties
     *
     * @param string  $categories_gid categories GUIDs
     * @param integer $lang_id        language identifier
     *
     * @return array
     */
    public function get_all_properties($categories_gid, $lang_id = null)
    {
        if (!$lang_id) {
            $lang_id = $this->ci->session->userdata('lang_id');
        }
        $properties = array();
        $categories = $this->get_categories($categories_gid, $lang_id);
        foreach ($categories as $cid => $name) {
            $t = $this->ci->pg_language->ds->get_reference($this->module_gid, $this->property_prefix . $cid, $lang_id);
            $properties[$cid] = $t['option'];
        }

        return $properties;
    }

}
