<?php

namespace Pg\Modules\Social_networking\Models\Widgets;

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking vkontakte widgets model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Vkontakte_widgets_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Available widgets
     *
     * @var array
     */
    public $widget_types = array(
        'comments',
        'like',
        'share',
    );

    /**
     * Openapi is used
     *
     * @var boolean
     */
    public $openapi = false;

    /**
     * Page header data is loaded
     *
     * @var boolean
     */
    public $head_loaded = false;

    /**
     * Application key is used
     *
     * @var boolean
     */
    protected $has_key = false;

    /**
     * Class constructor
     *
     * @return Vkontakte_widgets_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return widgets data for page header
     *
     * @param array  $service_data service data
     * @param string $locale       locale code
     * @param array  $types        widget types
     *
     * @return string
     */
    public function get_header($service_data = array(), $locale = '', $types = array())
    {
        $header = '';
        $appid = isset($service_data['app_key']) ? $service_data['app_key'] : false;
        if ($appid) {
            $this->has_key = true;
        }
        if (!$this->openapi) {
            $header .= $appid ? ('<script type="text/javascript" src="https://vk.com/js/api/openapi.js"></script><script type="text/javascript">VK.init({apiId: ' . $appid . ', onlyWidgets: true});</script>') : '';
            $this->openapi = true;
        }
        $header .= '<script type="text/javascript" src="//vk.com/js/api/share.js" charset="windows-1251"></script><script type="text/javascript" src="https://vk.com/js/api/share.js?11" charset="windows-1251"></script>';
        $this->head_loaded = $header ? true : false;

        return $header;
    }

    /**
     * Return code of 'Like' widget
     *
     * @return string
     */
    public function get_like()
    {
        return $this->head_loaded && $this->openapi && $this->has_key ? '<div id="vk_like"></div><script type="text/javascript">VK.Widgets.Like("vk_like", {type: "button", height: 28});</script>' : '';
    }

    /**
     * Return code of 'Share' widget
     *
     * @return string
     */
    public function get_share()
    {
        return $this->head_loaded ? '<script type="text/javascript">document.write(VK.Share.button(false,{type: "button", text:"' . l('text_vk_share', 'social_networking', '', 'javascript') . '"}));</script>' : '';
    }

    /**
     * Return code of 'Comments' widhet
     *
     * @return string
     */
    public function get_comments()
    {
        return $this->head_loaded && $this->openapi && $this->has_key ? '<div id="vk_comments"></div><script type="text/javascript">VK.Widgets.Comments("vk_comments", {limit: 10, width: "496", attach: "*"});</script>' : '';
    }

}
