<?php

namespace Pg\Modules\Spam\Controllers;

/**
 * Spam networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Spam api side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Spam
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Spam extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Spam
     */
    public function __construct()
    {
        parent::__construct();

        /* if($this->session->userdata("auth_type") == "user"){
          show_404();
          } */
    }

    /**
     * Mark object as spam
     *
     * @return void
     */
    public function mark_as_spam()
    {
        $this->load->model("spam/models/Spam_alert_model");

        if ($this->session->userdata("auth_type") != "user") {
            return;
        }

        $post_data = $this->input->post("data");
        $post_data["id_object"] = intval($this->input->post("object_id"));
        $post_data["id_type"] = $this->input->post("type_gid");
        
        $id_poster = $this->input->post("id_poster");
        if (!empty($id_poster)) {
            $post_data["id_poster"] = intval($id_poster);
        } else {
            $post_data["id_poster"] = intval($this->session->userdata("user_id"));
        }        
        $validate_data = $this->Spam_alert_model->validate_alert(null, $post_data);
        if (!empty($validate_data["errors"])) {            
            $this->set_api_content("errors", $validate_data["errors"]);

            return;
        } else {            
            $id = $this->Spam_alert_model->save_alert(null, $validate_data["data"]);            
            $this->Spam_alert_model->set_format_settings(array("get_object" => true));
            $alert = $this->Spam_alert_model->get_alert_by_id($id, false);
            if ($alert["type"]["send_mail"]) {                
                $email = $this->pg_module->get_module_config("spam", "send_alert_to_email");
                if ($email) {
                    $data = array(
                        "poster" => $alert["poster"]["output_name"],
                        "type" => $alert["type"]["output_name"],
                        "object" => $alert["object"],
                        "message" => $alert["message"],
                    );
                    //send mail
                    $this->load->model("Notifications_model");
                    $this->Notifications_model->send_notification($email, "spam_object", $data);                    
                }
            }
            $this->set_api_content("messages", l("success_created_alert", "spam"));

            return;
        }
    }

    /**
     * Get status of spam alert
     *
     * @param integer $id alert identifier
     *
     * @return void
     */
    public function get_alert_status($id)
    {
        $this->load->model("spam/models/Spam_alert_model");
        $alert = $this->Spam_alert_model->get_alert_by_id($id, true);
        $this->set_api_content("messages", l("alert_" . $alert["spam_status"] . "_status", "spam"));

        return;
    }
    
    public function get_reasons()
    {
        $this->load->model("spam/models/Spam_reason_model");
        $lang_id = $this->pg_language->current_lang_id;
        $reference = $this->pg_language->ds->get_reference($this->Spam_reason_model->module_gid,
                                                           $this->Spam_reason_model->content[0], $lang_id);
        $this->view->assign("reasons", $reference);
    }

}
