<?php

namespace Pg\Modules\Spam\Controllers;

use Pg\Libraries\View;

/**
 * Spam networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Spam user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Spam
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Spam extends \Controller
{

    /**
     * Constructor
     *
     * @return Spam
     */
    public function __construct()
    {
        parent::__construct();

        /* if($this->session->userdata("auth_type") == "user"){
          show_404();
          } */
    }

    /**
     * Mark object as spam
     *
     * @return void
     */
    public function mark_as_spam()
    {
        $this->load->model("spam/models/Spam_alert_model");

        if ($this->session->userdata("auth_type") != "user") {
            return;
        }

        $post_data = $this->input->post("data");
        $post_data["id_object"] = intval($this->input->post("object_id"));
        $post_data["id_type"] = $this->input->post("type_gid");
        $post_data["id_poster"] = intval($this->session->userdata("user_id"));

        $is_owner = $this->input->post("is_owner", true);
        if ($is_owner) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_is_owner', 'spam'));
            redirect($_SERVER['HTTP_REFERER']);
        }

        $validate_data = $this->Spam_alert_model->validate_alert(null, $post_data);
        if (!empty($validate_data["errors"])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            redirect($_SERVER['HTTP_REFERER']);
        } else {
            $id = $this->Spam_alert_model->save_alert(null, $validate_data["data"]);
            $this->Spam_alert_model->set_format_settings(array("get_object" => true, 'get_reason' => true));
            $alert = $this->Spam_alert_model->get_alert_by_id($id, true);
            if ($alert["type"]["send_mail"]) {
                $email = $this->pg_module->get_module_config("spam", "send_alert_to_email");
                if ($email) {
                    $data = array(
                        "poster" => $alert["poster"]["output_name"],
                        "type" => $alert["type"]["output_name"],
                        "object_id" => $alert["object"]['id'],
                        "reason" => $alert["reason"],
                        "message" => strip_tags($alert["message"]),
                    );
                    //send mail
                    $this->load->model("Notifications_model");
                    $this->Notifications_model->send_notification($email, "spam_object", $data);
                }
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_created_alert", "spam"));
            redirect($_SERVER['HTTP_REFERER']);
        }
    }

    /**
     * Mark object as spam by ajax
     *
     * @return void
     */
    public function ajax_mark_as_spam()
    {
        $this->load->model("spam/models/Spam_alert_model");

        if ($this->session->userdata("auth_type") != "user") {
            return;
        }

        $post_data = $this->input->post("data");
        $post_data["id_object"] = intval($this->input->post("object_id"));
        $post_data["id_type"] = $this->input->post("type_gid");
        $post_data["id_poster"] = intval($this->session->userdata("user_id"));

        $is_owner = $this->input->post("is_owner", true);
        if ($is_owner) {
            $return["error"] = l('error_is_owner', 'spam');
            $this->view->assign($return);

            $this->view->render();

            return;
        }

        $validate_data = $this->Spam_alert_model->validate_alert(null, $post_data);
        if (!empty($validate_data["errors"])) {
            $return["error"] = implode("<br>", $validate_data["errors"]);

            $this->view->assign($return);

            $this->view->render();

            return;
        } else {
            $id = $this->Spam_alert_model->save_alert(null, $validate_data["data"]);
            $this->Spam_alert_model->set_format_settings(array("get_object" => true, 'get_reason' => true));
            $alert = $this->Spam_alert_model->get_alert_by_id($id, true);
            if ($alert["type"]["send_mail"]) {
                $email = $this->pg_module->get_module_config("spam", "send_alert_to_email");
                if ($email) {
                    $data = array(
                        "poster" => $alert["poster"]["output_name"],
                        "type" => $alert["type"]["output_name"],
                        "object_id" => $alert["object"]['id'],
                        "reason" => $alert["reason"],
                        "message" => strip_tags($alert["message"]),
                    );
                    //send mail
                    $this->load->model("Notifications_model");
                    $this->Notifications_model->send_notification($email, "spam_object", $data);
                }
            }
            $return["success"] = l("success_created_alert", "spam");

            $this->view->assign($return);

            $this->view->render();

            return;
        }
    }

    /**
     * Form to mark object as spam by ajax
     *
     * @return void
     */
    public function ajax_get_form()
    {
        $this->load->model("spam/models/Spam_type_model");

        $object_id = intval($this->input->post("object_id", true));
        $type_gid = $this->input->post("type_gid", true);

        $type = $this->Spam_type_model->get_type_by_id($type_gid, true);
        $this->view->assign("data", $type);

        $is_owner = $this->input->post("is_owner", true);
        $this->view->assign("is_spam_owner", $is_owner);

        if ($type["form_type"] == "select_text") {
            $this->load->model("spam/models/Spam_reason_model");
            $lang_id = $this->pg_language->current_lang_id;
            $reference = $this->pg_language->ds->get_reference($this->Spam_reason_model->module_gid,
                                                               $this->Spam_reason_model->content[0], $lang_id);
            $this->view->assign("reasons", $reference);
        }

        $this->view->assign("object_id", $object_id);

        $this->view->render("mark_as_spam_form", "user", "spam");

        return;
    }

}
