'use strict';

function paginationH(optionArr) {

	var _self = this;

	this.properties = {
		baseContainer: '',
		baseContainerObj: {},
		btnLeftSelector: '.btn-left',
		btnLeftObj: {},
		btnRightSelector: '.btn-right',
		btnRightObj: {},
		childrenObj: {},
		currentPageSelector: '.current-page',
		currentPageObj: {},
		itemsPerPage: 1,
		parent: '',
		parentObj: {},
		page: 1,
		pagesOffsets: [],
		itemsOffsets: [],
		pagesNum: 1,
		pageText: '',
		pageTextSelector: '.page-text',
		pageTextObj: {},
		showItemsPerPage: true,
		totalPagesSelector: '.total-pages',
		totalPagesObj: {}
	};

	this.errors = {};

	var dirtyHackNum = 40;
	var autoScrollInProcess = false;

	var initObjects = function () {
		_self.properties.baseContainerObj = $(_self.properties.baseContainer + ':last');
		_self.properties.parentObj = $(_self.properties.parent);
		_self.properties.childrenObj = $(_self.properties.baseContainerObj.children());
		_self.properties.btnLeftObj = $(_self.properties.btnLeftSelector, _self.properties.parentObj);
		_self.properties.btnRightObj = $(_self.properties.btnRightSelector, _self.properties.parentObj);
		_self.properties.currentPageObj = $(_self.properties.currentPageSelector, _self.properties.parentObj);
		_self.properties.totalPagesObj = $(_self.properties.totalPagesSelector, _self.properties.parentObj);
		_self.properties.pageTextObj = $(_self.properties.pageTextSelector);
	};

	var bindEvents = function () {
		_self.properties.btnLeftObj.on('click', function () {
			if (_self.properties.page > 1) {
				prevPage();
			}
		});
		_self.properties.btnRightObj.on('click', function () {
			if (_self.properties.page < _self.properties.pagesNum) {
				nextPage();
			}
		});

		_self.properties.baseContainerObj.on('scroll', function() {
			if(!autoScrollInProcess) {
				setPage(getPageByOffset(_self.properties.baseContainerObj.scrollLeft()));
			}
		});
		
	};

	var nextPage = function () {
		if (_self.properties.page >= _self.properties.pagesNum) {
			return false;
		}
		if(!autoScrollInProcess) {
			moveTo(getPageOffset(_self.properties.page + 1)).then(function() {
				setPage(_self.properties.page + 1);
			});
		}
	};

	var prevPage = function () {
		if (_self.properties.page <= 0) {
			console.error('Wrong page');
			return false;
		}
		if(!autoScrollInProcess) {
			moveTo(getPageOffset(_self.properties.page - 1)).then(function() {
				setPage(_self.properties.page - 1);
			});
		}
	};

	var getPageOffset = function(page) {
		return _self.properties.pagesOffsets[page];
	};

	var setPage = function (page) {
		_self.properties.page = page;
		if(!_self.properties.showItemsPerPage) {
			_self.properties.currentPageObj.html(_self.properties.page);
		}
	};
	
	var moveTo = function(offset) {
		var dfd = new $.Deferred();
		autoScrollInProcess = true;
		_self.properties.baseContainerObj.animate(
			{scrollLeft: offset + "px"}, 
			'', 
			function() {
				autoScrollInProcess = false;
				dfd.resolve();
			}
		);
		return dfd;
	};
	
	var getPagesOffsets = function(itemsPerPage) {
		var i = 0;
		var offset = 0;
		var page = 1;
		var item = 0;
		_self.properties.pagesOffsets[page] = 0;
		_self.properties.itemsOffsets[item] = 0;
		_self.properties.childrenObj.each(function() {
			if(++i >= itemsPerPage) {
				i = 0;
				_self.properties.pagesOffsets[++page] = Math.round(offset);
			}
			_self.properties.itemsOffsets[item] = Math.round($(this).position().left);
			offset += _self.properties.itemsOffsets[item];
			item++;
		});
		
		_self.properties.pagesNum = _self.properties.pagesOffsets.length - 1;
		return _self.properties.pagesOffsets;
	};

	var getPageByOffset = function(offset) {
		if(offset <= 0) {
			return 1;
		}
		var min;
		var max;
		for(var i = 1; i < _self.properties.pagesNum; i++) {
			min = _self.properties.pagesOffsets[i];
			max = _self.properties.pagesOffsets[i + 1];
			if(offset >= min && offset < max - dirtyHackNum) {
				return i;
			};
		}
		return _self.properties.pagesNum;
	};

	this.Init = function (options) {
		_self.properties = $.extend(_self.properties, options);
		initObjects();
		getPagesOffsets(_self.properties.itemsPerPage);
		bindEvents();
		_self.properties.totalPagesObj.html(_self.properties.pagesNum);
		if(_self.properties.showItemsPerPage) {
			if(_self.properties.itemsPerPage > _self.properties.childrenObj.length) {
				_self.properties.currentPageObj.html(_self.properties.childrenObj.length);
			} else {
				_self.properties.currentPageObj.html(_self.properties.itemsPerPage);
			}
			_self.properties.totalPagesObj.html(_self.properties.childrenObj.length);
		}
	};
	_self.Init(optionArr);
}
