<?php

namespace Pg\Modules\Start\Models;

/**
 * Start module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Start Main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Start
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Start_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Module GID
     *
     * @var string
     */
    const MODULE_GID = 'start';

    /**
     * Upload GID
     *
     * @var string
     */
    const UPLOAD_GID = 'start-user-logo';

    /**
     * User object properties
     *
     * @var array
     */
    public $fields_user = array(
        'fname',
        'sname',
        'contact_email',
        'contact_phone',
        'address',
        'company_name',
        // 'lat',
        //  'lon',
        'user_logo'
    );

    /**
     * View user info object properties
     * 
     * @var array
     */
    public $fields_user_view = array(
        'fname',
        'sname',
        'contact_email',
        'contact_phone',
        'address',
        'company_name'
    );

    /**
     * Display sections of user profile
     * 
     * @var array
     */
    public $display_sections = array("company" => 0, "reviews" => 1, "contacts" => 1, "map" => 1);

    /**
     * Constructor
     *
     * @return Start_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Clear temp directories by cron
     * @return boolean
     */
    public function clearTempDirs()
    {
        $this->ci->config->load('captcha_settings');
        $captcha_settings = $this->ci->config->item('captcha_settings');
        $dirs = array(
            SITE_PHYSICAL_PATH . TRASH_FOLDER,
            $captcha_settings['img_path'],
        );
        $skip = array('.htaccess', 'index.html',);
        foreach ($dirs as $dir) {
            foreach (new \DirectoryIterator($dir) as $file_info) {
                if ($file_info->isDot() || in_array($file_info->getBasename(), $skip)) {
                    continue;
                } elseif ($file_info->isDir()) {
                    $this->filesystem->rmdir($file_info->getRealPath());
                } else {
                    unlink($file_info->getRealPath());
                }
            }
        }

        return true;
    }

    /**
     * Return display sections of user profile
     * 
     * @param array $data user data
     * @return array
     */
    public function get_display_sections($data)
    {
        if (!$data['is_map'] || !$this->ci->pg_module->is_module_active("geomap"))
            $this->display_sections["map"] = 0;
        if (!$data["review_count"])
            $this->display_sections["reviews"] = 0;
        return $this->display_sections;
    }

    /**
     * Return default user on site
     *
     * @return array
     */
    public function getUserDefaultOnSite()
    {
        static $user = null;
        
        if ($user !== null) {
            return $user;
        }
        
        $user = [];
        
        if ($this->ci->pg_module->is_module_active("users")) {
            return $user;
        }

        foreach ($this->fields_user as $field) {
            $user[$field] = $this->ci->pg_module->get_module_config(self::MODULE_GID, $field);
        }

        return $this->formatUserDefault($user);
    }

    /**
     * Edit Contact info user on site
     * 
     * @return array
     */
    public function getEditUserInfo($user)
    {
        $user_content = array();
        if (!empty($user)) {
            foreach ($this->fields_user as $key => $value) {
                $user_content[$key]['title'] = l('field_start_user_' . $value, 'start');
                $user_content[$key]['value'] = $user[$value];
                $user_content[$key]['name'] = $value;
                $user_content[$key]['placeholder'] = l('field_start_user_' . $value, 'start');
            }
        }
        return $user_content;
    }

    /**
     * Return Contact info user on site
     * 
     * @return array
     */
    public function getViewUserInfo($user)
    {
        $user_content = array();
        if (!empty($user)) {
            foreach ($this->fields_user_view as $key => $value) {
                $user_content[$key]['title'] = l('field_start_user_' . $value, 'start');
                $user_content[$key]['value'] = $user[$value];
            }
        }
        return $user_content;
    }

    /**
     * Validate data of user default object
     *
     * @param type $user
     *
     * @return array
     */
    public function validateUserDefault($user = array())
    {
        $return = array('errors' => array(), 'data' => array());
        $this->ci->config->load("reg_exps", TRUE);
        $is_moderation_installed = $this->ci->pg_module->is_module_active("moderation");

        if (isset($user['fname'])) {
            $return["data"]["fname"] = trim(strip_tags($user["fname"]));
            if (empty($return["data"]["fname"])) {
                $return["errors"][] = l("error_empty_fname", "start");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['fname']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_fname', 'start'), l("error_badwords_field", "start"));
                    }
                }
            }
        }
        if (isset($user['sname'])) {
            $return["data"]["sname"] = trim(strip_tags($user["sname"]));
            if (empty($return["data"]["sname"])) {
                $return["errors"][] = l("error_empty_sname", "start");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['sname']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_sname', 'start'), l("error_badwords_field", "start"));
                    }
                }
            }
        }
        if (isset($user["contact_email"])) {
            $return["data"]["contact_email"] = trim(strip_tags($user["contact_email"]));
            $email_expr = $this->ci->config->item("email", "reg_exps");
            if (!empty($return["data"]["contact_email"]) && !preg_match($email_expr, $return["data"]["contact_email"])) {
                $return["errors"][] = l("error_contact_email_incorrect", "start");
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['contact_email']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_contact_email', 'start'), l("error_badwords_field", "start"));
                    }
                }
            }
        }
        if (isset($user["contact_phone"])) {
            $return["data"]["contact_phone"] = trim(strip_tags($user["contact_phone"]));
            if (!empty($return["data"]["contact_phone"])) {
                $this->ci->load->helper("start");
                $phone_format = get_phone_format('regexp');
                if ($phone_format !== false && !preg_match($phone_format, $return["data"]["contact_phone"])) {
                    $return["errors"][] = l("error_contact_phone_incorrect", "start");
                } else {
                    if ($is_moderation_installed) {
                        $this->ci->load->model("moderation/models/Moderation_badwords_model");
                        $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['contact_phone']);
                        if ($bw_count) {
                            $return['errors'][] = str_replace('[field]', l('field_contact_phone', 'start'), l("error_badwords_field", "start"));
                        }
                    }
                }
            }
        }
        if (isset($user["address"])) {
            $return["data"]["address"] = trim(strip_tags($user["address"]));
            if (!empty($return["data"]["address"])) {
                if ($is_moderation_installed) {
                    $this->ci->load->model("moderation/models/Moderation_badwords_model");
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type_text, $return['data']['address']);
                    if ($bw_count) {
                        $return['errors'][] = str_replace('[field]', l('field_address', 'start'), l("error_badwords_field", "start"));
                    }
                }
            }
        }
        if (isset($user['company_name'])) {
            $return["data"]["company_name"] = trim(strip_tags($user["company_name"]));
        }
        if (isset($user["lat"])) {
            $return["data"]["lat"] = (float) $user["lat"];
        }

        if (isset($user["lon"])) {
            $return["data"]["lon"] = (float) $user["lon"];
        }

        if (isset($_FILES['user_logo']) && is_array($_FILES['user_logo']) && is_uploaded_file($_FILES['user_logo']["tmp_name"])) {
            $this->ci->load->model("Uploads_model");
            $validate_image = $this->ci->Uploads_model->validate_upload(self::UPLOAD_GID, 'user_logo');
            if (!empty($validate_image["error"])) {
                $return["errors"] = array_merge((array) $return["errors"], (array) $validate_image["error"]);
            }
        }

        return $return;
    }

    /**
     * Format data of user default object
     *
     * @param type $user
     *
     * @return array
     */
    private function formatUserDefault($user = array())
    {
        $user['name'] = $user['fname'];
        $user['output_name'] = trim($user['fname'] . ' ' . $user['sname']);
        $user['user_type'] = 'agent';
        $user['user_type_str'] = l('start_user_type_' . $user['user_type'], 'start');
        $user['id'] = '0';
        $user['status'] = '1';
        $user['location'] = $user['address'];
        $module = $this->pg_module->get_module_by_gid(self::MODULE_GID);
        $user['date_created'] = $user['date_modified'] = $module['date_add'];
        
        if ($module['date_update'] !== '0000-00-00 00:00:00') {
            $user['date_modified'] = $module['date_add'];
        }
        
        if ($this->ci->pg_module->is_module_active("listings")) {
            $this->ci->load->model("Listings_model");
            $where = array();
            $where["status"] = 1;
            foreach ($this->ci->Listings_model->operations_arr as $key => $type) {
                $where['id_type'] = $key;
                $user['listings_for_' . $type . '_count'] = $this->ci->Listings_model->get_listings_count($where);
            }
        }
        $this->ci->load->model("Uploads_model");
        $user['media']['user_logo'] = $this->ci->Uploads_model->format_upload(self::UPLOAD_GID, '', $user['user_logo']);

        return $user;
    }

    /**
     * Save data of user default object
     *
     * @param type $user
     *
     * @return void
     */
    public function setUserDefault($user = array())
    {
        if (!empty($_FILES['user_logo']) && is_uploaded_file($_FILES['user_logo']['tmp_name'])) {
            $this->ci->load->model('Uploads_model');
            if ($user['user_logo']) {
                $this->deleteUserLogo();
            }
            $img_return = $this->ci->Uploads_model->upload(self::UPLOAD_GID, '', 'user_logo');
            if (empty($img_return["errors"])) {
                $user['user_logo'] = $img_return["file"];
            }
        }

        foreach ($user as $setting => $value) {
            $this->pg_module->set_module_config(self::MODULE_GID, $setting, $value);
        }
    }

    /**
     * Encode password string
     * 
     * @param string $password password string
     * @return string
     */
    public function encode_password($password)
    {
        return md5($password);
    }

    /**
     * Return available pages of posted banners
     *
     * Callback method for banners module
     *
     * @return array
     */
    public function _banner_available_pages()
    {
        $return[] = array("link" => "start/index", "name" => l('header_start_page', 'start'));
        $return[] = array("link" => "start/homepage", "name" => l('header_homepage', 'start'));

        return $return;
    }

    /**
     * Return code of statistics block
     *
     * Callback method for dynamic blocks module
     *
     * @param array   $params block parameters
     * @param string  $view   view mode
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_get_stat_block($params = array(), $view = 'default', $width = 100)
    {
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_statistics', 'user', 'start');
    }

    /**
     * Return content of main search form block
     *
     * Callback method for dynamic blocks module
     *
     * @param array   $params block parameters
     * @param string  $view   view mode
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_get_search_form($params = array(), $view = 'default', $width = 100)
    {
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_search_form', 'user', 'start');
    }

    /**
     * Validate system settings
     *
     * @param array $data options data
     *
     * @return array
     */
    public function validate_system_settings($data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (isset($data['site_contacts'])) {
            foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                $return['data']['site_contacts'][$lid] = trim($data['site_contacts'][$lid]);
            }
        }

        return $return;
    }

    /**
     * Delete user icon
     *
     * @return void
     */
    public function deleteUserLogo()
    {
        $this->ci->load->model("Uploads_model");
        $user_logo = $this->ci->pg_module->get_module_config(self::MODULE_GID, 'user_logo');
        if (!empty($user_logo)) {
            $this->ci->Uploads_model->delete_upload(self::UPLOAD_GID, 'default', $user_logo);
            $this->pg_module->set_module_config(self::MODULE_GID, 'user_logo', '');
        }
    }

}
