<?php

namespace Pg\Modules\Subscriptions\Models;

/**
 * Subscriptions module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('SUBSCRIPTIONS_TYPES_TABLE')) {
    define('SUBSCRIPTIONS_TYPES_TABLE', DB_PREFIX . 'subscriptions_types');
}

/**
 * Subscriptions types model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Subscriptions
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Subscriptions_types_model extends \Model
{

    /**
     * Link to Code Igniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Subscriptions type object properties in data source
     *
     * @var array
     */
    protected $attrs = array('id', 'gid', 'module', 'model', 'method');

    /**
     * Class constructor
     *
     * @return Subscriptions_types_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return subscriptions types objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param array   $order_by          sorting data
     * @param array   $params            sql criteria of query to data source
     * @param array   $filter_object_ids filter types identifiers
     *
     * @return array
     */
    public function get_subscriptions_types_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(", ", $this->attrs))->from(SUBSCRIPTIONS_TYPES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->attrs)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $data = array();
        $results = $this->ci->db->get()->result_array();
        $data[] = array('id' => 0, 'name' => l('without_content', 'subscriptions'));
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $this->format_subscriptions_type($r, true);
            }
        }

        return $data;
    }

    /**
     * Format subscription type object
     *
     * @param array   $data      subscription type data
     * @param boolean $get_langs use data related to languages
     *
     * @return array
     */
    public function format_subscriptions_type($data, $get_langs = false)
    {
        $data["name_i"] = "subscriptions_type_" . $data["gid"];
        $data["name"] = ($get_langs) ? (l($data["name_i"], $data['module'])) : "";

        return $data;
    }

    /**
     * Return subscription type object by identifier
     *
     * @param integer $id type identifier
     *
     * @return array
     */
    public function get_subscriptions_type_by_id($id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->attrs))->from(SUBSCRIPTIONS_TYPES_TABLE)->where("id", $id)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return subscription type object by guid
     *
     * @param string $gid type guid
     *
     * @return array
     */
    public function get_subscriptions_type_by_gid($gid)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->attrs))->from(SUBSCRIPTIONS_TYPES_TABLE)->where("gid", $gid)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return subscription content
     *
     * @param integer $id      subscription identifier
     * @param integer $lang_id language identifier
     *
     * @return string
     */
    public function get_subscriptions_type_content($id, $lang_id = 1)
    {
        $st_object = $this->get_subscriptions_type_by_id($id);
        $function_result = array();
        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->get_default_lang_id();
        }
        if (!empty($st_object['module']) && !empty($st_object['model']) && !empty($st_object['method'])) {
            if (!$this->is_method_callable($st_object["module"], $st_object["model"], $st_object["method"])) {
                $errors[] = l('error_function_invalid', 'cronjob');
            } else {
                $model_url = $st_object["module"] . "/models/" . $st_object["model"];
                $model_path = MODULEPATH . strtolower($model_url) . EXT;
                $this->ci->load->model($model_url);

                $function_result = call_user_func_array(array(&$this->ci->$st_object["model"], $st_object["method"]),
                                                        array($lang_id));
            }
        }

        return $function_result;
    }

    /**
     * Check method is callable
     *
     * @param string $module module name
     * @param string $model  model name
     * @param string $method name
     *
     * @return boolean
     */
    protected function is_method_callable($module, $model, $method)
    {
        $result = false;

        $model_url = $module . "/models/" . $model;
        $model_path = MODULEPATH . strtolower($model_url) . EXT;

        if (file_exists($model_path)) {
            $this->ci->load->model($model_url);
            $object = array($this->ci->$model, $method);
            $result = is_callable($object);
        }

        return $result;
    }

    /**
     * Save subscription type object to data source
     *
     * @param integer $id   subscription type identifier
     * @param array   $data subscription type data
     *
     * @return integer
     */
    public function save_subscriptions_type($id, $data)
    {
        if (is_null($id)) {
            $this->ci->db->insert(SUBSCRIPTIONS_TYPES_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(SUBSCRIPTIONS_TYPES_TABLE, $data);
        }

        return $id;
    }

    /**
     * Remove subscription type object by guid
     *
     * @param string $gid subscription type guid
     *
     * @return void
     */
    public function delete_subscriptions_type_by_gid($gid)
    {
        $this->ci->db->where("gid", $gid);
        $this->ci->db->delete(SUBSCRIPTIONS_TYPES_TABLE);

        return;
    }

}
