<?php

namespace Pg\Modules\Subscriptions\Models;

/**
 * Subscriptions module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('SUBSCRIPTIONS_USERS_TABLE')) {
    define('SUBSCRIPTIONS_USERS_TABLE', DB_PREFIX . 'subscriptions_users');
}

/**
 * Subscriptions users model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Subscriptions
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Subscriptions_users_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Subscriber object properties in data source
     *
     * @var array
     */
    protected $attrs = array('id', 'id_user', 'id_subscription');

    /**
     * Class constructor
     *
     * @return Subscriptions_users_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return subscribers objects by subscription
     *
     * @param integer $id_subscription subscription identifier
     * @param integer $page            page of subscribers
     * @param integer $items_on_page   subscribers per page
     *
     * @return array
     */
    public function get_users_by_id_subscription($id_subscription, $page = null, $items_on_page = 1000)
    {
        $this->ci->load->model('Users_model');

        $this->ci->db->select('id_user, email, lang_id')->from(SUBSCRIPTIONS_USERS_TABLE);
        $this->ci->db->join(USERS_TABLE, USERS_TABLE . '.id = ' . SUBSCRIPTIONS_USERS_TABLE . '. 	id_user');
        $this->ci->db->where('id_subscription', $id_subscription);
        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();

        return $results;
    }

    /**
     * Subscribe user to subscriptions
     *
     * @param integer $id_user       user identifier
     * @param array   $subscriptions subscriptions objects
     *
     * @return void
     */
    public function save_user_subscriptions($id_user, $subscriptions)
    {
        $this->ci->db->where("id_user", $id_user);
        $this->ci->db->delete(SUBSCRIPTIONS_USERS_TABLE);
        if ($subscriptions) {
            foreach ($subscriptions as $key => $value) {
                $data['id_user'] = $id_user;
                $data['id_subscription'] = intval($value);
                $this->ci->db->insert(SUBSCRIPTIONS_USERS_TABLE, $data);
                $id = $this->ci->db->insert_id();
            }
        }
        $this->save_auto_subscriptions($id_user);
    }

    /**
     * Auto subscribe user to subscriptions
     *
     * @param integer $id_user       user identifier
     * @param array   $subscriptions subscriptions objects
     *
     * @return void
     */
    public function save_auto_subscriptions($id_user)
    {
        $this->ci->load->model('Subscriptions_model');
        $this->ci->load->model('Users_model');
        $this->ci->load->model('users/models/Users_types_model');
        $subscriber_types = array($this->ci->Users_types_model->getTypeById((int)$id_user)['name']);
        $query = "subscribe_type = 'auto' AND (";
        foreach ($subscriber_types as $user_type) {
            $query .= "subscriber_type='$user_type' OR ";
        }
        $query = mb_substr($query, 0, -4) . ')';
        $params = array(
            'where_sql' => array($query),
        );
        $auto_subscriptions = $this->ci->Subscriptions_model->get_subscriptions_list(null, null, null, $params);
        if ($auto_subscriptions) {
            foreach ($auto_subscriptions as $key => $value) {
                $data['id_user'] = $id_user;
                $data['id_subscription'] = intval($value['id']);
                $this->ci->db->insert(SUBSCRIPTIONS_USERS_TABLE, $data);
            }
        }
    }

    /**
     * Return user's subscriptions identifiers
     *
     * @param integer $id_user user identifier
     *
     * @return array
     */
    public function get_subscriptions_by_id_user($id_user)
    {
        $return = array();
        $this->ci->db->select(implode(", ", $this->attrs))->from(SUBSCRIPTIONS_USERS_TABLE);
        $this->ci->db->where('id_user', $id_user);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $return[$r['id_subscription']] = 1;
            }
        }

        return $return;
    }

    /**
     * Return number subscribers of subscription
     *
     * @param integer $id_subscription subscription identifier
     *
     * @return integer
     */
    public function get_subscription_users_count($id_subscription)
    {
        $this->ci->load->model('Users_model');

        $this->ci->db->select('COUNT(*) AS cnt')->from(SUBSCRIPTIONS_USERS_TABLE);
        $this->ci->db->join(USERS_TABLE, USERS_TABLE . '.id = ' . SUBSCRIPTIONS_USERS_TABLE . '.id_user');
        $this->ci->db->where('id_subscription', $id_subscription);

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

}
