<?php

namespace Pg\Modules\Users\Models;

/**
 * Users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users Export Model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_export_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Transfer configuration
     *
     * @var array
     */
    protected $fields_for_transfer = array();

    /**
     * Admin export form from field editor (GUID)
     *
     * @var string
     */
    public $admin_export_form_gid = 'admin_export_form';

    /**
     * Admin export form of all users from field editor (GUID)
     *
     * @var string
     */
    public $admin_export_form_user_gid = 'admin_export_form_13';

    /**
     * Admin export form of agent from field editor (GUID)
     *
     * @var string
     */
    public $admin_export_form_agent_gid = 'admin_export_form_14';

    /**
     * Admin export form of company from field editor (GUID)
     *
     * @var string
     */
    public $admin_export_form_company_gid = 'admin_export_form_15';

    /**
     * Admin export form of private person from field editor (GUID)
     *
     * @var string
     */
    public $admin_export_form_private_gid = 'admin_export_form_16';

    /**
     * User export form from field editor (GUID)
     *
     * @var string
     */
    public $user_export_form_gid = 'user_export_form';

    /**
     * User export form of all users from field editor (GUID)
     *
     * @var string
     */
    public $user_export_form_user_gid = 'user_export_form_13';

    /**
     * User export form of agent from field editor (GUID)
     *
     * @var string
     */
    public $user_export_form_agent_gid = 'user_export_form_14';

    /**
     * User export form of company from field editor (GUID)
     *
     * @var string
     */
    public $user_export_form_company_gid = 'user_export_form_15';

    /**
     * User export form of private person from field editor (GUID)
     *
     * @var string
     */
    public $user_export_form_private_gid = 'user_export_form_16';

    /**
     * Constructor
     *
     * return Users_export_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->CI = &get_instance();
        $this->CI->load->model("Users_model");

        $this->fields_for_transfer = array(
            array("field_name" => "id", "field_type" => "int", "name" => l("field_id", "users"), "required" => false),
            array("field_name" => "date_created", "field_type" => "text", "name" => l("field_date_created", "users"), "required" => false),
            array("field_name" => "date_modified", "field_type" => "text", "name" => l("field_date_modified", "users"), "required" => false),
            array("field_name" => "email", "field_type" => "text", "name" => l("field_email", "users"), "required" => true),
            array("field_name" => "unique_name", "field_type" => "text", "name" => l("field_unique_name", "users"), "required" => false),
            array("fk" => "user_type_str", "field_name" => "user_type", "field_type" => "select", "options" => array("header" => l("field_user_type",
                                                                                                                                   "users")), "required" => true),
//			array("field_name"=>"password", "field_type"=>"text", "name"=>l("field_password", "users"), "required"=>true),
            array("fk" => "status_output", "field_name" => "status", "field_type" => "select", "options" => array("header" => l("field_status",
                                                                                                                                "users")), "required" => false),
            array("field_name" => "confirm", "field_type" => "int", "name" => l("field_confirm", "users"), "required" => false),
            array("field_name" => "confirm_code", "field_type" => "text", "name" => l("field_confirm_code", "users"), "required" => false),
            array("field_name" => "fname", "field_type" => "text", "name" => l("field_fname", "users"), "required" => true),
            array("field_name" => "sname", "field_type" => "text", "name" => l("field_sname", "users"), "required" => true),
            array("field_name" => "company_name", "field_type" => "text", "name" => l("field_name", "users"), "required" => false),
            array("fk" => "language_output", "field_name" => "lang_id", "field_type" => "select", "options" => array("header" => l("field_language",
                                                                                                                                   "users")), "required" => false),
            array("fk" => "currency_output", "field_name" => "id_currency", "field_type" => "select", "options" => array("header" => l("field_currency",
                                                                                                                                       "users")), "required" => false),
//			array("field_name"=>"user_open_id", "field_type"=>"text", "name"=>l("field_open_id", "users"), "required"=>false),
            array("field_name" => "user_logo", "field_type" => "file", "name" => l("field_user_logo", "users"), "required" => false),
            array("fk" => "group_output", "field_name" => "group_id", "field_type" => "select", "options" => array("header" => l("field_group",
                                                                                                                                 "users")), "required" => false),
            array("field_name" => "account", "field_type" => "text", "name" => l("field_account", "users"), "required" => false),
            array("fk" => "country", "field_name" => "id_country", "field_type" => "select", "options" => array("header" => l("field_country",
                                                                                                                              "users")), "required" => false),
            array("fk" => "region", "field_name" => "id_region", "field_type" => "select", "options" => array("header" => l("field_state",
                                                                                                                            "users")), "required" => false),
            array("fk" => "city", "field_name" => "id_city", "field_type" => "select", "options" => array("header" => l("field_city",
                                                                                                                        "users")), "required" => false),
            array("fk" => "district", "field_name" => "id_district", "field_type" => "select", "options" => array("header" => l("field_district",
                                                                                                                                "users")), "required" => false),
            array("field_name" => "phone", "field_type" => "text", "name" => l("field_phone", "users"), "required" => false),
            array("field_name" => "contact_email", "field_type" => "text", "name" => l("field_contact_email", "users"), "required" => false),
            array("field_name" => "contact_phone", "field_type" => "text", "name" => l("field_contact_phone", "users"), "required" => false),
            array("field_name" => "contact_info", "field_type" => "text", "name" => l("field_contact_info", "users"), "required" => false),
            array("field_name" => "address", "field_type" => "text", "name" => l("field_address", "users"), "required" => false),
            array("field_name" => "postal_code", "field_type" => "text", "name" => l("field_postal_code", "users"), "required" => false),
            array("field_name" => "web_url", "field_type" => "text", "name" => l("field_web_url", "users"), "required" => false),
            array("field_name" => "featured_end_date", "field_type" => "text", "name" => l("field_featured_end_date",
                                                                                           "users"), "required" => false),
            array("field_name" => "show_logo_end_date", "field_type" => "text", "name" => l("field_show_logo_end_date",
                                                                                            "users"), "required" => false),
            array("fk" => "working_days_str", "field_name" => "working_days", "field_type" => "select", "options" => array("header" => l("field_working_days",
                                                                                                                                         "users")), "required" => false),
            array("fk" => "working_hours_begin_text", "field_name" => "working_hours_begin", "field_type" => "select", "options" => array("header" => l("field_working_hours_begin",
                                                                                                                                                        "users")), "required" => false),
            array("fk" => "working_hours_end_text", "field_name" => "working_hours_end", "field_type" => "select", "options" => array("header" => l("field_working_hours_end",
                                                                                                                                                    "users")), "required" => false),
            array("fk" => "lunch_time_begin_text", "field_name" => "lunch_time_begin", "field_type" => "select", "options" => array("header" => l("field_lunch_time_begin",
                                                                                                                                                  "users")), "required" => false),
            array("fk" => "lunch_time_end_text", "field_name" => "lunch_time_end", "field_type" => "select", "options" => array("header" => l("field_lunch_time_end",
                                                                                                                                              "users")), "required" => false),
            array("field_name" => "lat", "field_type" => "text", "name" => l("field_lat", "users"), "required" => false),
            array("field_name" => "lon", "field_type" => "text", "name" => l("field_lon", "users"), "required" => false),
            array("fk" => "company.output_name", "field_name" => "agent_company", "field_type" => "select", "options" => array("header" => l("field_agent_company",
                                                                                                                                             "users")), "required" => false),
            array("field_name" => "agent_status", "field_type" => "int", "name" => l("field_agent_status", "users"), "required" => false),
//			array("field_name"=>"listings_for_sale_count", "field_type"=>"int", "name"=>l("field_listings_for_sale_count", "users"), "required"=>false),
//			array("field_name"=>"listings_for_rent_count", "field_type"=>"int", "name"=>l("field_listings_for_rent_count", "users"), "required"=>false),
            array("field_name" => "twitter", "field_type" => "text", "name" => l("field_twitter", "users"), "required" => false),
            array("field_name" => "facebook", "field_type" => "text", "name" => l("field_facebook", "users"), "required" => false),
            array("field_name" => "vkontakte", "field_type" => "text", "name" => l("field_vkontakte", "users"), "required" => false),
//			array("field_name"=>"views", "field_type"=>"int", "name"=>l("field_views", "users"), "required"=>false),
//			array("field_name"=>"banned", "field_type"=>"int", "name"=>l("field_banned", "users"), "required"=>false),
            array('field_name' => 'seolink', 'field_type' => 'text', 'name' => l('field_seolink', 'users'), 'required' => false),
        );
    }

    /**
     * Return custom fields
     *
     * Callback method for module export
     *
     * @return array
     */
    public function callback_get_fields()
    {
        $this->CI->load->model('Field_editor_model');

        $user_types = $this->CI->Users_model->getUserTypes();
        $field_editor_types = $user_types;
        array_unshift($field_editor_types, 'user');
        foreach ($field_editor_types as $editor_type_gid) {
            $this->CI->Field_editor_model->initialize($editor_type_gid);

            // Sections fields
            $fe_sections = $this->CI->Field_editor_model->get_section_list();
            if (!empty($fe_sections)) {
                $where['where_in']['section_gid'] = array_keys($fe_sections);
                $where['where']['editor_type_gid'] = $editor_type_gid;
                $fe_fields = $this->CI->Field_editor_model->get_fields_list($where);
                foreach ($fe_fields as $field_name => $field_data) {
                    if (!isset($fe_sections[$field_data['section_gid']])) {
                        continue;
                    }
                    $section_prefix = $fe_sections[$field_data['section_gid']]['name'] . ' - ';
                    switch ($field_data['field_type']) {
                        case 'select':
                            $fe_fields[$field_name]['options']['header'] = $section_prefix . $fe_fields[$field_name]['options']['header'];
                            break;
                        default:
                            $fe_fields[$field_name]['name'] = $section_prefix . $fe_fields[$field_name]['name'];
                            break;
                    }
                }
                $this->fields_for_transfer = array_merge($this->fields_for_transfer, $fe_fields);
            }
        }

        foreach ($this->fields_for_transfer as $i => $field) {
            switch ($field["field_type"]) {
                case "select":
                    switch ($field['field_name']) {
                        case 'user_type':
                            $custom_fields[] = array("name" => 'user_type_code', "type" => "text", "label" => $field["options"]["header"] . " (text, code)");
                            break;
                        case 'id_country':
                            $custom_fields[] = array('name' => 'country_code', 'type' => 'text', 'label' => $field['options']['header'] . ' (text, code)');
                            break;
                        case 'id_region':
                            $custom_fields[] = array('name' => 'region_code', 'type' => 'text', 'label' => $field['options']['header'] . ' (text, code)');
                            $custom_fields[] = array('name' => 'id_region', 'type' => 'int', 'label' => $field['options']['header'] . ' (int, id)');
                            break;
                        case 'lang_id':
                            $custom_fields[] = array('name' => 'language_gid', 'type' => 'text', 'label' => $field['options']['header'] . ' (text, GUID)');
                            break;
                        case 'id_currency':
                            $custom_fields[] = array('name' => 'currency_gid', 'type' => 'text', 'label' => $field['options']['header'] . ' (text, GUID)');
                            break;
                        case 'working_days':
                            $custom_fields[] = array('name' => 'working_days_code', 'type' => 'text', 'label' => $field['options']['header'] . ' (text, code)');
                            break;
                        default:
                            $custom_fields[] = array("name" => $field["field_name"], "type" => "int", "label" => $field["options"]["header"] . " (int, id)");
                            break;
                    }
                    if (isset($field['fk'])) {
                        $custom_fields[] = array("fk" => $field["fk"], "name" => $field["field_name"], "type" => "text", "label" => $field["options"]["header"] . " (text)");
                    } else {
                        $custom_fields[] = array("fk" => "field_editor." . $field["field_name"] . "_output", "name" => $field["field_name"], "type" => "text", "label" => $field["options"]["header"] . " (text)");
                    }
                    break;
                case "textarea":
                    $custom_fields[] = array("name" => $field["field_name"], "type" => "text", "label" => $field["name"] . " (text)");
                    break;
                case "file":
                    $custom_fields[] = array("name" => $field["field_name"], "type" => "url", "label" => $field["name"] . " (url)");
                    $custom_fields[] = array("name" => $field["field_name"], "type" => "file", "label" => $field["name"] . " (file)");
                    break;
                case 'checkbox':
                    $custom_fields[] = array('name' => 'field_editor_' . $field['gid'], 'type' => 'int', 'label' => $field['name'] . ' (int)', 'default' => '0');
                    $custom_fields[] = array('name' => 'field_editor_' . $field['gid'], 'type' => 'text', 'label' => $field['name'] . ' (text)', 'default' => l('no_str',
                                                                                                                                                                'start'));
                    break;
                case 'multiselect':
                    foreach ($field['options']['option'] as $value => $name) {
                        $custom_fields[] = array('name' => 'field_editor_' . $field['gid'] . '_option_' . $value, 'type' => 'int', 'label' => $field['name'] . ' - ' . $name . ' (int)', 'default' => '0');
                        $custom_fields[] = array('name' => 'field_editor_' . $field['gid'] . '_option_' . $value, 'type' => 'text', 'label' => $field['name'] . ' - ' . $name . ' (text)', 'default' => l('no_str',
                                                                                                                                                                                                          'start'));
                    }
                    break;
                default:
                    $custom_fields[] = array("name" => $field["field_name"], "type" => $field["field_type"], "label" => $field["name"] . " (" . $field["field_type"] . ")");
                    break;
            }
        }

        return $custom_fields;
    }

    /**
     * Return admin form
     *
     * Callback method for module export
     *
     * @param array $data selected values
     *
     * @return string
     */
    public function callback_get_admin_form($data)
    {
        $user_types = $this->CI->Users_model->getUserTypes();
        $this->CI->view->assign('user_types', $user_types);

        if (isset($data['field_editor_data']['admin_export_form'])) {
            if (is_array($data['field_editor_data']['admin_export_form'])) {
                $this->CI->load->model('Field_editor_model');
                $this->CI->Field_editor_model->initialize('user');

                $params = array();
                $fields_data = $this->CI->Field_editor_model->get_fields_list($params);
                foreach ($fields_data as $field_data) {
                    if (isset($data['field_editor_data']['admin_export_form'][$field_data['gid']])) {
                        $data[$field_data['field_name']] = $data['field_editor_data']['admin_export_form'][$field_data['gid']];
                    }
                    if (isset($data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_min'])) {
                        $data[$field_data['field_name'] . '_min'] = $data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_min'];
                    }
                    if (isset($data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_max'])) {
                        $data[$field_data['field_name'] . '_max'] = $data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_max'];
                    }
                }

                if (isset($data['user_type']) && !empty($data['user_type'])) {
                    $this->CI->Field_editor_model->initialize($data['user_type']);

                    $params = array();
                    $fields_data = $this->CI->Field_editor_model->get_fields_list($params);
                    foreach ($fields_data as $field_data) {
                        if (isset($data['field_editor_data']['admin_export_form'][$field_data['gid']])) {
                            $data[$field_data['field_name']] = $data['field_editor_data']['admin_export_form'][$field_data['gid']];
                        }
                        if (isset($data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_min'])) {
                            $data[$field_data['field_name'] . '_min'] = $data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_min'];
                        }
                        if (isset($data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_max'])) {
                            $data[$field_data['field_name'] . '_max'] = $data['field_editor_data']['admin_export_form'][$field_data['gid'] . '_max'];
                        }
                    }
                }
            }
            unset($data['field_editor_data']);
        }

        $main_extend_form = $this->get_export_extend_form('user', 'admin_export_form_gid');
        $this->CI->view->assign('main_extend_form', $main_extend_form);

        if (isset($data['user_type']) && !empty($data['user_type'])) {
            $advanced_extend_form = $this->get_export_extend_form($data['user_type'], 'admin_export_form_gid');
            $this->CI->view->assign('advanced_extend_form', $advanced_extend_form);
        }

        $this->CI->view->assign('data', $data);

        return $this->CI->view->fetch("export_form", "admin", "users");
    }

    /**
     * Return user form
     *
     * Callback method for module export
     *
     * @param array $data selected values
     *
     * @return string
     */
    public function callback_get_user_form($data)
    {
        $user_types = $this->CI->Users_model->getUserTypes();
        $this->CI->view->assign('user_types', $user_types);

        if (isset($data['field_editor_data']['user_export_form'])) {
            if (is_array($data['field_editor_data']['user_export_form'])) {
                $this->CI->load->model('Field_editor_model');
                $this->CI->Field_editor_model->initialize('user');

                $params = array();
                $fields_data = $this->CI->Field_editor_model->get_fields_list($params);
                foreach ($fields_data as $field_data) {
                    if (isset($data['field_editor_data']['user_export_form'][$field_data['gid']])) {
                        $data[$field_data['field_name']] = $data['field_editor_data']['user_export_form'][$field_data['gid']];
                    }
                    if (isset($data['field_editor_data']['user_export_form'][$field_data['gid'] . '_min'])) {
                        $data[$field_data['field_name'] . '_min'] = $data['field_editor_data']['user_export_form'][$field_data['gid'] . '_min'];
                    }
                    if (isset($data['field_editor_data']['user_export_form'][$field_data['gid'] . '_max'])) {
                        $data[$field_data['field_name'] . '_max'] = $data['field_editor_data']['user_export_form'][$field_data['gid'] . '_max'];
                    }
                }

                if (isset($data['user_type'])) {
                    $this->CI->Field_editor_model->initialize($data['user_type']);

                    $params = array();
                    $fields_data = $this->CI->Field_editor_model->get_fields_list($params);
                    foreach ($fields_data as $field_data) {
                        if (isset($data['field_editor_data']['user_export_form'][$field_data['gid']])) {
                            $data[$field_data['field_name']] = $data['field_editor_data']['user_export_form'][$field_data['gid']];
                        }
                        if (isset($data['field_editor_data']['user_export_form'][$field_data['gid'] . '_min'])) {
                            $data[$field_data['field_name'] . '_min'] = $data['field_editor_data']['user_export_form'][$field_data['gid'] . '_min'];
                        }
                        if (isset($data['field_editor_data']['user_export_form'][$field_data['gid'] . '_max'])) {
                            $data[$field_data['field_name'] . '_max'] = $data['field_editor_data']['user_export_form'][$field_data['gid'] . '_max'];
                        }
                    }
                }
            }
            unset($data['field_editor_data']);
        }

        $this->CI->view->assign('data', $data);

        return $this->CI->view->fetch("export_form", "user", "users");
    }

    /**
     * Process submit from export form
     *
     * Callback method for module export
     *
     * @param boolean $admin_mode admin mode
     *
     * @return string
     */
    public function callback_process_form($admin_mode = true)
    {
        $post_data = (array) $this->CI->input->post('filters', true);

        //field editor
        if ($form) {
            $post_data['field_editor_data'][$form] = $this->input->post('data', true);
            if (!is_array($post_data['field_editor_data'][$form])) {
                $post_data['field_editor_data'][$form] = array();
            }
            $field_editor_range = $this->input->post('field_editor_range', true);
            if (!is_array($field_editor_range)) {
                $field_editor_range = array();
            }
            $short_value = $this->input->post('short_value', true);

            foreach ($post_data['field_editor_data'][$form] as $field_gid => $field_value) {
                if (isset($field_value['range']['min'])) {
                    if (isset($field_editor_range[$field_gid]['min'])) {
                        $min_value = $field_editor_range[$field_gid]['min'];
                    } else {
                        $min_value = 0;
                    }
                    if ($field_value['range']['min'] <= $min_value) {
                        unset($post_data['field_editor_data'][$form][$field_gid]['range']['min']);
                    }
                }
                if (isset($field_value['range']['max'])) {
                    if (isset($field_editor_range[$field_gid]['max'])) {
                        $max_value = $field_editor_range[$field_gid]['max'];
                        if ($field_value['range']['max'] >= $max_value) {
                            unset($post_data['field_editor_data'][$form][$field_gid]['range']['max']);
                        }
                    } elseif (empty($field_value['range']['max'])) {
                        unset($post_data['field_editor_data'][$form][$field_gid]['range']['max']);
                    }
                }
                if (isset($post_data['field_editor_data'][$form][$field_gid]['range']) &&
                        empty($post_data['field_editor_data'][$form][$field_gid]['range'])) {
                    unset($post_data['field_editor_data'][$form][$field_gid]['range']);
                }
            }

            foreach ($post_data['field_editor_data'][$form] as $field_gid => $field_data) {
                if (empty($field_data)) {
                    unset($post_data['field_editor_data'][$form][$field_gid]);
                } elseif (is_array($field_data)) {
                    $empty = true;
                    foreach ($field_data as $value) {
                        if (!empty($value)) {
                            $empty = false;
                            break;
                        }
                    }
                    if ($empty) {
                        unset($post_data['field_editor_data'][$form][$field_gid]);
                    }
                }
            }
            if (empty($post_data['field_editor_data'][$form])) {
                unset($post_data['field_editor_data'][$form]);
            }
        }

        foreach ($post_data as $key => $value) {
            if (empty($value)) {
                unset($post_data[$key]);
            }
        }

        return $post_data;
    }

    /**
     * Return export data
     *
     * Callback method for module export
     *
     * @param array $filter_data filters data
     * @param array $relations   relations data
     *
     * @return array
     */
    public function callback_export_data($filter_data, $relations = array())
    {
        $this->CI->load->model('Field_editor_model');

        $field_editor_names = array();

        if (isset($filter_data['user_type']) && !empty($filter_data['user_type'])) {
            $user_types = array($filter_data['user_type']);
        } else {
            $user_types = $this->CI->Users_model->getUserTypes();
        }

        $field_editor_types = $user_types;
        array_unshift($user_types, 'user');

        foreach ($field_editor_types as $editor_type_gid) {
            $this->CI->Field_editor_model->initialize($editor_type_gid);

            // Sections fields
            $where['where']['editor_type_gid'] = $editor_type_gid;
            $fields = $this->CI->Field_editor_model->get_fields_list($where);
            foreach ($fields as $i => $field) {
                switch ($field['field_type']) {
                    case 'select':
                        $field_editor_names[] = $field['field_name'];
                        $fields[$i]['fk'] = 'field_editor_' . $field['gid'] . '_output';
                        $fields[$i]['default'] = '';
                        break;
                    case 'checkbox':
                        $field_editor_names[] = 'field_editor_' . $field['gid'];
                        $fields[] = array('fk' => 'field_editor_' . $field['gid'] . '_text', 'field_name' => 'field_editor_' . $field['gid'], 'default' => l('no_str',
                                                                                                                                                             'start'));
                        unset($fields[$i]);
                        break;
                    case 'multiselect':
                        foreach ($field['options']['option'] as $value => $name) {
                            $field_editor_names[] = 'field_editor_' . $field['gid'] . '_option_' . $value;
                            $fields[] = array('fk' => 'field_editor_' . $field['gid'] . '_option_' . $value . '_text', 'field_name' => 'field_editor_' . $field['gid'] . '_option_' . $value, 'default' => l('no_str',
                                                                                                                                                                                                             'start'));
                        }
                        unset($fields[$i]);
                        break;
                    default:
                        $field_editor_names[] = $field['field_name'];
                        break;
                }
            }

            $this->fields_for_transfer = array_merge($this->fields_for_transfer, $fields);
        }

        reset($user_types);
        $user_type = current($user_types);

        // Field editor forms
        if (!empty($filter_data['field_editor_data'])) {
            $field_editor_data = $filter_data['field_editor_data'];
            unset($filter_data['field_editor_data']);

            $this->CI->load->model('field_editor/models/Field_editor_forms_model');
            $fe_criteria = array();
            foreach ($field_editor_data as $form_name => $form_data) {
                foreach ($form_data as $field_gid => $value) {
                    if (isset($value['range']) && is_array($value['range'])) {
                        if (isset($value['range']['min'])) {
                            $form_data[$field_gid . '_min'] = $value['range']['min'];
                        }
                        if (isset($value['range']['max'])) {
                            $form_data[$field_gid . '_max'] = $value['range']['max'];
                        }
                        unset($form_data[$field_gid]);
                    }
                }
                $form_gid = $this->{$form_name . '_user_gid'};
                $temp = $this->CI->Field_editor_forms_model->get_search_criteria($form_gid, $form_data, $editor_type_gid);
                if (!empty($temp)) {
                    $fe_criteria = array_merge($fe_criteria, $temp);
                }

                $form_gid = $this->{$form_name . '_' . $user_type . '_gid'};
                $temp = $this->CI->Field_editor_forms_model->get_search_criteria($form_gid, $form_data, $editor_type_gid);
                if (!empty($temp)) {
                    $fe_criteria = array_merge($fe_criteria, $temp);
                }
            }
            if (!empty($fe_criteria)) {
                if (isset($filter_data['field_editor'])) {
                    if (!in_array($editor_type_gid, $filter_data['field_editor'][0])) {
                        $filter_data['field_editor'][0][] = $editor_type_gid;
                    }
                    $filter_data['field_editor'][1] = array_merge($filter_data['field_editor'][0], $fe_criteria);
                } else {
                    $filter_data['field_editor'] = array(array($editor_type_gid), $fe_criteria);
                }
            }
        }

        if ($user_id) {
            $filter_data['user'] = $user_id;
        }

        $relation_names = array();

        foreach ($relations as $relation) {
            switch ($relation['link']) {
                default:
                    $relation_names[] = $relation['link'];
                    break;
            }
        }

        $this->CI->Users_model->set_format_settings(array(
            'get_safe' => false,
            'get_language' => true,
            'get_currency' => true,
            'get_group' => true,
            'get_company' => true,
        ));

        if (count(array_intersect($relation_names, $field_editor_names))) {
            $this->CI->Users_model->set_format_settings('get_field_editor', true);
        }

        $data = $this->CI->Users_model->get_users_list_by_filters($filter_data, null, null, null, true, true);

        $this->CI->Users_model->set_format_settings(array(
            'get_safe' => true,
            'get_language' => false,
            'get_currency' => false,
            'get_group' => false,
            'get_company' => false,
            'get_field_editor',
        ));

        $return = array('data' => $data);

        foreach ($relations as $i => $relation) {
            if ($relation['link'] == 'user_type_code') {
                foreach ($data as $j => $row) {
                    $return['fk'][$j]['user_type_code'] = $row['user_type'];
                }
            } elseif ($relation['link'] == 'country_code') {
                foreach ($data as $j => $row) {
                    $return['fk'][$j]['country_code'] = $row['id_country'];
                }
            } elseif ($relation['link'] == 'region_code') {
                foreach ($data as $j => $row) {
                    $return['fk'][$j]['region_code'] = $row['region_code'];
                }
            } elseif ($relation['link'] == 'language_gid') {
                foreach ($data as $j => $row) {
                    $return['fk'][$j]['language_gid'] = isset($row['language']) ? $row['language']['gid'] : '';
                }
            } elseif ($relation['link'] == 'currency_gid') {
                foreach ($data as $j => $row) {
                    $return['fk'][$j]['currency_gid'] = isset($row['currency']) ? $row['currency']['gid'] : '';
                }
            } elseif ($relation['link'] == 'working_days_code') {
                foreach ($data as $j => $row) {
                    $return['fk'][$j]['working_days_code'] = $row['working_days'];
                }
            } elseif ($relation['link'] == 'user_logo') {
                foreach ($data as $j => $row) {
                    if (!empty($row['user_logo_moderation'])) {
                        $return['data'][$j]['user_logo'] = $row['media']['user_logo_moderation']['file_url'];
                    } elseif (!empty($row['user_logo'])) {
                        $return['data'][$j]['user_logo'] = $row['media']['user_logo']['file_url'];
                    } else {
                        $return['data'][$j]['user_logo'] = '';
                    }
                }
            } elseif ($relation['link'] == 'seolink') {
                $this->CI->load->helper('seo');

                $current_lang = $this->CI->pg_language->get_lang_by_id($this->CI->pg_language->current_lang_id);
                $current_lang_code = $current_lang['code'];

                foreach ($data as $j => $row) {
                    $return['fk'][$j]['seolink'] = rewrite_link('users', 'view', $row);
                }
            } elseif ($relation["type"] == "text") {
                foreach ($this->fields_for_transfer as $field) {
                    if ($relation["link"] == $field["field_name"]) {
                        if (isset($field["fk"])) {
                            foreach ($data as $j => $row) {
                                if (strpos($field["fk"], ".")) {
                                    $chunks = explode(".", $field["fk"]);
                                    $return['fk'][$j][$relation["link"]] = isset($data[$j][$chunks[0]][$chunks[1]]) ? $data[$j][$chunks[0]][$chunks[1]] : '';
                                } else {
                                    $return['fk'][$j][$relation["link"]] = $data[$j][$field["fk"]];
                                }
                            }
                        }
                        break;
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Return export form from field editor
     *
     * @param string  $user_type  user type
     * @param string  $form_gid   form GUID
     * @param boolean $admin_mode admin mode
     *
     * @return string
     */
    public function get_export_extend_form($user_type, $form_gid, $admin_mode = true)
    {
        $this->CI->load->model('Field_editor_model');

        $this->CI->Field_editor_model->initialize($user_type);

        $form_gid = $this->{$this->{$form_gid} . '_' . $user_type . '_gid'};

        $this->CI->load->model('field_editor/models/Field_editor_forms_model');
        $form = $this->CI->Field_editor_forms_model->get_form_by_gid($form_gid, $user_type);
        $form = $this->CI->Field_editor_forms_model->format_output_form($form);
        if (count($form['field_data'])) {
            $this->CI->view->assign('form_data', $form['field_data']);
        }

        if ($admin_mode) {
            $mode = 'admin';
            $template_name = 'export_form_block';
        } else {
            $mode = 'user';
            $template_name = 'export_form_block';
        }

        $this->CI->view->assign('form_rand', rand(100000, 999999));

        return $this->CI->view->fetch($template_name, $mode, 'users');
    }

}
