<?php

namespace Pg\Modules\Users\Models;

use Pg\Libraries\Setup;

/**
 * Users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;
    protected $modules_data = [];

    /**
     * Constructor
     *
     * @return Users_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->modules_data = Setup::getModuleData('users', Setup::TYPE_MODULES_DATA);
    }

    /**
     * Install users data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu'], "create", $gid, 0, $this->modules_data['menu'][$gid]["items"]);
        }

        if (!empty($this->modules_data['menu_indicators'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->modules_data['menu_indicators'] as $data) {
                $this->ci->Indicators_model->save_type(null, $data);
            }
        }
    }

    /**
     * Import users languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data");

            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu'], "update", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_file);
        }

        // Indicators
        if (!empty($this->modules_data['menu_indicators'])) {
            $langs_file = $this->ci->Install_model->language_file_read('users', 'indicators', $langs_ids);
            if (!$langs_file) {
                log_message('info', '(users) Empty indicators langs data');

                return false;
            } else {
                $this->ci->load->model('menu/models/Indicators_model');
                $this->ci->Indicators_model->update_langs($this->modules_data['menu_indicators'], $langs_file, $langs_ids);
            }
        }

        return true;
    }

    /**
     * Export users languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], "export", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        if (!empty($this->modules_data['menu_indicators'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            $indicators_langs = $this->ci->Indicators_model->export_langs($this->modules_data['menu_indicators'], $langs_ids);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall users data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->modules_data['menu'][$gid]["items"]);
            }
        }

        if (!empty($this->modules_data['menu_indicators'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->modules_data['menu_indicators'] as $data) {
                $this->ci->Indicators_model->delete_type($data['gid']);
            }
        }
    }

    /**
     * Install users data to uploads module
     *
     * @return void
     */
    public function install_uploads()
    {
        ///// upload config
        $this->ci->load->model("uploads/models/Uploads_config_model");

        $watermark_ids = array();

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = array(
                "gid" => $upload_data["gid"],
                "name" => $upload_data["name"],
                "max_height" => $upload_data["max_height"],
                "max_width" => $upload_data["max_width"],
                "max_size" => $upload_data["max_size"],
                "name_format" => $upload_data["name_format"],
                "file_formats" => serialize((array) $upload_data["file_formats"]),
                "default_img" => $upload_data["default_img"],
                "indicator_img" => $upload_data["indicator_img"],
                "date_add" => date("Y-m-d H:i:s"),
            );
            $config_id = $this->ci->Uploads_config_model->save_config(null, $config_data);

            $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid("image-wm");
            $wm_id = isset($wm_data["id"]) ? $wm_data["id"] : 0;

            foreach ((array) $upload_data["thumbs"] as $thumb_gid => $thumb_data) {
                if (isset($thumb_data["watermark"])) {
                    if (!isset($watermark_ids[$thumb_data["watermark"]])) {
                        $wm_data = $this->ci->Uploads_config_model->get_watermark_by_gid($thumb_data["watermark"]);
                        $watermark_ids[$thumb_data["watermark"]] = isset($wm_data["id"]) ? $wm_data["id"] : 0;
                    }
                    $watermark_id = $watermark_ids[$thumb_data["watermark"]];
                } else {
                    $watermark_id = 0;
                }

                $thumb_data = array(
                    "config_id" => $config_id,
                    "prefix" => $thumb_gid,
                    "width" => $thumb_data["width"],
                    "height" => $thumb_data["height"],
                    "effect" => "none",
                    "watermark_id" => $watermark_id,
                    "crop_param" => $thumb_data["crop_param"],
                    "crop_color" => $thumb_data["crop_color"],
                    "date_add" => date("Y-m-d H:i:s"),
                );
                $this->ci->Uploads_config_model->save_thumb(null, $thumb_data);
            }
        }
    }

    /**
     * Uninstall users data from uploads module
     *
     * @return void
     */
    public function deinstall_uploads()
    {
        $this->ci->load->model("uploads/models/Uploads_config_model");

        foreach ((array) $this->modules_data['uploads'] as $upload_data) {
            $config_data = $this->ci->Uploads_config_model->get_config_by_gid($upload_data["gid"]);
            if (!empty($config_data["id"])) {
                $this->ci->Uploads_config_model->delete_config($config_data["id"]);
            }
        }
    }

    /**
     * Install users data to site map module
     *
     * @return void
     */
    public function install_site_map()
    {
        //////// Site Map
        $this->ci->load->model("Site_map_model");
        $site_map_data = array(
            "module_gid" => "users",
            "model_name" => "Users_model",
            "get_urls_method" => "get_sitemap_urls",
        );
        $this->ci->Site_map_model->set_sitemap_module("users", $site_map_data);
    }

    /**
     * Uninstall users data from site map module
     *
     * @return void
     */
    public function deinstall_site_map()
    {
        $this->ci->load->model("Site_map_model");
        $this->ci->Site_map_model->delete_sitemap_module("users");
    }

    /**
     * Install users data to banners module
     *
     * @return void
     */
    public function install_banners()
    {
        ///// add banners module
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->Banner_group_model->set_module("users", "Users_model", "_banner_available_pages");
        $this->add_banners();
    }

    /**
     * Import users languages to banners module
     *
     * @return void
     */
    public function install_banners_lang_update()
    {
        $lang_ids = array_keys($this->ci->pg_language->languages);
        $lang_id = $this->ci->pg_language->get_default_lang_id();
        $lang_data[$lang_id] = "Users pages";
        $this->ci->pg_language->pages->set_string_langs("banners", "banners_group_users_groups", $lang_data, $lang_ids);
    }

    /**
     * Unistall users data from banners module
     *
     * @return void
     */
    public function deinstall_banners()
    {
        // delete banners module
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->Banner_group_model->delete_module("users");
        $this->remove_banners();
    }

    /**
     * Add available banners positions on users pages
     *
     * @return void
     */
    public function add_banners()
    {
        $lang_id = $this->ci->pg_language->get_default_lang_id();
        $lang_ids = array_keys($this->ci->pg_language->languages);

        $this->ci->load->model("Users_model");
        $this->ci->load->model("banners/models/Banner_group_model");
        $this->ci->load->model("banners/models/Banner_place_model");

        $group_id = $this->ci->Banner_group_model->get_group_id_by_gid("users_groups");
        ///add pages in group
        $pages = $this->ci->Users_model->_banner_available_pages();
        if ($pages) {
            foreach ($pages as $key => $value) {
                $page_attrs = array(
                    "group_id" => $group_id,
                    "name" => $value["name"],
                    "link" => $value["link"],
                );
                $this->ci->Banner_group_model->add_page($page_attrs);
            }
        }

        $lang_data[$lang_id] = "Users pages";
        $this->ci->pg_language->pages->set_string_langs("banners", "banners_group_users_groups", $lang_data, $lang_ids);
    }

    /**
     * Remove available banners positions on users pages
     *
     * @return void
     */
    public function remove_banners()
    {
        $this->ci->load->model("banners/models/Banner_group_model");
        $group_id = $this->ci->Banner_group_model->get_group_id_by_gid("users_groups");
        $this->ci->Banner_group_model->delete($group_id);
    }

    /**
     * Install users data to linker module
     *
     * @return void
     */
    public function install_linker()
    {
        ///// add linker entry
        $this->ci->load->model("linker/models/linker_type_model");
        $this->ci->linker_type_model->create_type("users_contacts");
    }

    /**
     * Uninstall users data from linker module
     *
     * @return void
     */
    public function deinstall_linker()
    {
        $this->ci->load->model("linker/models/linker_type_model");
        $this->ci->linker_type_model->delete_type("users_contacts");
    }

    /**
     * Install users data to moderation module
     *
     * @return void
     */
    public function install_moderation()
    {
        $this->ci->load->model('moderation/models/Moderation_type_model');
        foreach ($this->modules_data['moderation_types'] as $mtype) {
            $mtype['date_add'] = date("Y-m-d H:i:s");
            $this->ci->Moderation_type_model->save_type(null, $mtype);
        }
    }

    /**
     * Import users languages to moderation module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_moderation_lang_update($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }
        $langs_file = $this->ci->Install_model->language_file_read('users', 'moderation', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty moderation langs data');

            return false;
        }

        $this->ci->load->model('moderation/models/Moderation_type_model');
        $this->ci->Moderation_type_model->update_langs($this->modules_data['moderation_types'], $langs_file);

        return true;
    }

    /**
     * Export users languages from moderation module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderation_lang_export($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }
        $this->ci->load->model('moderation/models/Moderation_type_model');
        $moderation = $this->ci->Moderation_type_model->export_langs($this->modules_data['moderation_types'], $langs_ids);

        return array('moderation' => $moderation);
    }

    /**
     * Uninstall users data from moderation module
     *
     * @return void
     */
    public function deinstall_moderation()
    {
        $this->ci->load->model('moderation/models/Moderation_type_model');
        foreach ($this->modules_data['moderation_types'] as $mtype) {
            $type = $this->ci->Moderation_type_model->get_type_by_name($mtype["name"]);
            $this->ci->Moderation_type_model->delete_type($type['id']);
        }
    }

    /**
     * Install users data to moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model("Moderators_model");

        foreach ((array) $this->modules_data['moderators'] as $method_data) {
            //$validate_data = $this->CI->Moderators_model->validate_method($method_data, true);
            $validate_data = array("errors" => array(), "data" => $method_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Moderators_model->save_method(null, $validate_data["data"]);
        }
    }

    /**
     * Import users languages to moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users", "moderators", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty moderators langs data");

            return false;
        }

        // install moderators permissions
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "users";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method["method"]])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method["method"]]);
            }
        }

        return true;
    }

    /**
     * Export users languages from moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "users";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method["method"]] = $method["langs"];
        }

        return array("moderators" => $return);
    }

    /**
     * Uninstall users data from moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        // delete moderation methods in moderators
        $this->ci->load->model("Moderators_model");
        $params = array();
        $params["where"]["module"] = "users";
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install users data to notifications module
     *
     * @return void
     */
    public function install_notifications()
    {
        // add notification
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        $templates_ids = array();

        foreach ((array) $this->modules_data['notifications']["templates"] as $template_data) {
            if (is_array($template_data["vars"])) {
                $template_data["vars"] = implode(", ", $template_data["vars"]);
            }
            $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                log_message('error', '(users) install_notifications ' . var_export($validate_data["errors"], 1));
                continue;
            }
            $templates_ids[$template_data['gid']] = $this->ci->Templates_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->modules_data['notifications']["notifications"] as $notification_data) {
            if (!isset($templates_ids[$notification_data["template"]])) {
                $template = $this->ci->Templates_model->get_template_by_gid($notification_data["template"]);
                $templates_ids[$notification_data["template"]] = $template["id"];
            }
            $notification_data["id_template_default"] = $templates_ids[$notification_data["template"]];
            $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Notifications_model->save_notification(null, $validate_data["data"]);
        }
    }

    /**
     * Import users languages to notifiactions module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_notifications_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Notifications_model");

        $langs_file = $this->ci->Install_model->language_file_read("users", "notifications", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty notifications langs data");

            return false;
        }

        $this->ci->Notifications_model->update_langs($this->modules_data['notifications'], $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users languages from notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_notifications_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Notifications_model");
        $langs = $this->ci->Notifications_model->export_langs($this->modules_data['notifications'], $langs_ids);

        return array("notifications" => $langs);
    }

    /**
     * Uninstall users data from notifications module
     *
     * @return void
     */
    public function deinstall_notifications()
    {
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach ((array) $this->modules_data['notifications']["notifications"] as $notification_data) {
            $this->ci->Notifications_model->delete_notification_by_gid($notification_data["gid"]);
        }

        foreach ((array) $this->modules_data['notifications']["templates"] as $template_data) {
            $this->ci->Templates_model->delete_template_by_gid($template_data["gid"]);
        }
    }

    /**
     * Install users data to reviews module
     *
     * @return void
     */
    public function install_reviews()
    {
        $this->ci->load->model("Users_model");

        // add reviews type
        $this->ci->load->model("reviews/models/Reviews_type_model");

        $reviews_fields = (array) $this->modules_data['reviews']["reviews_fields"];
        $this->ci->Users_model->install_reviews_fields($reviews_fields);

        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $validate_data = $this->ci->Reviews_type_model->validate_type(null, $review_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Reviews_type_model->save_type(null, $validate_data["data"]);
        }
    }

    /**
     * Import users languages to reviews module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_reviews_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Reviews_model");

        $langs_file = $this->ci->Install_model->language_file_read("users", "reviews", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty reviews langs data");

            return false;
        }

        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $this->ci->Reviews_model->update_langs($review_data, $langs_file, $langs_ids);
        }

        foreach ($langs_ids as $lang_id) {
            foreach ((array) $this->modules_data['reviews']["rate_types"] as $type_gid => $type_data) {
                $types_data = array();
                foreach ($type_data as $rate_type => $votes) {
                    $votes_data = array();
                    foreach ($votes as $vote) {
                        $votes_data[$vote] = isset($langs_file[$type_gid . '_' . $rate_type . "_votes_" . $vote][$lang_id]) ?
                            $langs_file[$type_gid . '_' . $rate_type . "_votes_" . $vote][$lang_id] : $vote;
                    }
                    $types_data[$rate_type] = array(
                        "header" => $langs_file[$type_gid . '_' . $rate_type . "_header"][$lang_id],
                        "votes" => $votes_data,
                    );
                }
                $this->ci->Reviews_model->add_rate_type($type_gid, $types_data, $lang_id);
            }
        }

        return true;
    }

    /**
     * Export users languages from reviews module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_reviews_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Reviews_model");
        $langs = array();
        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $langs = array_merge($langs, $this->ci->Reviews_model->export_langs($review_data['gid'], $langs_ids));
        }

        return array("reviews" => $langs);
    }

    /**
     * Uninstall users data of reviews module
     *
     * @return void
     */
    public function deinstall_reviews()
    {
        $this->ci->load->model("Users_model");

        //add reviews type
        $this->ci->load->model("reviews/models/Reviews_type_model");

        foreach ((array) $this->modules_data['reviews']["reviews"] as $review_data) {
            $this->ci->Reviews_type_model->delete_type($review_data["gid"]);
        }

        $this->ci->Users_model->deinstall_reviews_fields(array_keys((array) $this->modules_data['reviews']["reviews_fields"]));
    }

    /**
     * Install users data to spam module
     *
     * @return void
     */
    public function install_spam()
    {
        // add spam type
        $this->ci->load->model("spam/models/Spam_type_model");

        foreach ((array) $this->modules_data['spam'] as $spam_data) {
            $validate_data = $this->ci->Spam_type_model->validate_type(null, $spam_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Spam_type_model->save_type(null, $validate_data["data"]);
        }
    }

    /**
     * Import users languages to spam module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_spam_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $this->ci->load->model("spam/models/Spam_type_model");

        $langs_file = $this->ci->Install_model->language_file_read("users", "spam", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty spam langs data");

            return false;
        }

        $this->ci->Spam_type_model->update_langs($this->modules_data['spam'], $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users languages from spam module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_spam_lang_export($langs_ids = null)
    {
        $this->ci->load->model("spam/models/Spam_type_model");
        $langs = $this->ci->Spam_type_model->export_langs((array) $this->modules_data['spam'], $langs_ids);

        return array("spam" => $langs);
    }

    /**
     * Uninstall users data from spam module
     *
     * @return void
     */
    public function deinstall_spam()
    {
        //add spam type
        $this->ci->load->model("spam/models/Spam_type_model");

        foreach ((array) $this->modules_data['spam'] as $spam_data) {
            $this->ci->Spam_type_model->delete_type($spam_data["gid"]);
        }
    }

    /**
     * Install users data to geomap module
     *
     * @return void
     */
    public function install_geomap()
    {
        //add geomap settings
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        foreach ((array) $this->modules_data['geomap'] as $settings) {
            $map_gid = $settings["map_gid"];
            unset($settings["map_gid"]);

            $id_user = $settings["id_user"];
            unset($settings["id_user"]);

            $id_object = $settings["id_object"];
            unset($settings["id_object"]);

            $gid = $settings["gid"];
            unset($settings["gid"]);

            $validate_data = $this->ci->Geomap_settings_model->validate_settings($settings);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Geomap_settings_model->save_settings($map_gid, $id_user, $id_object, $gid, $validate_data["data"]);
        }
    }

    /**
     * Install users languages to geomap module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_geomap_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("users", "geomap", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty geomap langs data");

            return false;
        }

        $this->ci->load->model("geomap/models/Geomap_settings_model");

        $gids = array();
        foreach ((array) $this->modules_data['geomap'] as $geomap_data) {
            $gids[] = 'map_' . $geomap_data['gid'];
        }
        $this->ci->Geomap_settings_model->update_lang($gids, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users languages from geomap module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_geomap_lang_export($langs_ids = null)
    {
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        $gids = array();
        foreach ((array) $this->modules_data['geomap'] as $geomap_data) {
            $gids[] = 'map_' . $geomap_data['gid'];
        }
        $langs = $this->ci->Geomap_settings_model->export_lang($gids, $langs_ids);

        return array("geomap" => $langs);
    }

    /**
     * Uninstall users data from geomap module
     *
     * @return void
     */
    public function deinstall_geomap()
    {
        //add geomap settings
        $this->ci->load->model("geomap/models/Geomap_settings_model");

        foreach ((array) $this->modules_data['geomap'] as $settings) {
            $map_gid = $settings["map_gid"];
            unset($settings["data"]["map_gid"]);

            $id_user = $settings["id_user"];
            unset($settings["id_user"]);

            $id_object = $settings["id_object"];
            unset($settings["id_object"]);

            $gid = $settings["gid"];
            unset($settings["gid"]);

            $this->ci->Geomap_settings_model->delete_settings($map_gid, $id_user, $id_object, $gid);
        }
    }

    /**
     * Install users data to social networking module
     *
     * @return void
     */
    public function install_social_networking()
    {
        ///// add social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        foreach ((array) $this->modules_data['social_networking'] as $social_networking_data) {
            $validation_data = $this->ci->Social_networking_pages_model->validate_page(null, $social_networking_data);
            if (!empty($validation_data['errors'])) {
                continue;
            }
            $this->ci->Social_networking_pages_model->save_page(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall users data from social networking module
     *
     * @return void
     */
    public function deinstall_social_networking()
    {
        ///// delete social netorking page
        $this->ci->load->model('social_networking/models/Social_networking_pages_model');
        $this->ci->Social_networking_pages_model->delete_pages_by_controller('users');
    }

    /**
     * Install users data to listings module
     *
     * @return void
     */
    public function install_listings()
    {
        $this->ci->load->model("Users_model");
        $this->ci->Users_model->install_listings_fields((array) $this->modules_data['listings']["listings_fields"]);
    }

    /**
     * Uninstall users data from listings module
     *
     * @return void
     */
    public function deinstall_listings()
    {
        $this->ci->load->model("Users_model");
        $this->ci->Users_model->deinstall_listings_fields(array_keys((array) $this->modules_data['listings']["listings_fields"]));
    }

    /**
     * Install users data to export module
     *
     * @return void
     */
    public function install_export()
    {
        $this->ci->load->model("export/models/Export_module_model");

        $validate_data = $this->ci->Export_module_model->validate_module(
            null, (array) $this->modules_data['export_data']
        );
        if (!empty($validate_data["errors"])) {
            return;
        }
        $this->ci->Export_module_model->save_module(null, $validate_data["data"]);
    }

    /**
     * Import users languages to export module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_export_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("users", "export", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty export langs data");

            return false;
        }

        $this->ci->load->model("export/models/Export_module_model");
        $this->ci->Export_module_model->update_lang(array("module_users"), $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users languages from export module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_export_lang_export($langs_ids = null)
    {
        $this->ci->load->model("export/models/Export_module_model");
        $langs = $this->ci->Export_module_model->export_lang(array("module_users"), $langs_ids);

        return array("export" => $langs);
    }

    /**
     * Unistall users data from export module
     *
     * @return void
     */
    public function deinstall_export()
    {
        $this->ci->load->model("export/models/Export_module_model");
        $this->ci->Export_module_model->remove_module("users");
    }

    /**
     * Install users data from import module
     *
     * @return void
     */
    public function install_import()
    {
        $this->ci->load->model("import/models/Import_module_model");

        $validate_data = $this->ci->Import_module_model->validate_module(null, (array) $this->modules_data['import_data']);
        if (!empty($validate_data["errors"])) {
            return;
        }
        $this->ci->Import_module_model->save_module(null, $validate_data["data"]);
    }

    /**
     * Import users languages to import module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_import_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("users", "import", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty import langs data");

            return false;
        }

        $this->ci->load->model("import/models/Import_module_model");
        $this->ci->Import_module_model->update_lang(array("module_users"), $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users languages from import module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_import_lang_export($langs_ids = null)
    {
        $this->ci->load->model("import/models/Import_module_model");
        $langs = $this->ci->Import_module_model->export_lang(array("module_users"), $langs_ids);
        return array("import" => $langs);
    }

    /**
     * Unistall users data from import module
     *
     * @return void
     */
    public function deinstall_import()
    {
        $this->ci->load->model("import/models/Import_module_model");
        $this->ci->Import_module_model->remove_module("users");
    }

    /**
     * Install users data to field editor module
     *
     * @return void
     */
    public function install_field_editor()
    {
        $this->ci->load->model("Field_editor_model");
        $this->ci->load->model("field_editor/models/Field_editor_forms_model");

        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            include MODULEPATH . 'users/install/users_fields_' . $type_gid . '_data.php';
            $this->ci->Field_editor_model->import_type_structure($type_gid, $fe_sections, $fe_fields, $fe_forms);
            unset($fe_sections);
            unset($fe_fields);
            unset($fe_form);
        }

        return true;
    }

    /**
     * Import users languages to field editor module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_field_editor_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users", "field_editor", $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty field editor langs data');

            return false;
        }
        $this->ci->load->model('Field_editor_model');
        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            include MODULEPATH . 'users/install/users_fields_' . $type_gid . '_data.php';
            $this->ci->Field_editor_model->initialize($type_gid);
            $this->ci->Field_editor_model->update_sections_langs($fe_sections, $langs_file);
            $this->ci->Field_editor_model->update_fields_langs($type_gid, $fe_fields, $langs_file);
        }

        return true;
    }

    /**
     * Export users languages from field editor module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_field_editor_lang_export($langs_ids = null)
    {
        $this->ci->load->model('Field_editor_model');

        $sections = $fields = array();

        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            $this->ci->Field_editor_model->initialize($type_gid);
            list($fe_sections, $fe_fields, $fe_forms) = $this->ci->Field_editor_model->export_type_structure($type_gid, 'application/modules/users/install/users_fields_' . $type_gid . '_data.php');
            if (!empty($fe_sections)) {
                $sections = array_merge($sections, $this->ci->Field_editor_model->export_sections_langs($fe_sections, $langs_ids));
            }
            if (!empty($fe_fields)) {
                $fields = array_merge($fields, $this->ci->Field_editor_model->export_fields_langs($type_gid, $fe_fields, $langs_ids));
            }
        }

        return array('field_editor' => array_merge($sections, $fields));
    }

    /**
     * Uninstall users data from field editor module
     *
     * @return void
     */
    public function deinstall_field_editor()
    {
        $this->ci->load->model("Field_editor_model");
        $this->ci->load->model("field_editor/models/field_editor_forms_model");

        foreach ($this->modules_data['field_editor_types'] as $type_gid) {
            $this->ci->Field_editor_model->initialize($type_gid);

            list($fe_sections, $fe_fields, $fe_forms) = $this->ci->Field_editor_model->export_type_structure(
                $type_gid, 'application/modules/users/install/users_fields_' . $type_gid . '_data.php'
            );

            foreach ((array) $fe_forms as $form_gid => $form_data) {
                if ($form_data['data']["editor_type_gid"] != $type_gid) {
                    continue;
                }
                $this->ci->Field_editor_forms_model->delete_form_by_gid($form_data['data']['gid']);
            }

            foreach ((array) $fe_fields as $field_gid => $field_data) {
                if ($field_data['data']["editor_type_gid"] != $type_gid) {
                    continue;
                }
                $this->ci->Field_editor_model->delete_field_by_gid($field_data['data']['gid']);
            }
            foreach ((array) $fe_sections as $section_gid => $section_data) {
                if ($section_data['data']["editor_type_gid"] != $type_gid) {
                    continue;
                }
                $this->ci->Field_editor_model->delete_section_by_gid($section_data['data']['gid']);
            }
        }
    }

    /**
     * Install data of cronjobs module
     *
     * @return void
     */
    public function install_cronjob()
    {
        // add lift up cronjob
        $this->ci->load->model('Cronjob_model');
        foreach ((array) $this->modules_data['cronjobs'] as $cron_data) {
            $validation_data = $this->ci->Cronjob_model->validate_cron(null, $cron_data);
            if (!empty($validation_data['errors'])) {
                continue;
            }
            $this->ci->Cronjob_model->save_cron(null, $validation_data['data']);
        }
    }

    /**
     * Uninstall data of cronjobs module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model('Cronjob_model');
        $cron_data = array();
        $cron_data["where"]["module"] = "users";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    /**
     * Install data of users module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        $this->ci->load->model("users/models/Users_groups_model");

        // create groups
        foreach ((array) $this->modules_data['groups'] as $group_data) {
            $validate_data = $this->ci->Users_groups_model->validate_group(null, $group_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Users_groups_model->save_group(null, $validate_data["data"]);
        }

        // seo
        $this->ci->pg_seo->set_seo_module("users", (array) $this->modules_data['seo']);

        // add seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $seo_settings = [
            'account' => [
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:account]',
            ],
            'login_form' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:login]',
            ),
            'restore' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:restore]',
            ),
            'profile' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:profile/][tpl:1:action:literal:view]',
            ),
            'index' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:users/][tpl:1:user_type:literal:all][text:-index][text:/][postfix:2:order:literal][text:/][postfix:3:order_direction:literal][text:/][postfix:4:page:literal]',
            ),
            'search' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:users/][tpl:1:user_type:literal:all][text:-search/][tpl:2:keyword:literal:empty][text:/][postfix:3:order:literal][text:/][postfix:4:order_direction:literal][text:/][postfix:5:page:literal]',
            ),
            'view' => array(
                'noindex' => 0,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:users/][opt:user_type:literal:all][text:/][opt:name:literal:noname][text:-][tpl:1:id:numeric:][text:/][tpl:2:section:literal:contacts]',
            ),
            'reg_company' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:registration-as-company]',
            ),
            'reg_agent' => array(
                'noindex' => 1,
                'title' => '',
                'keyword' => '',
                'description' => '',
                'header' => '',
                'og_title' => '',
                'og_type' => '',
                'og_descrtiption' => '',
                'url_template' => '[text:registration-as-agent]',
            ),
        ];

        foreach ($seo_settings as $key => $record) {
            $this->ci->pg_seo->set_settings('user', 'users', $key, $record);
            $xml_data['users'][$key] = $this->ci->pg_seo->url_template_transform('users', $key, $record['url_template'], 'base', 'xml');
        }

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        //get administrator email
        if ($this->pg_module->is_module_active('moderators')) {
            $this->ci->load->model('Moderators_model');
            $users = $this->ci->Moderators_model->get_users_list(null, 1, array('id' => 'DESC'), array('where' => array('user_type' => 'admin')));
            if (!empty($users)) {
                $this->ci->pg_module->set_module_config("users", 'deactivated_email', $users[0]["email"]);
            }
        }

        $this->addUserTypes();

        // DEMO
        if (!INSTALL_DONE) {
            $this->add_demo_content();
        }
    }

    protected function addUserTypes()
    {
        $this->ci->load->model('users/models/Users_types_model');
        $this->ci->Users_types_model->addTypes($this->modules_data['user_types']);
    }

    /**
     * Install users data to payments module
     *
     * @return void
     */
    public function install_payments()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            $this->modules_data['menu_payments'][$gid]["id"] = linked_install_set_menu(
                $gid, $menu_data["action"], $menu_data["name"]
            );
            linked_install_process_menu_items($this->modules_data['menu_payments'], "create", $gid, 0, $menu_data["items"]);
        }

        if (!empty($this->modules_data['menu_indicators_payments'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->modules_data['menu_indicators_payments'] as $data) {
                $this->ci->Indicators_model->save_type(null, $data);
            }
        }
    }

    /**
     * Import users languages to payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_payments_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users", "menu_payments", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data for payments (users)");

            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu_payments'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
        }

        // Indicators
        if (!empty($this->modules_data['menu_indicators_payments'])) {
            $langs_file = $this->ci->Install_model->language_file_read('users', 'indicators', $langs_ids);
            if (!$langs_file) {
                log_message('info', '(users) Empty indicators langs data');

                return false;
            } else {
                $this->ci->load->model('menu/models/Indicators_model');
                $this->ci->Indicators_model->update_langs($this->modules_data['menu_indicators_payments'], $langs_file, $langs_ids);
            }
        }

        return true;
    }

    /**
     * Export users languages from payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_payments_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu_payments'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        if (!empty($this->modules_data['menu_indicators_payments'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            $indicators_langs = $this->ci->Indicators_model->export_langs($this->modules_data['menu_indicators_payments'], $langs_ids);
        }

        return array("menu_payments" => $return);
    }

    /**
     * Uninstall users data from payments module
     *
     * @return void
     */
    public function deinstall_payments()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_payments'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }

        if (!empty($this->modules_data['menu_indicators_payments'])) {
            $this->ci->load->model('menu/models/Indicators_model');
            foreach ($this->modules_data['menu_indicators_payments'] as $data) {
                $this->ci->Indicators_model->delete_type($data['gid']);
            }
        }
    }

    /**
     * Install users data to users_services module
     *
     * @return void
     */
    public function install_users_services()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_users_services'] as $gid => $menu_data) {
            $this->modules_data['menu_users_services'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu_users_services'], "create", $gid, 0, $menu_data["items"]);
        }
    }

    /**
     * Import users languages to users_services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_users_services_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users", "menu_users_services", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data for users_services (users)");

            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_users_services'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu_users_services'], "update", $gid, 0, $menu_data["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export users languages from users_services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_users_services_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu_users_services'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu_users_services'], "export", $gid, 0, $menu_data["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu_users_services" => $return);
    }

    /**
     * Uninstall users data from users_services module
     *
     * @return void
     */
    public function deinstall_users_services()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu_users_services'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $menu_data["items"]);
            }
        }
    }

    /**
     * Import languages of users module
     *
     * @return boolean
     */
    public function _arbitrary_lang_install()
    {
        $langs_file = $this->ci->Install_model->language_file_read("users", "arbitrary");
        if (!$langs_file) {
            log_message("info", "Empty arbitrary langs data");
            return false;
        }

        $this->ci->load->model("users/models/Users_groups_model");

        foreach ((array) $this->modules_data['groups'] as $group_data) {
            $group = $this->ci->Users_groups_model->get_group_by_gid($group_data["gid"]);
            $this->ci->pg_language->pages->set_string_langs("groups_langs", "group_item_" . $group["id"], $langs_file["groups_demo_" . $group_data["gid"]], array_keys($langs_file["groups_demo_" . $group_data["gid"]]));
        }

        foreach ($this->modules_data['seo_settings'] as $lang_key => $sett_key) {
            $post_data = array(
                "title" => $langs_file["seo_tags_{$lang_key}_title"],
                "keyword" => $langs_file["seo_tags_{$lang_key}_keyword"],
                "description" => $langs_file["seo_tags_{$lang_key}_description"],
                "header" => $langs_file["seo_tags_{$lang_key}_header"],
                "og_title" => $langs_file["seo_tags_{$lang_key}_og_title"],
                "og_type" => $langs_file["seo_tags_{$lang_key}_og_type"],
                "og_description" => $langs_file["seo_tags_{$lang_key}_og_description"],
            );
            $this->ci->pg_seo->set_settings("user", "users", $sett_key, $post_data);
        }
    }

    /**
     * Export languages of users module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        $this->ci->load->model("users/models/Users_groups_model");

        foreach ((array) $this->modules_data['groups'] as $group_data) {
            $group = $this->ci->Users_groups_model->get_group_by_gid($group_data["gid"]);
            $blocks['groups_demo_' . $group_data["gid"]] = "group_item_" . $group["id"];
        }

        $group_langs = $this->ci->pg_language->export_langs("groups_langs", $blocks, $langs_ids);
        $arbitrary_return = array();
        if (!empty($group_langs)) {
            $blocks = array_flip($blocks);
            foreach ($group_langs as $group_gid => $group_data) {
                if (!isset($blocks[$group_gid])) {
                    continue;
                }
                $arbitrary_return[$blocks[$group_gid]] = $group_data;
            }
        }

        foreach ($this->modules_data['seo_settings'] as $lang_key => $sett_key) {
            // arbitrary
            $settings = $this->ci->pg_seo->get_settings("user", "users", $sett_key, $langs_ids);
            $arbitrary_return["seo_tags_{$lang_key}_title"] = $settings["title"];
            $arbitrary_return["seo_tags_{$lang_key}_keyword"] = $settings["keyword"];
            $arbitrary_return["seo_tags_{$lang_key}_description"] = $settings["description"];
            $arbitrary_return["seo_tags_{$lang_key}_header"] = $settings["header"];
            $arbitrary_return["seo_tags_{$lang_key}_og_title"] = $settings["og_title"];
            $arbitrary_return["seo_tags_{$lang_key}_og_type"] = $settings["og_type"];
            $arbitrary_return["seo_tags_{$lang_key}_og_description"] = $settings["og_description"];
        }

        return array("arbitrary" => $arbitrary_return);
    }

    /**
     * Unistall data of users module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        // remove seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'account', $data);

        $xml_data['users']['account'] = $this->ci->pg_seo->url_template_transform('users', 'account', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'login_form', $data);

        $xml_data['users']['login_form'] = $this->ci->pg_seo->url_template_transform('users', 'login_form', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'restore', $data);

        $xml_data['users']['restore'] = $this->ci->pg_seo->url_template_transform('users', 'restore', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'profile', $data);

        $xml_data['users']['profile'] = $this->ci->pg_seo->url_template_transform('users', 'profile', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'index', $data);

        $xml_data['users']['index'] = $this->ci->pg_seo->url_template_transform('users', 'index', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'search', $data);

        $xml_data['users']['search'] = $this->ci->pg_seo->url_template_transform('users', 'search', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'view', $data);

        $xml_data['users']['view'] = $this->ci->pg_seo->url_template_transform('users', 'view', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'reg_company', $data);

        $xml_data['users']['reg_company'] = $this->ci->pg_seo->url_template_transform('users', 'reg_company', $data['url_template'], 'base', 'xml');

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users', 'reg_agent', $data);

        $xml_data['users']['reg_agent'] = $this->ci->pg_seo->url_template_transform('users', 'reg_agent', $data['url_template'], 'base', 'xml');

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        $this->ci->pg_seo->delete_seo_module("users");
    }

    // looks like not in use

    /**
     * Return languages for removing from menu module
     *
     * @param array  $langs    languages data
     * @param string $menu_gid menu guid
     * @param string $item_gid menu item guid
     *
     * @return array
     */
    public function get_menu_lang_delete($langs, $menu_gid, $item_gid)
    {
        $lang_data = array();
        foreach ($this->ci->pg_language->languages as $lang) {
            $lang_data[$lang["id"]] = $langs[$lang["code"]][$menu_gid][$item_gid];
        }

        return $lang_data;
    }

    /**
     * Install demo content of users module
     *
     * @return void
     */
    public function add_demo_content()
    {
        $this->ci->load->model("Users_model");

        $demo_content = Setup::getModuleData('users', Setup::TYPE_DEMO_CONTENT);
        foreach ((array) $demo_content['users'] as $user_data) {
            $user_data['repassword'] = $user_data['password'];
            $validate_data = $this->ci->Users_model->validate_user(null, $user_data);

            if (!empty($validate_data['errors'])) {
                continue;
            }
            $this->ci->Users_model->save_user(null, $validate_data['data']);
        }

        if ($this->ci->pg_module->is_module_active('reviews')) {
            $this->ci->load->model('Reviews_model');
            foreach ((array) $demo_content['reviews'] as $review) {
                $validate_data = $this->ci->Reviews_model->validate_review(null, $review);
                if (!empty($validate_data['errors'])) {
                    continue;
                }
                $this->ci->Reviews_model->save_review(null, $validate_data['data']);
            }
        }

        return true;
    }

}
