<?php

namespace Pg\Modules\Users_connections\Controllers;

/**
 * Users connections module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users connections user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage	Users connectiopns
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_connections extends \Controller
{

    /**
     * Constructor
     *
     * @return Users_connections
     */
    public function __construct()
    {
        parent::__construct();
		
		$this->load->model('Menu_model');
    }

    /**
     * Login user by data provided social service with oauth used
     *
     * @param integer $service_id      service identifier
     * @param string  $user_type       user type
     * @param integer $service_user_id user identifier provided by service
     *
     * @return void
     */
    public function oauth_login($service_id = false, $user_type = false, $service_user_id = false)
    {
        // Грузим модели
        $this->load->model('social_networking/models/Social_networking_services_model');
        $this->load->model('social_networking/models/Social_networking_connections_model');
        $this->load->model('Users_connections_model');
        // Данные
        $service = $this->Social_networking_services_model->get_service_by_id($service_id);

        // Проверка подключения
        if ($service['oauth_version'] == 2) {
            $result = $this->Social_networking_connections_model->_check_oauth2_connection($service,
                                                                                           site_url('users_connections/oauth_login/' . $service_id . '/' . $user_type));
        } else {
            $result = $this->Social_networking_connections_model->_check_oauth_connection($service,
                                                                                          site_url('users_connections/oauth_login/' . $service_id . '/' . $user_type));
        }

        // Авторизуем или посылаем на авторизацию
        if ($result['result']) {
                        
            // Если получен ключ ответа
            if (isset($result['result']['oauth_token'])) {
                $result['result']['access_token'] = $result['result']['oauth_token'];
            }
            if (isset($result['result']['access_token'])) {
                $result['result']['access_secret'] = isset($result['result']['oauth_token_secret']) ? $result['result']['oauth_token_secret'] : '';
                $result['result']['expires_in'] = isset($result['result']['expires_in']) ? $result['result']['expires_in'] : 0;
                $user_id = $this->session->userdata('user_id');
                $service_user_id = isset($result['result']['user_id']) ? $result['result']['user_id'] : false;
                $service_user_fname = '';
                $service_user_sname = '';
                $service_user_email = '';
                $service_model = $service['gid'] . '_service_model';
                $service_file = APPPATH . 'modules/social_networking/models/services/' . $service_model . '.php';
                if (file_exists($service_file)) {
                    include_once $service_file;
					$service_model = 'Pg\\Modules\\Social_networking\\Models\\services\\' . $service_model;
                    $this->service = new $service_model();
                    if (method_exists($this->service, 'get_user_data')) {
                        $user_data = $this->service->get_user_data($service_user_id, $result['result']['access_token'],
                                                                   $result['result']['access_secret']);
                        if (($user_data) && isset($user_data['id'])) {
                            $service_user_id = $user_data['id'];
                            $service_user_fname = $user_data['fname'];
                            $service_user_sname = $user_data['sname'];
                            $service_user_email = $user_data['email'];
                        }
                    }
                }
                                
                if ($service_user_id) {
                    $connection = $this->Users_connections_model->get_connection_by_data($service_id, $service_user_id);
                    if ($connection && isset($connection['id'])) {
                        $this->Users_connections_model->delete_connection($connection['id']);
                        $user_id = $connection['user_id'];

                        $this->load->model("users/models/Auth_model");
                        $auth_data = $this->Auth_model->login($user_id);
                        if (empty($auth_data["errors"])) {
                            $connection = array(
                                'service_id' => $service_id,
                                'user_id' => $user_id,
                                'access_token' => $result['result']['access_token'],
                                'access_token_secret' => $result['result']['access_secret'],
                                'data' => $service_user_id,
                                'date_end' => date("Y-m-d H:i:s", time() + $result['result']['expires_in']),
                            );
                            $this->Users_connections_model->save_connection(null, $connection);
                            
                            if ($_ENV['DEMO_MODE']) {
                                redirect('http://demo.realtysoft.pro/demo/?path=users/account');
                            } else {
                                redirect(site_url("users/account"));
                            }
                        }
                    }

                    $this->load->model('Users_model');

                    $user_types = ld('user_type', 'users');
                    $user_types = $user_types['option'];
                    foreach ($user_types as $user_type => $user_type_str) {
                        $register_enabled = $this->pg_module->get_module_config("users",
                                                                                $user_type . "_register_enabled");
                        if (!$register_enabled) {
                            unset($user_types[$user_type]);
                        }
                    }

                    if (empty($user_types)) {
                        $this->system_messages->addMessage(View::MSG_ERROR, l('error_invalid_settings', 'users'));
                        
                        $redirect_url = site_url();
                        
                        if ($_ENV['DEMO_MODE']) {
                            $redirect_url = str_replace('/realestate/', '/demo/', $redirect_url);
                        }
                        
                        redirect($redirect_url);
                    }

                    $this->view->assign('service_id', $service_id);
                    $this->view->assign('service_name', $service['name']);
                    $this->view->assign('access_token', $result['result']['access_token']);
                    $this->view->assign('access_token_secret', $result['result']['access_secret']);
                    $this->view->assign('date_end', date("Y-m-d H:i:s", time() + $result['result']['expires_in']));
                    $this->view->assign('service_user_id', $service_user_id);
                    $this->view->assign('service_user_fname', $service_user_fname);
                    $this->view->assign('service_user_sname', $service_user_sname);
                    $this->view->assign('service_user_email', $service_user_email);
                    $this->view->assign('user_types', $user_types);

					$this->Menu_model->breadcrumbs_set_active(str_replace('[service_name]', $service['name'], l('register_by', 'users_connections')));
					
                    $this->view->render('oauth_usertype');

                    return;
                }
            }
        }
        if ($result['error']) {
            $this->system_messages->addMessage(View::MSG_ERROR, $result['error']);
        }
        
        $redirect_url = site_url();
                        
        if ($_ENV['DEMO_MODE']) {
            $redirect_url = str_replace('/realestate/', '/demo/', $redirect_url);
        }
            
        redirect($redirect_url);
    }

    /**
     * Register user by data provided social service with oauth used
     *
     * @return void
     */
    public function oauth_register()
    {
        $service_id = $this->input->post('service_id', false);
        $access_token = $this->input->post('access_token', false);
        $access_token_secret = $this->input->post('access_token_secret', false);
        $date_end = $this->input->post('date_end', false);
        $service_user_id = $this->input->post('service_user_id', false);
        $service_user_fname = ($this->input->post('service_user_fname', false) ? $this->input->post('service_user_fname',
                                                                                                    false) : 'fname');
        $service_user_sname = ($this->input->post('service_user_sname', false) ? $this->input->post('service_user_sname',
                                                                                                    false) : 'sname');
        $this->load->library('Translit');
        $service_user_fname = $this->translit->convert('ru', $service_user_fname);
        $service_user_sname = $this->translit->convert('ru', $service_user_sname);
        $service_user_email = ($this->input->post('service_user_email', false) ? $this->input->post('service_user_email',
                                                                                                    false) : $service_user_fname . $service_user_sname . '@mail.com');

        $params = array();
        $params["where"]["email"] = $service_user_email;
        $this->load->model("Users_model");
        $count = $this->Users_model->get_users_count($params);
        if ($count > 0) {
            $service_user_email = rand(100, 300) . $service_user_email;
        }

        $lang_id = $this->pg_language->current_lang_id;
        $this->load->model('users/models/Users_groups_model');
        $group_id = $this->Users_groups_model->get_default_group_id();

        $user_data = array(
            'fname' => $service_user_fname,
            'sname' => $service_user_sname,
            'email' => $service_user_email,
            'confirm' => 1,
            'user_type' => $user_type,
            'status' => '1',
            'lang_id' => $lang_id,
            'group_id' => $group_id,
        );

        $user_type = "private";
        $post_user_type = $this->input->post('user_type', false);
        if (isset($post_user_type['private'])) {
            $user_type = "private";
        } elseif (isset($post_user_type['company'])) {
            $user_type = "company";
            $user_data['company_name'] = $service_user_fname . " " . $service_user_sname;
        } elseif (isset($post_user_type['agent'])) {
            $user_type = "agent";
        }
        $user_data['user_type'] = $user_type;

        if ($service_id && $access_token && $date_end && $service_user_id) {
            // Грузим модели
            $this->load->model('social_networking/models/Social_networking_services_model');
            $this->load->model('users_connections/models/Users_connections_model');
            // Данные
            $service = $this->Social_networking_services_model->get_service_by_id($service_id);
            if ($service) {
                $user_id = $this->Users_model->save_user(null, $user_data);

                $this->load->model("users/models/Auth_model");
                $auth_data = $this->Auth_model->login($user_id);

                if (!empty($auth_data["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $auth_data["errors"]);
                    
                    $redirect_url = site_url();
                        
                    if ($_ENV['DEMO_MODE']) {
                        $redirect_url = str_replace('/realestate/', '/demo/', $redirect_url);
                    }
                    
                    redirect($redirect_url);
                } else {
                    $connection = array(
                        'service_id' => $service_id,
                        'user_id' => $user_id,
                        'access_token' => $access_token,
                        'access_token_secret' => $access_token_secret,
                        'data' => $service_user_id,
                        'date_end' => $date_end,
                    );
                    $this->Users_connections_model->save_connection(null, $connection);
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('please_set_email', 'users_connections'));
                    
                    if ($_ENV['DEMO_MODE']) {
                        redirect('http://demo.realtysoft.pro/demo/?path=users/change_email');
                    } else {
                        redirect(site_url("users/change_email"));
                    }
                }
            }
        }
        
        $redirect_url = site_url();
                        
        if ($_ENV['DEMO_MODE']) {
            $redirect_url = str_replace('/realestate/', '/demo/', $redirect_url);
        }
                        
        redirect($redirect_url);
    }

    /**
     * Connection to social service by oauth
     *
     * @param integer $service_id service identifier
     *
     * @return void
     */
    public function oauth_link($service_id = false)
    {
        $user_id = $this->session->userdata('user_id');
        if ($user_id) {
            // Грузим модели
            $this->load->model('social_networking/models/Social_networking_services_model');
            $this->load->model('social_networking/models/Social_networking_connections_model');
            $this->load->model('Users_connections_model');
            // Данные
            $service = $this->Social_networking_services_model->get_service_by_id($service_id);
            // Проверка подключения
            if ($service['oauth_version'] == 2) {
                $result = $this->Social_networking_connections_model->_check_oauth2_connection($service,
                                                                                               site_url('users_connections/oauth_link/' . $service_id));
            } else {
                $result = $this->Social_networking_connections_model->_check_oauth_connection($service,
                                                                                              site_url('users_connections/oauth_link/' . $service_id));
            }
            // Авторизуем или посылаем на авторизацию
            if ($result['result']) {
                // Если получен ключ ответа
                if (isset($result['result']['oauth_token'])) {
                    $result['result']['access_token'] = $result['result']['oauth_token'];
                }
                if (isset($result['result']['access_token'])) {
                    $result['result']['access_secret'] = isset($result['result']['oauth_token_secret']) ? $result['result']['oauth_token_secret'] : '';
                    $result['result']['expires_in'] = isset($result['result']['expires_in']) ? $result['result']['expires_in'] : 0;
                    $service_user_id = isset($result['result']['user_id']) ? $result['result']['user_id'] : false;
                    $service_model = $service['gid'] . '_service_model';
                    $service_file = APPPATH . 'modules/social_networking/models/services/' . $service_model . '.php';
                    if (file_exists($service_file)) {
                        include_once $service_file;
                        $this->service = new $service_model();
                        if (method_exists($this->service, 'get_user_data')) {
                            $user_data = $this->service->get_user_data($service_user_id,
                                                                       $result['result']['access_token'],
                                                                       $result['result']['access_secret']);
                            if (($user_data) && isset($user_data['id'])) {
                                $service_user_id = $user_data['id'];
                            }
                        }
                    }
                    if ($service_user_id) {
                        $connection = $this->Users_connections_model->get_connection_by_data($service_id,
                                                                                             $service_user_id);
                        if ($connection && isset($connection['id'])) {
                            $this->Users_connections_model->delete_connection($connection['id']);
                        }
                        $connection = array(
                            'service_id' => $service_id,
                            'user_id' => $user_id,
                            'access_token' => $result['result']['access_token'],
                            'access_token_secret' => $result['result']['access_secret'],
                            'data' => $service_user_id,
                            'date_end' => date("Y-m-d H:i:s", time() + $result['result']['expires_in']),
                        );
                        $this->Users_connections_model->save_connection(null, $connection);
                        if (isset($service['name'])) {
                            $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                               l('account_link_to', 'users_connections') . ' ' . $service['name']);
                        }
                        
                        if ($_ENV['DEMO_MODE']) {
                            redirect('http://demo.realtysoft.pro/demo/?path=users/profile');
                        } else {
                            redirect(site_url("users/profile"));
                        }       
                    }
                }
            }
            if ($result['error']) {
                $this->system_messages->addMessage(View::MSG_ERROR, $result['error']);
            }
        }
        
        $redirect_url = site_url();
                        
        if ($_ENV['DEMO_MODE']) {
            $redirect_url = str_replace('/realestate/', '/demo/', $redirect_url);
        }
                    
        redirect($redirect_url);
    }

    /**
     * Disconnection from social service
     *
     * @param integer $service_id service idnetifier
     *
     * @return void
     */
    public function oauth_unlink($service_id = false)
    {
        // Грузим модели
        $this->load->model('social_networking/models/Social_networking_services_model');
        $this->load->model('Users_connections_model');
        $user_id = $this->session->userdata('user_id');
        $service = $this->Social_networking_services_model->get_service_by_id($service_id);
        if ($user_id && $service) {
            $connection = $this->Users_connections_model->get_connection_by_user_id($service_id, $user_id);
            if ($connection && isset($connection['id'])) {
                $this->Users_connections_model->delete_connection($connection['id']);
                if (isset($service['name'])) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                       l('account_unlink_from', 'users_connections') . ' ' . $service['name']);
                }
                
                if ($_ENV['DEMO_MODE']) {
                    redirect('http://demo.realtysoft.pro/demo/?path=users/profile');
                } else {
                    redirect(site_url("users/profile"));
                }
            }
        }
        
        $redirect_url = site_url();
                        
        if ($_ENV['DEMO_MODE']) {
            $redirect_url = str_replace('/realestate/', '/demo/', $redirect_url);
        }
                        
        redirect($redirect_url);
    }

}
