<?php

/**
 * Social networking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Social networking facebook service model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Social networking
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Facebook_service_model extends Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    public $CI;

    /**
     * Url of facebook service api
     *
     * @var string
     */
    public $api_url = 'https://graph.facebook.com/';

    /**
     * Class constructor
     *
     * @return Facebook_service_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
    }

    /**
     * Return user data from facebook
     *
     * @param integer $user_id    user identifier
     * @param string  $access_key access token
     *
     * @return array/false
     */
    public function get_user_data($user_id = 0, $access_key = '')
    {
        $this->CI->load->model('Users_connections_model');
        $params = array(
            'access_token' => $access_key,
            'fields'       => 'id,first_name,last_name,email',
        );
        $response = $this->CI->Users_connections_model->curl_get($this->api_url . 'me', $params);
        $data = (array) @json_decode($response);
        if (isset($data['id']) && isset($data['first_name']) && isset($data['last_name'])) {
            $user_data = array(
                'id'    => $data['id'],
                'fname' => $data['first_name'],
                'sname' => $data['last_name'],
            );
            if (isset($data['email'])) {
                $user_data['email'] = $data['email'];
            }

            return $user_data;
        }

        return false;
    }

    /**
     * Return parameters of authorization request
     *
     * @param array  $service_data service data
     * @param string $redirect     redirect url
     *
     * @return array
     */
    public function get_auth_params($service_data = array(), $redirect)
    {
        $app_key = isset($service_data['app_key']) ? $service_data['app_key'] : false;
        $params = array(
            'client_id'     => $app_key,
            'redirect_uri'  => $redirect,
            'response_type' => 'code',
            'scope'         => 'email',
        );
        ksort($params);

        return $params;
    }
}
