<?php

namespace Pg\Modules\Users_deactivate\Controllers;

/**
 * Users deactivate module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users admin deactivate side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users deactivate
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Users_deactivate extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Users_deactivate
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "feedbacks-items");
    }

    /**
     * Main page
     *
     * @return void
     */
    public function index()
    {
        $this->deactivated_alerts();
    }

    /**
     * Manage user deactivation settings
     *
     * @return void
     */
    public function settings()
    {
        $this->deactivated_settings();
    }

    /**
     * Manage user deactivation settings
     *
     * @return void
     */
    public function deactivated_settings()
    {
        $this->Menu_model->set_menu_active_item("admin_users_settings_menu", "users_deactivated_settings_item");

        if ($this->input->post("btn_save")) {
            $this->load->model("users_deactivate/models/Users_deactivated_alert_model");

            $post_data["deactivated_send_mail"] = $this->input->post("deactivated_send_mail") ? 1 : 0;
            $post_data["deactivated_send_alert"] = $this->input->post("deactivated_send_alert") ? 1 : 0;
            $post_data["deactivated_email"] = $this->input->post("deactivated_email");

            $validate_data = $this->Users_deactivated_alert_model->validate_settings($post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $data = $validate_data["data"];
            } else {
                foreach ($validate_data["data"] as $setting => $value) {
                    $this->pg_module->set_module_config("users_deactivate", $setting, $value);
                }
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "users_deactivate"));
                $data = $validate_data["data"];
            }
        } else {
            $data["deactivated_send_mail"] = $this->pg_module->get_module_config("users_deactivate", "deactivated_send_mail");
            $data["deactivated_send_alert"] = $this->pg_module->get_module_config("users_deactivate", "deactivated_send_alert");
            $data["deactivated_email"] = $this->pg_module->get_module_config("users_deactivate", "deactivated_email");
        }

        $this->view->assign("data", $data);
        $this->view->setHeader(l("admin_header_deactivated_alerts_list", "users_deactivate"));
        $this->view->render("deactivated_settings");
    }

    /**
     * Deactivated alerts management
     *
     * @param string  $order           sorting order
     * @param string  $order_direction sorting direction
     * @param integer $page            page of results
     * @result void
     */
    public function deactivated_alerts($order = null, $order_direction = null, $page = null)
    {
        $this->Menu_model->set_menu_active_item("admin_users_settings_menu", "deactivated_alerts_item");

        $this->load->model("users_deactivate/models/Users_deactivated_alert_model");

        $current_settings = isset($_SESSION["users_deactivated_alerts_list"]) ? $_SESSION["users_deactivated_alerts_list"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_add";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        if (!$order) {
            $order = $current_settings["order"];
        }
        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        if (!$page) {
            $page = $current_settings["page"];
        }

        $current_settings["page"] = $page;

        if (!$order) {
            $order = $current_settings["order"];
        }
        if ($order == "reason") {
            $order = "id_reason";
        }
        $this->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $this->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        $alerts_count = $this->Users_deactivated_alert_model->get_alerts_count();

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "admin_items_per_page");

        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $alerts_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["users_deactivated_alert_list"] = $current_settings;

        $sort_links = array(
            "name" => site_url() . "admin/users_deactivate/deactivated_alerts/name/" . (($order != "name" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "email" => site_url() . "admin/users_deactivate/deactivated_alerts/email/" . (($order != "email" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "id_reason" => site_url() . "admin/users_deactivate/deactivated_alerts/reason/" . (($order != "id_reason" xor $order_direction == "DESC") ? "ASC" : "DESC"),
            "date_add" => site_url() . "admin/users_deactivate/deactivated_alerts/date_add/" . (($order != "date_add" xor $order_direction == "DESC") ? "ASC" : "DESC"),
        );

        $this->view->assign("sort_links", $sort_links);

        if ($alerts_count > 0) {
            $this->Users_deactivated_alert_model->set_format_settings(array("get_reason" => true));
            $alerts = $this->Users_deactivated_alert_model->get_alerts_list($page, $items_on_page, array($order => $order_direction));
            $this->view->assign("alerts", $alerts);
        }

        $this->load->helper('navigation');

        $url = site_url() . "admin/users_deactivate/deactivated_alerts/" . $order . "/" . $order_direction . "/";
        $page_data = get_admin_pages_data($url, $alerts_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $this->view->setHeader(l("admin_header_deactivated_alerts_list", "users_deactivate"));
        $this->view->render("deactivated_alerts_list");
    }

    /**
     * Display deactivated alert
     *
     * @param integer $id deactivated alert identifier
     *
     * @return void
     */
    public function deactivated_alerts_show($id)
    {
        $this->Menu_model->set_menu_active_item("admin_menu", "feedbacks-items");

        $this->load->model("users_deactivate/models/Users_deactivated_alert_model");
        $this->Users_deactivated_alert_model->set_format_settings(array("get_reason" => true));
        $data = $this->Users_deactivated_alert_model->get_alert_by_id($id, true);
        $this->view->assign("data", $data);

        $this->Users_deactivated_alert_model->mark_as_read($id);

        $this->load->helper('navigation');

        $cur_set = $_SESSION["users_deactivated_alert_list"];
        $filter = $cur_set["order"] . "/" . $cur_set["order_direction"] . "/" . $cur_set["page"];
        $this->view->assign("filter", $filter);

        $date_format = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("date_format", $date_format);
        $this->view->setHeader(l("admin_header_deactivated_alerts_list", "users_deactivate"));
        $this->view->render("deactivated_alert_view");
    }

    /**
     * Remove deactivated alert
     *
     * @param array $ids alerts identifiers
     *
     * @return void
     */
    public function deactivated_alerts_delete($ids = null)
    {
        $errors = false;
        $messages = array();
        if (!$ids) {
            $ids = $this->input->post("ids");
        }
        if (!empty($ids)) {
            $this->load->model("users_deactivate/models/Users_deactivated_alert_model");
            foreach ((array) $ids as $id) {
                $error = $this->Users_deactivated_alert_model->delete_alert($id);
                if ($error) {
                    $errors = true;
                    $messages[] = $error;
                } else {
                    $messages[] = l("success_deleted_deactivated_alert", "users_deactivate");
                }
            }
            if ($errors) {
                $this->system_messages->addMessage(View::MSG_ERROR, $messages);
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS, $messages);
            }
        }

        $cur_set = $_SESSION["users_deactivated_alert_list"];
        $url = site_url() . "admin/users_deactivate/deactivated_alerts/" . $cur_set["order"] . "/" . $cur_set["order_direction"] . "/" . $cur_set["page"];
        $this->view->setRedirect($url);
    }

    /**
     * Deactivated reasons management
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function deactivated_reasons($lang_id = null)
    {
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");
        $this->Menu_model->set_menu_active_item("admin_users_settings_menu", "users_deactivated_reasons_item");

        $this->load->model("users_deactivate/models/Users_deactivated_reason_model");

        if (!$lang_id || !array_key_exists($lang_id, $this->pg_language->languages)) {
            $lang_id = $this->pg_language->current_lang_id;
        }

        $reference = $this->pg_language->ds->get_reference($this->Users_deactivated_reason_model->module_gid, $this->Users_deactivated_reason_model->content[0], $lang_id);

        $this->view->assign("langs", $this->pg_language->languages);
        $this->view->assign("current_lang_id", $lang_id);
        $this->view->assign("reference", $reference);
        $this->view->assign("module_gid", $this->Users_deactivated_reason_model->module_gid);

        $this->view->setHeader(l("admin_header_deactivated_reasons", "users_deactivate"));
        $this->view->setHeader(l("admin_header_deactivated_alerts_list", "users_deactivate"));

        $this->view->render("deactivated_reasons");
    }

    /**
     * Edit deactivated reason
     *
     * @param integer $lang_id    language identifier
     * @param string  $option_gid option guid
     *
     * @return void
     */
    public function deactivated_reason_edit($lang_id, $option_gid = null)
    {
        $this->load->model("users_deactivate/models/Users_deactivated_reason_model");

        if (!$lang_id || !array_key_exists($lang_id, $this->pg_language->languages)) {
            $lang_id = $this->pg_language->current_lang_id;
        }

        $reference = $this->pg_language->ds->get_reference($this->Users_deactivated_reason_model->module_gid, $this->Users_deactivated_reason_model->content[0], $lang_id);
        if ($option_gid) {
            $add_flag = false;
            foreach ($this->pg_language->languages as $lid => $lang) {
                $r = $this->pg_language->ds->get_reference($this->Users_deactivated_reason_model->module_gid, $this->Users_deactivated_reason_model->content[0], $lid);
                $lang_data[$lid] = $r["option"][$option_gid];
            }
        } else {
            $option_gid = "";
            $lang_data = array();
            $add_flag = true;
        }

        if ($this->input->post("btn_save")) {
            $lang_data = $this->input->post("lang_data", true);

            $validate_data = $this->Users_deactivated_reason_model->validate_reason($option_gid, $lang_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Users_deactivated_reason_model->save_reason($option_gid, $validate_data["langs"]);
                if ($add_flag) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_reason_created", "users_deactivate"));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_reason_updated", "users_deactivate"));
                }
                $url = site_url() . "admin/users_deactivate/deactivated_reasons/" . $lang_id;
                $this->view->setRedirect($url);
            }
        }

        $this->view->assign("lang_data", $lang_data);
        $this->view->assign("langs", $this->pg_language->languages);
        $this->view->assign("current_lang_id", $lang_id);
        $this->view->assign("current_module_gid", $this->Users_deactivated_reason_model->module_gid);
        $this->view->assign("option_gid", $option_gid);
        $this->view->assign("add_flag", $add_flag);

        $this->view->setHeader(l("admin_header_deactivated_alerts_list", "users_deactivate"));

        $this->Menu_model->set_menu_active_item("admin_users_settings_menu", "users_deactivated_reasons_item");
        $this->Menu_model->set_menu_active_item("admin_menu", "system-items");

        $this->view->render("deactivated_reasons_edit");
    }

    /**
     * Resort reasons options by ajax
     *
     * @return void
     */
    public function ajax_deactivated_reason_save_sorter()
    {
        $this->load->model("users_deactivate/models/Users_deactivated_reason_model");

        $sorter = $this->input->post("sorter");
        foreach ($sorter as $parent_str => $items_data) {
            foreach ($items_data as $item_str => $sort_index) {
                $option_gid = str_replace("item_", "", $item_str);
                $sorter_data[$sort_index] = $option_gid;
            }
        }

        if (empty($sorter_data)) {
            return;
        }
        ksort($sorter_data);
        $this->pg_language->ds->set_reference_sorter($this->Users_deactivated_reason_model->module_gid, $this->Users_deactivated_reason_model->content[0], $sorter_data);

        return;
    }

    /**
     * Remove deactivated reason from options list
     *
     * @param string $option_gid option guid
     *
     * @return void
     */
    public function ajax_deactivated_reason_delete($option_gid)
    {
        $this->load->model("users_deactivate/models/Users_deactivated_reason_model");

        if ($option_gid) {
            foreach ($this->pg_language->languages as $lid => $lang) {
                $reference = $this->pg_language->ds->get_reference($this->Users_deactivated_reason_model->module_gid, $this->Users_deactivated_reason_model->content[0], $lid);
                if (isset($reference["option"][$option_gid])) {
                    unset($reference["option"][$option_gid]);
                    $this->pg_language->ds->set_module_reference($this->Users_deactivated_reason_model->module_gid, $this->Users_deactivated_reason_model->content[0], $reference, $lid);
                }
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_reason_deleted", "users_deactivate"));
        }

        return;
    }

}
