<?php

namespace Pg\Modules\Users_deactivate\Models;

/**
 * Users deactivate module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users deactivate install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users deactivate
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_deactivate_install_model extends \Model
{
    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(

        "company_account_menu" => array(
            "action" => "none",
            "name" => "",
            "items"  => array(
                "company_my_profile_item" => array(
                    "action" => "none",
                    "items"  => array(
                        "company_my_profile_deactivate_item" => array("action" => "create", "link" => "users_deactivate/index", "status" => 1, "sorter" => 4),
                    ),
                ),
            ),
        ),

        "agent_account_menu" => array(
            "action" => "none",
            "name" => "",
            "items"  => array(
                "agent_my_profile_item" => array(
                    "action" => "none",
                    "items"  => array(
                        "agent_my_profile_deactivate_item" => array("action" => "create", "link" => "users_deactivate/index", "status" => 1, "sorter" => 4),
                    ),
                ),
            ),
        ),
        "admin_menu" => array(
            "action" => "none",
            "name" => "",
            "items"  => array(
                "feedbacks-items" => array(
                    "action" => "none",
                    "items"  => array(
                        "users_deact_alers_menu_item" => array("action" => "create", "link" => "admin/users_deactivate/deactivated_alerts", "status" => 1, "sorter" => 4),
                    ),
                ),
            ),
        ),
        "admin_users_deactivate_menu_list" => array(
            "action" => "create",
            "name"   => "Admin mode - Content - Profile de-activation",
            "items"  => array(
                "deactivated_alerts"   => array("action" => "create", "link" => "admin/users_deactivate/deactivated_alerts", "status" => 1, "sorter" => 1),
                "deactivated_settings" => array("action" => "create", "link" => "admin/users_deactivate/deactivated_settings", "status" => 1, "sorter" => 2),
                "deactivated_reasons"  => array("action" => "create", "link" => "admin/users_deactivate/deactivated_reasons", "status" => 1, "sorter" => 3),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators = array(
        array("module" => "users_deactivate", "method" => "index", "is_default" => 1),
    );

    /**
     * Seo configuration
     *
     * @var array
     */
    protected $seo = array(
        "module_gid"              => "users_deactivate",
        "model_name"              => "Users_deactivate_model",
        "get_settings_method"     => "get_seo_settings",
        "get_rewrite_vars_method" => "request_seo_rewrite",
        "get_sitemap_urls_method" => "get_sitemap_xml_urls",
    );

    /**
     * Notifications configuration
     *
     * @var array
     */
    protected $notifications = array(
        "templates" => array(
            array("gid" => "users_deactivate", "name" => "Deactivate account", "vars" => array("email", "fname", "sname"), "content_type" => "text"),
            array("gid" => "moderators_deactivated", "name" => "Deactivate account (for admin)", "vars" => array("email", "fname", "sname"), "content_type" => "text"),
        ),
        "notifications" => array(
            array("gid" => "users_deactivate", "template" => "users_deactivate", "send_type" => "simple"),
            array("gid" => "moderators_deactivated", "template" => "moderators_deactivated", "send_type" => "simple"),
        ),
    );

    /**
     * Constructor
     *
     * @return Users_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Install users_deactivate data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, "create", $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import users_deactivate languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users_deactivate", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data");
            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, "update", $gid, 0, $this->menu[$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export users_deactivate languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, "export", $gid, 0, $this->menu[$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall users_deactivate data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]["items"]);
            }
        }
    }

    /**
     * Install users_deactivate data to site map module
     *
     * @return void
     */
    public function install_site_map()
    {
        // Site Map
        $this->ci->load->model("Site_map_model");
        $site_map_data = array(
            "module_gid"      => "users_deactivate",
            "model_name"      => "users_deactivate_model",
            "get_urls_method" => "get_sitemap_urls",
        );
        $this->ci->Site_map_model->set_sitemap_module("users_deactivate", $site_map_data);
    }

    /**
     * Uninstall users_deactivate data from site map module
     *
     * @return void
     */
    public function deinstall_site_map()
    {
        $this->ci->load->model("Site_map_model");
        $this->ci->Site_map_model->delete_sitemap_module("users_deactivate");
    }

    /**
     * Install users_deactivate data to moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model("Moderators_model");

        foreach ((array) $this->moderators as $method_data) {
            $validate_data = array("errors" => array(), "data" => $method_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Moderators_model->save_method(null, $validate_data["data"]);
        }
    }

    /**
     * Import users_deactivate languages to moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users_deactivate", "moderators", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty moderators langs data");

            return false;
        }

        // install moderators permissions
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "users_deactivate";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method["method"]])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method["method"]]);
            }
        }

        return true;
    }

    /**
     * Export users_deactivate languages from moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "users_deactivate";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method["method"]] = $method["langs"];
        }

        return array("moderators" => $return);
    }

    /**
     * Uninstall users_deactivate data from moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        // delete moderation methods in moderators
        $this->ci->load->model("Moderators_model");
        $params = array();
        $params["where"]["module"] = "users_deactivate";
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install users data to notifications module
     *
     * @return void
     */
    public function install_notifications()
    {
        // add notification
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        $templates_ids = array();

        foreach ((array) $this->notifications["templates"] as $template_data) {
            if (is_array($template_data["vars"])) {
                $template_data["vars"] = implode(", ", $template_data["vars"]);
            }
            $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $templates_ids[$template_data['gid']] = $this->ci->Templates_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->notifications["notifications"] as $notification_data) {
            if (!isset($templates_ids[$notification_data["template"]])) {
                $template = $this->ci->Templates_model->get_template_by_gid($notification_data["template"]);
                $templates_ids[$notification_data["template"]] = $template["id"];
            }
            $notification_data["id_template_default"] = $templates_ids[$notification_data["template"]];
            $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Notifications_model->save_notification(null, $validate_data["data"]);
        }
    }

    /**
     * Import users_deactivate languages to notifiactions module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_notifications_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model("Notifications_model");

        $langs_file = $this->ci->Install_model->language_file_read("users_deactivate", "notifications", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty notifications langs data");

            return false;
        }

        $this->ci->Notifications_model->update_langs($this->notifications, $langs_file, $langs_ids);
        return true;
    }

    /**
     * Export users languages from notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_notifications_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Notifications_model");
        $langs = $this->ci->Notifications_model->export_langs($this->notifications, $langs_ids);

        return array("notifications" => $langs);
    }

    /**
     * Uninstall users_deactivate data from notifications module
     *
     * @return void
     */
    public function deinstall_notifications()
    {
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach ((array) $this->notifications["notifications"] as $notification_data) {
            $this->ci->Notifications_model->delete_notification_by_gid($notification_data["gid"]);
        }

        foreach ((array) $this->notifications["templates"] as $template_data) {
            $this->ci->Templates_model->delete_template_by_gid($template_data["gid"]);
        }
    }

    /**
     * Install data of users_deactivate module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {

        // seo
        $this->ci->pg_seo->set_seo_module("users_deactivate", (array) $this->seo);

        // add seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex'         => 1,
            'title'           => '',
            'keyword'         => '',
            'description'     => '',
            'header'          => '',
            'og_title'        => '',
            'og_type'         => '',
            'og_descrtiption' => '',
            'url_template'    => '[text:users_deactivate]',
        );
        $this->ci->pg_seo->set_settings('user', 'users_deactivate', 'deactivate_profile', $data);
    }

    /**
     * Import languages of users module
     *
     * @return boolean
     */
    public function _arbitrary_lang_install()
    {
    }

    /**
     * Export languages of users module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
    }

    /**
     * Unistall data of users module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        // remove seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex'         => 1,
            'title'           => '',
            'keyword'         => '',
            'description'     => '',
            'header'          => '',
            'og_title'        => '',
            'og_type'         => '',
            'og_descrtiption' => '',
            'url_template'    => '',
        );
        $this->ci->pg_seo->set_settings('user', 'users_deactivate', 'deactivate_profile', $data);
        $this->ci->pg_seo->delete_seo_module("users_deactivate");
    }

    // looks like not in use

    /**
     * Return languages for removing from menu module
     *
     * @param array  $langs    languages data
     * @param string $menu_gid menu guid
     * @param string $item_gid menu item guid
     *
     * @return array
     */
    public function get_menu_lang_delete($langs, $menu_gid, $item_gid)
    {
        $lang_data = array();
        foreach ($this->ci->pg_language->languages as $lang) {
            $lang_data[$lang["id"]] = $langs[$lang["code"]][$menu_gid][$item_gid];
        }

        return $lang_data;
    }
}
