<?php

namespace Pg\Modules\Users_deactivate\Models;

/**
 * Users deactivate module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

/**
 * Users deactivate main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users deactivate
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_deactivate_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Class constructor
     *
     * @return Users_deactivate_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return settings for rewriting seo url and metatags
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function get_seo_settings($method = "", $lang_id = "")
    {
        if (!empty($method)) {
            return $this->_get_seo_settings($method, $lang_id);
        } else {
            $actions = array('deactivate_profile');
            $this->ci->load->model('Users_model');
            foreach ($this->ci->Users_model->getUserTypes() as $user_type) {
                $actions[] = "reg_" . $user_type;
            }
            $return = array();
            foreach ($actions as $action) {
                $return[$action] = $this->_get_seo_settings($action, $lang_id);
            }

            return $return;
        }
    }

    /**
     * Return settings for rewriting seo url and metatags (internal)
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    protected function _get_seo_settings($method, $lang_id = "")
    {
        switch ($method) {
            case "deactivate_profile":
                return array(
                    "templates" => array(),
                    "url_vars" => array(),
                );
                break;
        }
    }

    /**
     * Transform seo values from query string to module variables
     *
     * @param string $var_name_from variable name in url
     * @param string $var_name_to   variable name in method
     * @param mixed  $value         variable value
     *
     * @return mixed
     */
    public function request_seo_rewrite($var_name_from, $var_name_to, $value)
    {
        $user_data = array();

        if ($var_name_from == $var_name_to) {
            switch ($var_name_to) {
                case "name":
                case "type":
                    return '';
                    break;
                default:
                    return $value;
                    break;
            }
        }
    }

    /**
     * Return urls of pages for xml sitemap
     *
     * @return array
     */
    public function get_sitemap_xml_urls()
    {
        return array();
    }

    /**
     * Return urls of pages for site map
     *
     * @return array
     */
    public function get_sitemap_urls()
    {
        $this->ci->load->helper('seo');
        $auth = $this->ci->session->userdata("auth_type");

        $block[] = array(
            "name" => l('text_deactivate_profile', 'users_deactivate'),
            "link" => rewrite_link('users_deactivate', 'deactivate_profile'),
            "clickable" => ($auth == "user") ? true : false,
        );

        return $block;
    }

    /**
     * Return available banners places
     *
     * Callback method of banners module
     *
     * @return array
     */
    public function _banner_available_pages()
    {
        $return[] = array("link" => "users_deactivate/deactivate_profile", "name" => l("text_deactivate_profile", "users_deactivate"));

        return $return;
    }

}
