<?php

namespace Pg\Modules\Users_payments\Controllers;

/**
 * Users payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users payments user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users payments
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_payments extends \Controller
{

    /**
     * Constructor
     *
     * @return Users_payments
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Save payment for payment system
     *
     * @return void
     */
    public function save_payment()
    {
        if ($this->input->post('btn_payment_save')) {
            $user_id = $this->session->userdata('user_id');
            $amount = floatval($this->input->post('amount', true));
            $system_gid = $this->input->post('system_gid', true);

            if (empty($amount)) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_empty_amount', 'users_payments'));
            } elseif ($amount < 0) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_invalid_amount', 'users_payments'));
            } elseif (empty($system_gid)) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_empty_system_gid', 'users_payments'));
            } else {
                $base_currency_gid = $this->pg_module->get_module_config('start', 'base_currency');

                $this->load->model('Currencies_model');
                $default_currency = $this->Currencies_model->default_currency;

                $this->load->helper('payments');

                $additional['name'] = l('header_add_funds', 'users_payments');
                $payment_data = send_payment('account', $user_id, $amount, $default_currency['gid'], $system_gid,
                                             $additional, 'form');
                if (!empty($payment_data['errors'])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $payment_data['errors']);
                }
                if (!empty($payment_data['info'])) {
                    $this->system_messages->addMessage(View::MSG_INFO, $payment_data['info']);
                }
            }
        }
        redirect(site_url() . 'users/account');
    }

}
