<?php

namespace Pg\Modules\Users_services\Models;

use Pg\Libraries\Setup;

/**
 * Users services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users payments install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users services
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_services_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    protected $modules_data;

    /**
     * Constructor
     *
     * @return Users_services_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->modules_data = Setup::getModuleData('users_services', Setup::TYPE_MODULES_DATA);
    }

    /**
     * Install users services data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $this->modules_data['menu'][$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->modules_data['menu'], "create", $gid, 0, $this->modules_data['menu'][$gid]["items"]);
        }
    }

    /**
     * Import users services languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users_services", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data");
            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            linked_install_process_menu_items($this->modules_data['menu'], "update", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_file);
        }
        return true;
    }

    /**
     * Export users services languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->modules_data['menu'], "export", $gid, 0, $this->modules_data['menu'][$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall users services data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper("menu");
        foreach ($this->modules_data['menu'] as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->modules_data['menu'][$gid]["items"]);
            }
        }
    }

    /**
     * Install users services data to services module
     *
     * @return void
     */
    public function install_services()
    {
        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $validate_data = $this->ci->Services_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Services_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $validate_data = $this->ci->Services_model->validate_service(null, $service_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Services_model->save_service(null, $validate_data["data"]);
        }
    }

    /**
     * Import users services languages to services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_services_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users_services", "services", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty services langs data");

            return false;
        }

        $services_data = array(
            "service" => array(),
            "template" => array(),
            "param" => array(),
        );

        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $data_admin = isset($template_data["data_admin"]) ? array_keys($template_data["data_admin"]) : array();
            $data_user = isset($template_data["data_user"]) ? array_keys($template_data["data_user"]) : array();
            $data_membership = isset($template_data["data_membership"]) ? array_keys($template_data["data_membership"]) : array();
            $services_data["template"][] = $template_data["gid"];
            $services_data["param"][$template_data["gid"]] = array_unique(array_merge($data_admin, $data_user, $data_membership));
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $services_data["service"][] = $service_data["gid"];
        }

        $this->ci->Services_model->update_langs($services_data, $langs_file);

        return true;
    }

    /**
     * Export users services languages from services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_services_lang_export($langs_ids = null)
    {
        $services_data = array(
            "service" => array(),
            "template" => array(),
            "param" => array(),
        );

        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $data_admin = isset($template_data["data_admin"]) ? array_keys($template_data["data_admin"]) : array();
            $data_user = isset($template_data["data_user"]) ? array_keys($template_data["data_user"]) : array();
            $data_membership = isset($template_data["data_membership"]) ? array_keys($template_data["data_membership"]) : array();
            $services_data["template"][] = $template_data["gid"];
            $services_data["param"][$template_data["gid"]] = array_merge($data_admin, $data_user, $data_membership);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $services_data["service"][] = $service_data["gid"];
        }

        return array("services" => $this->ci->Services_model->export_langs($services_data, $langs_ids));
    }

    /**
     * Uninstall users services data from services module
     *
     * @return void
     */
    public function deinstall_services()
    {
        $this->ci->load->model("Services_model");

        foreach ((array) $this->modules_data['services']["templates"] as $template_data) {
            $this->ci->Services_model->delete_template_by_gid($template_data["gid"]);
        }

        foreach ((array) $this->modules_data['services']["services"] as $service_data) {
            $this->ci->Services_model->delete_service_by_gid($service_data["gid"]);
        }
    }

    /**
     * Install users services data to notifications module
     *
     * @return void
     */
    public function install_notifications()
    {
        // add notification
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        $templates_ids = array();

        foreach ((array) $this->modules_data['notifications']["templates"] as $template_data) {
            if (is_array($template_data["vars"])) {
                $template_data["vars"] = implode(",", $template_data["vars"]);
            }

            $validate_data = $this->ci->Templates_model->validate_template(null, $template_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $templates_ids[$template_data['gid']] = $this->ci->Templates_model->save_template(null, $validate_data["data"]);
        }

        foreach ((array) $this->modules_data['notifications']["notifications"] as $notification_data) {
            if (!isset($templates_ids[$notification_data["template"]])) {
                $template = $this->ci->Templates_model->get_template_by_gid($notification_data["template"]);
                $templates_ids[$notification_data["template"]] = $template["id"];
            }
            $notification_data["id_template_default"] = $templates_ids[$notification_data["template"]];
            $validate_data = $this->ci->Notifications_model->validate_notification(null, $notification_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Notifications_model->save_notification(null, $validate_data["data"]);
        }
    }

    /**
     * Import users services languages to notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_notifications_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }

        $this->ci->load->model("Notifications_model");

        $langs_file = $this->ci->Install_model->language_file_read("users_services", "notifications", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty notifications langs data");

            return false;
        }

        $this->ci->Notifications_model->update_langs($this->modules_data['notifications'], $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users services languages from notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_notifications_lang_export($langs_ids = null)
    {
        if (!is_array($langs_ids)) {
            $langs_ids = (array) $langs_ids;
        }
        $this->ci->load->model("Notifications_model");
        $langs = $this->ci->Notifications_model->export_langs((array) $this->modules_data['notifications'], $langs_ids);

        return array("notifications" => $langs);
    }

    /**
     * Uninstall users services data from notifications module
     *
     * @return void
     */
    public function deinstall_notifications()
    {
        // add notification
        $this->ci->load->model("Notifications_model");
        $this->ci->load->model("notifications/models/Templates_model");

        foreach ((array) $this->modules_data['notifications']["notifications"] as $notification_data) {
            $this->ci->Notifications_model->delete_notification_by_gid($notification_data["gid"]);
        }

        foreach ((array) $this->modules_data['notifications']["templates"] as $template_data) {
            $this->ci->Templates_model->delete_template_by_gid($template_data["gid"]);
        }
    }

    /**
     * Install users data to dynamic blocks module
     *
     * @return void
     */
    public function install_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");

        $area_ids = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $validate_data = $this->ci->Dynamic_blocks_model->validate_block(null, $block_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $id_block = $this->ci->Dynamic_blocks_model->save_block(null, $validate_data["data"]);

            if (!empty($block_data["area"])) {
                if (!isset($area_ids[$block_data["area"]])) {
                    $area = $this->ci->Dynamic_blocks_model->get_area_by_gid($block_data["area"]);
                    $area_ids[$block_data["area"]] = $area["id"];
                }

                // index area
                $area_data = array(
                    "id_area" => $area_ids[$block_data["area"]],
                    "id_block" => $id_block,
                );
                $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block(null, $area_data);
                if (empty($validate_data["errors"])) {
                    $temp_id = $this->ci->Dynamic_blocks_model->save_area_block(null, $validate_data["data"]);
                    $this->ci->Dynamic_blocks_model->save_area_block_sorter($temp_id, $block_data["sorter"]);
                }
            }

            if (!empty($block_data["presets"])) {
                foreach ($block_data["presets"] as $preset_data) {
                    $preset = $this->ci->Dynamic_blocks_model->get_preset_by_gid($preset_data['gid_preset']);
                    $preset_data['id_preset'] = $preset['id'];
                    $preset_data['id_block'] = $id_block;
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block(null, $preset_data);
                    if (!empty($validate_data["errors"])) {
                        continue;
                    }
                    $this->ci->Dynamic_blocks_model->save_preset_block(null, $validate_data["data"]);
                }
            }
        }
    }

    /**
     * Import users languages to dynamic blocks module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_dynamic_blocks_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        $langs_file = $this->ci->Install_model->language_file_read("users_services", "dynamic_blocks", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty dynamic_blocks langs data");

            return false;
        }

        $this->ci->load->model("Dynamic_blocks_model");

        $data = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }

        $this->ci->Dynamic_blocks_model->update_langs($data, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users languages from dynamic blocks module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_dynamic_blocks_lang_export($langs_ids = null)
    {
        $this->ci->load->model("Dynamic_blocks_model");

        $data = array();

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $block = $this->ci->Dynamic_blocks_model->get_block_by_gid($block_data["gid"]);
            if (!$block) {
                continue;
            }
            $data[] = $block;
        }

        $langs = $this->ci->Dynamic_blocks_model->export_langs($data, $langs_ids);

        return array("dynamic_blocks" => $langs);
    }

    /**
     * Unistall users data from dynamic blocks module
     *
     * @return void
     */
    public function deinstall_dynamic_blocks()
    {
        $this->ci->load->model("Dynamic_blocks_model");

        foreach ((array) $this->modules_data['dynamic_blocks'] as $block_data) {
            $this->ci->Dynamic_blocks_model->delete_block_by_gid($block_data["gid"]);
        }
    }

    /**
     * Install users services data to cronjob module
     *
     * @return void
     */
    public function install_cronjob()
    {
        $this->ci->load->model("Cronjob_model");

        foreach ((array) $this->modules_data['cronjob'] as $cron_data) {
            $validate_data = $this->ci->Cronjob_model->validate_cron(null, $cron_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Cronjob_model->save_cron(null, $validate_data["data"]);
        }
    }

    /**
     * Uninstall users services data from cronjob module
     *
     * @return void
     */
    public function deinstall_cronjob()
    {
        $this->ci->load->model("Cronjob_model");
        $cron_data = array();
        $cron_data["where"]["module"] = "users_services";
        $this->ci->Cronjob_model->delete_cron_by_param($cron_data);
    }

    public function install_seo()
    {
        $this->ci->pg_seo->set_seo_module("users_services", (array) $this->modules_data['seo']);
        $this->ci->load->model('Seo_model');
        $this->ci->Seo_model->addSeoLinks('users_services', $this->modules_data['seo_links']);
    }

    public function install_seo_lang_install($langs_ids)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users_services", "seo", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty users_services seo langs data");
            return false;
        }

        foreach($this->modules_data['seo_links'] as $seo_link) {
            $link_data = array(
                "title" => $langs_file["seo_tags_{$seo_link['method']}_title"],
                "keyword" => $langs_file["seo_tags_{$seo_link['method']}_keyword"],
                "description" => $langs_file["seo_tags_{$seo_link['method']}_description"],
                "header" => $langs_file["seo_tags_{$seo_link['method']}_header"],
                "og_title" => $langs_file["seo_tags_{$seo_link['method']}_og_title"],
                "og_type" => $langs_file["seo_tags_{$seo_link['method']}_og_type"],
                "og_description" => $langs_file["seo_tags_{$seo_link['method']}_og_description"],
            );
            $this->ci->pg_seo->set_settings($seo_link['controller'], $seo_link['module'], $seo_link['method'], $link_data);
        }
    }
    
    public function install_seo_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }

        foreach($this->modules_data['seo_links'] as $seo_link) {
            $settings = $this->ci->pg_seo->get_settings($seo_link['controller'], $seo_link['module'], $seo_link['method'], $langs_ids);
            $seo_langs["seo_tags_{$seo_link['method']}_title"] = $settings["title"];
            $seo_langs["seo_tags_{$seo_link['method']}_keyword"] = $settings["keyword"];
            $seo_langs["seo_tags_{$seo_link['method']}_description"] = $settings["description"];
            $seo_langs["seo_tags_{$seo_link['method']}_header"] = $settings["header"];
            $seo_langs["seo_tags_{$seo_link['method']}_og_title"] = $settings["og_title"];
            $seo_langs["seo_tags_{$seo_link['method']}_og_type"] = $settings["og_type"];
            $seo_langs["seo_tags_{$seo_link['method']}_og_description"] = $settings["og_description"];
         }
        return array("seo" => $seo_langs);
    }
    
    public function deinstall_seo()
    {
        $this->ci->pg_seo->delete_seo_module("users_services");
        $this->ci->load->model('Seo_model');
        $this->ci->Seo_model->rewrite_route_php_file();
    }
    
    /**
     * Install data of users services module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        
    }

    /**
     * Import languages of users services module
     *
     * @param array $langs_ids languages identfiers
     *
     * @return boolean
     */
    public function _arbitrary_lang_install($langs_ids = null)
    {

    }

    /**
     * Export languages of users services module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        
    }

    /**
     * Uninstall data of users services module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {

    }

    /**
     * Return content of featured users block
     *
     * Сallback method of dynamic blocks module
     *
     * @param array   $params block parameters
     * @param string  $view   view mode
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_get_featured_users($params, $view = 'default', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count'])) {
            $params['count'] = 8;
        }
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_featured_users', 'user', 'users');
    }

    /**
     * Return content of featured companies block
     *
     * Callback method of dynamic blocks module
     *
     * @param array   $params block parameters
     * @param string  $view   view mode
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_get_featured_companies($params, $view = 'default', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count'])) {
            $params['count'] = 8;
        }
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_featured_companies', 'user', 'users');
    }

    /**
     * Return content of featured agents block (callback method of dynamic blocks module)
     *
     * @param array   $params block parameters
     * @param string  $view   view mode
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_get_featured_agents($params, $view = 'default', $width = 100)
    {
        $sep_pos = strrpos($view, '_');
        $size = substr($view, $sep_pos + 1);
        $view = substr($view, 0, $sep_pos);
        if (!isset($params['count'])) {
            $params['count'] = 8;
        }
        $this->ci->view->assign('block_count', $params['count']);
        $this->ci->view->assign('block_size', $size);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_featured_agents', 'user', 'users');
    }

}
